package com.qmth.sop.business.entity;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.sop.common.base.BaseEntity;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.hibernate.validator.constraints.Range;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;

/**
 * <p>
 * 考勤组表
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
@ApiModel(value="SysDingGroup对象", description="考勤组表")
public class SysDingGroup extends BaseEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "服务单元id")
    @JsonSerialize(using = ToStringSerializer.class)
    @NotNull(message = "服务单元id不能为空")
    @Range(min = 1L, message = "服务单元id不能为空")
    private Long serviceId;

    @ApiModelProperty(value = "名称")
    @NotBlank(message = "名称不能为空")
    private String name;

    @ApiModelProperty(value = "供应商id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long supplierId;

    @ApiModelProperty(value = "签到时段")
    private String signInTime;

    @ApiModelProperty(value = "签退时段")
    private String signOutTime;

    @ApiModelProperty(value = "是否开启人脸，0：不开启，1：开启")
    @NotNull(message = "是否开启人脸不能为空")
    private Boolean faceOpen;

    @ApiModelProperty(value = "允许补卡次数")
    private Integer reissueCardCount;

    @ApiModelProperty(value = "是否启用，0：停用，1：启用")
    private Boolean enable;

    public Long getServiceId() {
        return serviceId;
    }

    public void setServiceId(Long serviceId) {
        this.serviceId = serviceId;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Long getSupplierId() {
        return supplierId;
    }

    public void setSupplierId(Long supplierId) {
        this.supplierId = supplierId;
    }

    public String getSignInTime() {
        return signInTime;
    }

    public void setSignInTime(String signInTime) {
        this.signInTime = signInTime;
    }

    public String getSignOutTime() {
        return signOutTime;
    }

    public void setSignOutTime(String signOutTime) {
        this.signOutTime = signOutTime;
    }

    public Boolean getFaceOpen() {
        return faceOpen;
    }

    public void setFaceOpen(Boolean faceOpen) {
        this.faceOpen = faceOpen;
    }

    public Integer getReissueCardCount() {
        return reissueCardCount;
    }

    public void setReissueCardCount(Integer reissueCardCount) {
        this.reissueCardCount = reissueCardCount;
    }

    public Boolean getEnable() {
        return enable;
    }

    public void setEnable(Boolean enable) {
        this.enable = enable;
    }
}
