package com.qmth.sop.business.mapper;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.sop.business.bean.dto.DataPermissionDto;
import com.qmth.sop.business.bean.result.ServiceUnitResult;
import com.qmth.sop.business.bean.result.SysLogResult;
import com.qmth.sop.business.entity.TBService;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.qmth.sop.common.enums.ProductTypeEnum;
import com.qmth.sop.common.enums.ServiceStatusEnum;
import org.apache.ibatis.annotations.Param;

import javax.xml.ws.Service;
import java.util.List;

/**
 * <p>
 * 服务单元表 Mapper 接口
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
public interface TBServiceMapper extends BaseMapper<TBService> {
    /**
     * 服务单元分页查询
     *
     * @param iPage           分页参数
     * @param type            业务类型
     * @param serviceLeadId   负责人
     * @param status          当前状态
     * @param createStartTime 创建开始时间
     * @param creatEndTime    创建结束时间
     * @param serviceUnitName 服务单元名称
     * @return 分页查询结果
     */
    IPage<ServiceUnitResult> findServiceUnitPage(@Param("iPage") Page<SysLogResult> iPage,
                                                 @Param("type") ProductTypeEnum type,
                                                 @Param("serviceLeadId") Long serviceLeadId,
                                                 @Param("status") ServiceStatusEnum status,
                                                 @Param("createStartTime") Long createStartTime,
                                                 @Param("createEndTime") Long creatEndTime,
                                                 @Param("serviceUnitName") String serviceUnitName);

    /**
     * 查询服务单元
     *
     * @param type       产品类型
     * @param statusList 状态集合
     * @param name       服务单元名称
     * @param dpr        数据权限
     * @return 服务单元
     */
    List<TBService> queryServiceUnit(@Param("type") ProductTypeEnum type,
                                     @Param("statusList") List<ServiceStatusEnum> statusList,
                                     @Param("name") String name,
                                     @Param("dpr") DataPermissionDto dpr);

}
