package com.qmth.sop.business.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.sop.business.bean.dto.UserArchivesImportDto;
import com.qmth.sop.business.bean.result.ArchivesSourceResult;
import com.qmth.sop.business.bean.result.UserArchivesResult;
import com.qmth.sop.business.entity.TBUserArchives;
import com.qmth.sop.common.enums.RoleTypeEnum;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * <p>
 * 人员档案表 Mapper 接口
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
public interface TBUserArchivesMapper extends BaseMapper<TBUserArchives> {
    IPage<UserArchivesResult> findUserArchivesPage(@Param("iPage") Page<UserArchivesResult> iPage,
                                                   @Param("city") String city,
                                                   @Param("supplierId") Long supplierId,
                                                   @Param("archivesName") String archivesName,
                                                   @Param("roleType") RoleTypeEnum roleType,
                                                   @Param("status") Boolean status,
                                                   @Param("archivesTimeStart") Long archivesTimeStart,
                                                   @Param("archivesTimeEnd") Long archivesTimeEnd,
                                                   @Param("remainValidTime") Long remainValidTime,
                                                   @Param("currentTime") Long currentTime);

    List<UserArchivesImportDto> findExportData(@Param("city") String city,
                                               @Param("supplierId") Long supplierId,
                                               @Param("archivesName") String archivesName,
                                               @Param("roleType") RoleTypeEnum roleType,
                                               @Param("status") Boolean status,
                                               @Param("archivesTimeStart") Long archivesTimeStart,
                                               @Param("archivesTimeEnd") Long archivesTimeEnd,
                                               @Param("remainValidTime") Long remainValidTime,
                                               @Param("currentTime") Long currentTime);

    /**
     * 根据用户id或档案id查询档案详情
     *
     * @param archivesId 档案id
     * @param userId     用户id
     * @return 档案详情
     */
    List<UserArchivesResult> findUserArchivesById(@Param("archivesId") Long archivesId, @Param("userId") Long userId);

    /**
     * 根据角色类型查询该角色的档案资源
     *
     * @param roleType 角色类型
     * @return 档案资源
     */
    List<ArchivesSourceResult> findArchivesSourceByType(@Param("roleType") RoleTypeEnum roleType);

    /**
     * 查询已用档案
     *
     * @param roleType 角色类型
     * @param crmNo    派单号
     * @return 派单已分配档案
     */
    List<ArchivesSourceResult> findArchivesUsed(@Param("roleType") RoleTypeEnum roleType,
                                                @Param("crmNo") String crmNo);


    List<UserArchivesResult> findUserArchivesList(@Param("city") String city,
                                                  @Param("supplierId") Long supplierId,
                                                  @Param("archivesName") String archivesName,
                                                  @Param("roleType") RoleTypeEnum roleType,
                                                  @Param("status") Boolean status,
                                                  @Param("archivesTimeStart") Long archivesTimeStart,
                                                  @Param("archivesTimeEnd") Long archivesTimeEnd,
                                                  @Param("remainValidTime") Long remainValidTime,
                                                  @Param("currentTime") Long currentTime);

    /**
     * 档案小计
     *
     * @return 档案详细信息
     */
    List<UserArchivesResult> findSubTotal();
}
