package com.qmth.sop.business.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.sop.business.bean.dto.RoleDto;
import com.qmth.sop.business.bean.params.SysRolePrivilegeParam;
import com.qmth.sop.business.bean.result.RoleResult;
import com.qmth.sop.business.entity.SysRole;
import com.qmth.sop.common.enums.RoleTypeEnum;

import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 角色表 服务类
 * </p>
 *
 * @author wangliang
 * @since 2023-07-17
 */
public interface SysRoleService extends IService<SysRole> {

    /**
     * 根据userId查询角色
     *
     * @param userId
     * @return
     */
    List<RoleDto> listRolesByUserId(Long userId);

    /**
     * 查询角色列表
     *
     * @param iPage
     * @param name
     * @param enable
     * @return
     */
    IPage<RoleDto> query(IPage<Map> iPage, String name, Boolean enable);

    /**
     * 新增/修改角色
     *
     * @param sysRolePrivilegeParams
     * @return
     */
    Boolean saveRole(SysRolePrivilegeParam sysRolePrivilegeParams);

    /**
     * 根据id删除角色
     *
     * @param roleId
     * @return
     * @throws NoSuchAlgorithmException
     */
    Boolean deleteRole(Long roleId) throws NoSuchAlgorithmException;

    /**
     * 根据外包认证角色类型获取角色信息
     *
     * @param roleTypeEnum 外包角色枚举
     * @return 角色
     */
    RoleResult findRoleInfoByArchivesType(RoleTypeEnum roleTypeEnum);

    /**
     * 查询用户在某派单中执行的工程师
     *
     * @param userId 用户id
     * @param crmNo  派单号
     * @return 角色
     */
    RoleTypeEnum findRoleTypeByUserCrm(Long userId, String crmNo);
}
