USE sop_db;

-- ----------------------------
-- Table structure for basic_attachment
-- ----------------------------
DROP TABLE IF EXISTS `basic_attachment`;
CREATE TABLE `basic_attachment` (
                                    `id` bigint(20) NOT NULL COMMENT '主键',
                                    `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                    `name` varchar(100) NOT NULL COMMENT '文件名',
                                    `type` varchar(10) NOT NULL COMMENT '文件类型',
                                    `size` decimal(8,2) NOT NULL COMMENT '单位（KB）',
                                    `md5` varchar(32) NOT NULL COMMENT '文件md5值',
                                    `path` varchar(2000) NOT NULL COMMENT '存储路径',
                                    `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                    `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                    `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                    `obj_id` bigint(20) DEFAULT NULL COMMENT '关联业务id',
                                    `remark` varchar(500) DEFAULT NULL,
                                    PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='所有附件记录表';

-- ----------------------------
-- Table structure for basic_school
-- ----------------------------
DROP TABLE IF EXISTS `basic_school`;
CREATE TABLE `basic_school` (
                                `id` bigint(20) NOT NULL COMMENT '主键',
                                `code` varchar(20) NOT NULL COMMENT '学校编码',
                                `domain_name` varchar(20) DEFAULT NULL COMMENT '域名',
                                `name` varchar(100) DEFAULT NULL COMMENT '学校名称',
                                `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                                `access_key` varchar(50) DEFAULT NULL COMMENT '访问key',
                                `access_secret` varchar(50) DEFAULT NULL COMMENT '访问秘钥',
                                `remark` text COMMENT '备注',
                                `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                `logo` mediumtext COMMENT '学校logo',
                                PRIMARY KEY (`id`) USING BTREE,
                                UNIQUE KEY `code` (`code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校表';

-- ----------------------------
-- Table structure for basic_sms_message
-- ----------------------------
DROP TABLE IF EXISTS `basic_sms_message`;
CREATE TABLE `basic_sms_message` (
                                     `id` bigint(20) NOT NULL COMMENT '主键',
                                     `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                     `user_id` bigint(20) NOT NULL COMMENT '消息接收人用户',
                                     `user_name` varchar(100) DEFAULT NULL COMMENT '用户名称',
                                     `mobile_number` varchar(11) DEFAULT NULL COMMENT '电话号码',
                                     `message_type` varchar(45) DEFAULT NULL COMMENT '消息类型',
                                     `business_operate` varchar(45) NOT NULL COMMENT '业务操作',
                                     `template_code` varchar(45) NOT NULL COMMENT '消息模板代码',
                                     `variable_params` text COMMENT '变量参数内容',
                                     `template_content` mediumtext COMMENT '模板内容',
                                     `send_status` varchar(45) DEFAULT NULL COMMENT '消息发送状态',
                                     `send_result` varchar(200) DEFAULT NULL COMMENT '消息发送结果',
                                     `remark` text COMMENT '备注',
                                     `resend_count` int(11) DEFAULT NULL COMMENT '重试次数',
                                     `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                     `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                     `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                     `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                     PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='短信通知表';

-- ----------------------------
-- Table structure for basic_verify_code
-- ----------------------------
DROP TABLE IF EXISTS `basic_verify_code`;
CREATE TABLE `basic_verify_code` (
                                     `id` bigint(20) NOT NULL COMMENT '主键',
                                     `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                     `user_id` bigint(20) NOT NULL COMMENT '用户ID',
                                     `mobile_number` varchar(11) NOT NULL COMMENT '手机号',
                                     `expire_time` bigint(20) NOT NULL COMMENT '到期时间',
                                     `valid_period` int(11) NOT NULL COMMENT '单位（分）',
                                     `verify_code` varchar(4) NOT NULL COMMENT '4位数字',
                                     `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                     `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                     `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                     `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                     PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='短信验证码记录表';

-- ----------------------------
-- Table structure for qrtz_blob_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_blob_triggers`;
CREATE TABLE `qrtz_blob_triggers` (
                                      `SCHED_NAME` varchar(120) NOT NULL,
                                      `TRIGGER_NAME` varchar(200) NOT NULL,
                                      `TRIGGER_GROUP` varchar(200) NOT NULL,
                                      `BLOB_DATA` blob,
                                      PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                      KEY `SCHED_NAME` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_calendars
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_calendars`;
CREATE TABLE `qrtz_calendars` (
                                  `SCHED_NAME` varchar(120) NOT NULL,
                                  `CALENDAR_NAME` varchar(200) NOT NULL,
                                  `CALENDAR` blob NOT NULL,
                                  PRIMARY KEY (`SCHED_NAME`,`CALENDAR_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_cron_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_cron_triggers`;
CREATE TABLE `qrtz_cron_triggers` (
                                      `SCHED_NAME` varchar(120) NOT NULL,
                                      `TRIGGER_NAME` varchar(200) NOT NULL,
                                      `TRIGGER_GROUP` varchar(200) NOT NULL,
                                      `CRON_EXPRESSION` varchar(120) NOT NULL,
                                      `TIME_ZONE_ID` varchar(80) DEFAULT NULL,
                                      PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_fired_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_fired_triggers`;
CREATE TABLE `qrtz_fired_triggers` (
                                       `SCHED_NAME` varchar(120) NOT NULL,
                                       `ENTRY_ID` varchar(95) NOT NULL,
                                       `TRIGGER_NAME` varchar(200) NOT NULL,
                                       `TRIGGER_GROUP` varchar(200) NOT NULL,
                                       `INSTANCE_NAME` varchar(200) NOT NULL,
                                       `FIRED_TIME` bigint(20) NOT NULL,
                                       `SCHED_TIME` bigint(20) NOT NULL,
                                       `PRIORITY` int(11) NOT NULL,
                                       `STATE` varchar(16) NOT NULL,
                                       `JOB_NAME` varchar(200) DEFAULT NULL,
                                       `JOB_GROUP` varchar(200) DEFAULT NULL,
                                       `IS_NONCONCURRENT` varchar(1) DEFAULT NULL,
                                       `REQUESTS_RECOVERY` varchar(1) DEFAULT NULL,
                                       PRIMARY KEY (`SCHED_NAME`,`ENTRY_ID`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_TRIG_INST_NAME` (`SCHED_NAME`,`INSTANCE_NAME`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_INST_JOB_REQ_RCVRY` (`SCHED_NAME`,`INSTANCE_NAME`,`REQUESTS_RECOVERY`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_J_G` (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_JG` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_T_G` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_TG` (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_job_details
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_job_details`;
CREATE TABLE `qrtz_job_details` (
                                    `SCHED_NAME` varchar(120) NOT NULL,
                                    `JOB_NAME` varchar(200) NOT NULL,
                                    `JOB_GROUP` varchar(200) NOT NULL,
                                    `DESCRIPTION` varchar(250) DEFAULT NULL,
                                    `JOB_CLASS_NAME` varchar(250) NOT NULL,
                                    `IS_DURABLE` varchar(1) NOT NULL,
                                    `IS_NONCONCURRENT` varchar(1) NOT NULL,
                                    `IS_UPDATE_DATA` varchar(1) NOT NULL,
                                    `REQUESTS_RECOVERY` varchar(1) NOT NULL,
                                    `JOB_DATA` blob,
                                    PRIMARY KEY (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
                                    KEY `IDX_QRTZ_J_REQ_RECOVERY` (`SCHED_NAME`,`REQUESTS_RECOVERY`) USING BTREE,
                                    KEY `IDX_QRTZ_J_GRP` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_locks
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_locks`;
CREATE TABLE `qrtz_locks` (
                              `SCHED_NAME` varchar(120) NOT NULL,
                              `LOCK_NAME` varchar(40) NOT NULL,
                              PRIMARY KEY (`SCHED_NAME`,`LOCK_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_paused_trigger_grps
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_paused_trigger_grps`;
CREATE TABLE `qrtz_paused_trigger_grps` (
                                            `SCHED_NAME` varchar(120) NOT NULL,
                                            `TRIGGER_GROUP` varchar(200) NOT NULL,
                                            PRIMARY KEY (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_scheduler_state
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_scheduler_state`;
CREATE TABLE `qrtz_scheduler_state` (
                                        `SCHED_NAME` varchar(120) NOT NULL,
                                        `INSTANCE_NAME` varchar(200) NOT NULL,
                                        `LAST_CHECKIN_TIME` bigint(20) NOT NULL,
                                        `CHECKIN_INTERVAL` bigint(20) NOT NULL,
                                        PRIMARY KEY (`SCHED_NAME`,`INSTANCE_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_simple_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_simple_triggers`;
CREATE TABLE `qrtz_simple_triggers` (
                                        `SCHED_NAME` varchar(120) NOT NULL,
                                        `TRIGGER_NAME` varchar(200) NOT NULL,
                                        `TRIGGER_GROUP` varchar(200) NOT NULL,
                                        `REPEAT_COUNT` bigint(20) NOT NULL,
                                        `REPEAT_INTERVAL` bigint(20) NOT NULL,
                                        `TIMES_TRIGGERED` bigint(20) NOT NULL,
                                        PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_simprop_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_simprop_triggers`;
CREATE TABLE `qrtz_simprop_triggers` (
                                         `SCHED_NAME` varchar(120) NOT NULL,
                                         `TRIGGER_NAME` varchar(200) NOT NULL,
                                         `TRIGGER_GROUP` varchar(200) NOT NULL,
                                         `STR_PROP_1` varchar(512) DEFAULT NULL,
                                         `STR_PROP_2` varchar(512) DEFAULT NULL,
                                         `STR_PROP_3` varchar(512) DEFAULT NULL,
                                         `INT_PROP_1` int(11) DEFAULT NULL,
                                         `INT_PROP_2` int(11) DEFAULT NULL,
                                         `LONG_PROP_1` bigint(20) DEFAULT NULL,
                                         `LONG_PROP_2` bigint(20) DEFAULT NULL,
                                         `DEC_PROP_1` decimal(13,4) DEFAULT NULL,
                                         `DEC_PROP_2` decimal(13,4) DEFAULT NULL,
                                         `BOOL_PROP_1` varchar(1) DEFAULT NULL,
                                         `BOOL_PROP_2` varchar(1) DEFAULT NULL,
                                         PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_triggers`;
CREATE TABLE `qrtz_triggers` (
                                 `SCHED_NAME` varchar(120) NOT NULL,
                                 `TRIGGER_NAME` varchar(200) NOT NULL,
                                 `TRIGGER_GROUP` varchar(200) NOT NULL,
                                 `JOB_NAME` varchar(200) NOT NULL,
                                 `JOB_GROUP` varchar(200) NOT NULL,
                                 `DESCRIPTION` varchar(250) DEFAULT NULL,
                                 `NEXT_FIRE_TIME` bigint(20) DEFAULT NULL,
                                 `PREV_FIRE_TIME` bigint(20) DEFAULT NULL,
                                 `PRIORITY` int(11) DEFAULT NULL,
                                 `TRIGGER_STATE` varchar(16) NOT NULL,
                                 `TRIGGER_TYPE` varchar(8) NOT NULL,
                                 `START_TIME` bigint(20) NOT NULL,
                                 `END_TIME` bigint(20) DEFAULT NULL,
                                 `CALENDAR_NAME` varchar(200) DEFAULT NULL,
                                 `MISFIRE_INSTR` smallint(6) DEFAULT NULL,
                                 `JOB_DATA` blob,
                                 PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_J` (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_JG` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_C` (`SCHED_NAME`,`CALENDAR_NAME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_G` (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_STATE` (`SCHED_NAME`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_N_STATE` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_N_G_STATE` (`SCHED_NAME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NEXT_FIRE_TIME` (`SCHED_NAME`,`NEXT_FIRE_TIME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_ST` (`SCHED_NAME`,`TRIGGER_STATE`,`NEXT_FIRE_TIME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_MISFIRE` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_ST_MISFIRE` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_ST_MISFIRE_GRP` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for sequence
-- ----------------------------
DROP TABLE IF EXISTS `sequence`;
CREATE TABLE `sequence` (
                            `name` varchar(50) NOT NULL COMMENT '名称',
                            `current_value` bigint(20) NOT NULL DEFAULT '0' COMMENT '当前值',
                            `increment` int(11) NOT NULL DEFAULT '1' COMMENT '增长值',
                            PRIMARY KEY (`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='序列表';

-- ----------------------------
-- Table structure for sys_config
-- ----------------------------
DROP TABLE IF EXISTS `sys_config`;
CREATE TABLE `sys_config` (
                              `id` bigint(20) NOT NULL COMMENT '主键',
                              `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                              `config_key` varchar(100) NOT NULL COMMENT '参数键名',
                              `config_name` varchar(100) NOT NULL COMMENT '参数名称',
                              `config_value` varchar(100) DEFAULT NULL COMMENT '参数键值',
                              `remark` varchar(100) DEFAULT NULL COMMENT '备注',
                              `enable` tinyint(4) DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                              `sort` int(11) DEFAULT '1' COMMENT '排序',
                              `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                              `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                              `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                              `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                              PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参数表';

-- ----------------------------
-- Table structure for sys_custom
-- ----------------------------
DROP TABLE IF EXISTS `sys_custom`;
CREATE TABLE `sys_custom` (
                              `id` bigint(20) NOT NULL COMMENT '主键',
                              `name` varchar(100) NOT NULL COMMENT '名称',
                              `code` varchar(100) DEFAULT NULL COMMENT '编码',
                              `type` varchar(20) DEFAULT NULL COMMENT '类型，OFFICE：教务处，CLOUD_MARK：研究生',
                              `province` varchar(100) NOT NULL COMMENT '省份',
                              `city` varchar(100) NOT NULL COMMENT '城市',
                              `area` varchar(100) DEFAULT NULL COMMENT '县区',
                              `address` varchar(150) DEFAULT NULL COMMENT '地址',
                              `level_id` bigint(20) DEFAULT NULL COMMENT '档位id',
                              `peoper_day` int(11) DEFAULT NULL COMMENT '标准人天',
                              `sync` tinyint(4) NOT NULL DEFAULT '0' COMMENT '是否crm同步，0：否，1：是',
                              `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                              `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                              `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                              `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                              `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                              PRIMARY KEY (`id`),
                              UNIQUE KEY `code` (`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='客户表';

-- ----------------------------
-- Table structure for sys_custom_addr
-- ----------------------------
DROP TABLE IF EXISTS `sys_custom_addr`;
CREATE TABLE `sys_custom_addr` (
                                   `id` bigint(20) NOT NULL COMMENT '主键',
                                   `custom_id` bigint(20) NOT NULL COMMENT '客户ID',
                                   `province` varchar(100) NOT NULL COMMENT '省份',
                                   `city` varchar(100) NOT NULL COMMENT '城市',
                                   `area` varchar(100) NOT NULL COMMENT '县区',
                                   `address` varchar(150) NOT NULL COMMENT '地址',
                                   `longitude` varchar(255) NOT NULL COMMENT '经度',
                                   `latitude` varchar(255) NOT NULL COMMENT '纬度',
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='客户地址表';

-- ----------------------------
-- Table structure for sys_device
-- ----------------------------
DROP TABLE IF EXISTS `sys_device`;
CREATE TABLE `sys_device` (
                              `id` bigint(20) NOT NULL COMMENT '主键',
                              `device_code` varchar(100) NOT NULL COMMENT '设备编号',
                              `serial_no` varchar(100) NOT NULL COMMENT '序列号',
                              `brand` varchar(100) NOT NULL COMMENT '品牌',
                              `model` varchar(255) NOT NULL COMMENT '型号',
                              `buy_time` bigint(20) NULL COMMENT '购买时间',
                              `supplier_id` bigint(20) DEFAULT NULL COMMENT '供应商id',
                              `status` varchar(20) DEFAULT NULL COMMENT '状态，NORMAL：正常，BREAK_DOWN：故障',
                              `location` varchar(100) DEFAULT NULL COMMENT '当前所在地',
                              `location_arr` varchar(100) DEFAULT NULL COMMENT '当前所在地（回显）',
                              `scan_count` bigint(20) DEFAULT NULL COMMENT '总扫描量',
                              `bound` varchar(20) DEFAULT NULL COMMENT '出库/入库',
                              `sync` tinyint(4) NOT NULL DEFAULT '0' COMMENT '是否crm同步，0：否，1：是',
                              `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                              `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                              `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                              `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                              `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                              PRIMARY KEY (`id`),
                              UNIQUE KEY `sys_device_un_device_code` (`device_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备配置表';

-- ----------------------------
-- Table structure for sys_device_brand
-- ----------------------------
DROP TABLE IF EXISTS `sys_device_brand`;
CREATE TABLE `sys_device_brand` (
                                    `id` bigint(20) NOT NULL COMMENT '主键',
                                    `brand` varchar(100) NOT NULL COMMENT '品牌',
                                    `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                    `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                    `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                    PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备品牌表';

-- ----------------------------
-- Table structure for sys_device_model
-- ----------------------------
DROP TABLE IF EXISTS `sys_device_model`;
CREATE TABLE `sys_device_model` (
                                    `id` bigint(20) NOT NULL COMMENT '主键',
                                    `brand_id` bigint(20) NOT NULL COMMENT '品牌ID',
                                    `model` varchar(255) NOT NULL COMMENT '型号',
                                    `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                    `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                    `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                    PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备品牌型号表';

-- ----------------------------
-- Table structure for sys_ding_date
-- ----------------------------
DROP TABLE IF EXISTS `sys_ding_date`;
CREATE TABLE `sys_ding_date` (
                                 `id` bigint(20) NOT NULL COMMENT '主键',
                                 `year` bigint(20) NOT NULL COMMENT '年',
                                 `month` bigint(20) NOT NULL COMMENT '月',
                                 `day` bigint(20) NOT NULL COMMENT '日',
                                 `type` varchar(50) NOT NULL COMMENT '日期类型：WEEKDAY("工作日"),WEEKEND("周末"),LEGAL_HOLIDAYS("法定节假日")',
                                 PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤特殊日期';

-- ----------------------------
-- Table structure for sys_ding_group
-- ----------------------------
DROP TABLE IF EXISTS `sys_ding_group`;
CREATE TABLE `sys_ding_group` (
                                  `id` bigint(20) NOT NULL COMMENT '主键',
                                  `service_id` bigint(20) NOT NULL COMMENT '服务单元id',
                                  `name` varchar(100) NOT NULL COMMENT '名称',
                                  `supplier_id` bigint(20) NOT NULL COMMENT '供应商id',
                                  `sign_in_time` varchar(50) DEFAULT NULL COMMENT '签到时段',
                                  `sign_out_time` varchar(50) DEFAULT NULL COMMENT '签退时段',
                                  `face_open` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否开启人脸，0：不开启，1：开启',
                                  `reissue_card_count` int(11) DEFAULT NULL COMMENT '允许补卡次数',
                                  `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                  `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                  `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                  `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤组表';

-- ----------------------------
-- Table structure for sys_ding_obj
-- ----------------------------
DROP TABLE IF EXISTS `sys_ding_obj`;
CREATE TABLE `sys_ding_obj` (
                                `id` bigint(20) NOT NULL COMMENT '主键',
                                `ding_group_id` bigint(20) NOT NULL COMMENT '考勤组id',
                                `role_id` bigint(20) NOT NULL COMMENT '角色id',
                                `type` varchar(20) NOT NULL COMMENT '类型，DING：考勤，APPROVE：审核',
                                PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤组对象表';

-- ----------------------------
-- Table structure for sys_job
-- ----------------------------
DROP TABLE IF EXISTS `sys_job`;
CREATE TABLE `sys_job` (
                           `id` bigint(20) NOT NULL COMMENT '主键',
                           `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                           `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                           `service_id` bigint(20) DEFAULT NULL COMMENT '服务单元id',
                           `user_id` bigint(20) NOT NULL COMMENT '责任人id',
                           `job_time` bigint(20) NOT NULL COMMENT '任务时间',
                           `type` varchar(20) NOT NULL COMMENT '类型，BEFORE：任务前提醒，AFTER：任务后提醒',
                           `unit` varchar(20) NOT NULL COMMENT '单位，HOUR：小时，MINUTE：分钟',
                           `status` varchar(20) NOT NULL COMMENT '状态，NOT_EXEC：未执行，EXEC：已执行',
                           `exec_time` bigint(20) NOT NULL COMMENT '执行时间',
                           `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                           PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='系统任务表';

-- ----------------------------
-- Table structure for sys_level
-- ----------------------------
DROP TABLE IF EXISTS `sys_level`;
CREATE TABLE `sys_level` (
                             `id` bigint(20) NOT NULL COMMENT '主键',
                             `level` varchar(50) NOT NULL COMMENT '档位',
                             `devices` bigint(20) DEFAULT NULL COMMENT '设备数量',
                             `type` varchar(20) NOT NULL COMMENT '类型，OFFICE：教务处，CLOUD_MARK：研究生',
                             `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                             `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                             `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                             PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='档位表';

-- ----------------------------
-- Table structure for sys_level_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_level_role`;
CREATE TABLE `sys_level_role` (
                                  `id` bigint(20) NOT NULL COMMENT '主键',
                                  `level_id` bigint(20) NOT NULL COMMENT '服务档位id',
                                  `role_id` bigint(20) NOT NULL COMMENT '角色id',
                                  `quota` bigint(20) NOT NULL COMMENT '配额',
                                  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务档位角色配置表';

-- ----------------------------
-- Table structure for sys_log
-- ----------------------------
DROP TABLE IF EXISTS `sys_log`;
CREATE TABLE `sys_log` (
                           `id` bigint(20) NOT NULL COMMENT '主键',
                           `user_id` bigint(20) NOT NULL COMMENT '用户id',
                           `user_name` varchar(100) DEFAULT NULL COMMENT '用户名称',
                           `type` varchar(20) NOT NULL COMMENT '类别，LOGIN：登录，LOGOUT：退出以及数据库DDL和DML操作',
                           `obj_name` varchar(1000) DEFAULT NULL COMMENT '实体名称',
                           `obj_id` bigint(20) DEFAULT NULL COMMENT '实体id',
                           `content` varchar(200) DEFAULT NULL COMMENT '内容',
                           `url` varchar(64) DEFAULT NULL COMMENT '请求的url',
                           `args` mediumtext COMMENT '请求的参数',
                           `update_before_obj` mediumtext COMMENT '更改前内容',
                           `update_after_obj` mediumtext COMMENT '更改后内容',
                           `create_time` bigint(20) NOT NULL COMMENT '创建时间',
                           `run_time` bigint(20) DEFAULT NULL COMMENT '请求的响应时间',
                           `run_status` varchar(64) DEFAULT NULL COMMENT '请求的结果',
                           PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='系统日志';

-- ----------------------------
-- Table structure for sys_message
-- ----------------------------
DROP TABLE IF EXISTS `sys_message`;
CREATE TABLE `sys_message` (
                               `id` bigint(20) NOT NULL COMMENT '主键',
                               `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                               `code` varchar(100) DEFAULT NULL COMMENT '编号',
                               `type` varchar(20) NOT NULL COMMENT '消息类型，BEFORE：提前提醒，AFTER：延期提醒，OFFICE_SOP：教务处SOP，CLOUD_MARK_SOP：研究生SOP，QUALITY：质量问题提醒，EXCEPTION_APPROVE：异常审核提醒，VIOLATION：违规提醒，SYSTEM_PLAN_CHANGE：系统计划变更提醒，SYSTEM：系统公告',
                               `obj_type` varchar(100) DEFAULT NULL COMMENT '实体业务类型',
                               `notice_id` bigint(20) DEFAULT NULL COMMENT '公告id',
                               `service_id` bigint(20) DEFAULT NULL COMMENT '服务单元id',
                               `supplier_id` bigint(20) DEFAULT NULL COMMENT '供应商ID',
                               `form_user_id` bigint(20) DEFAULT NULL COMMENT '发送人id',
                               `receive_user_id` bigint(20) NOT NULL COMMENT '接收人id',
                               `content` varchar(10000) NOT NULL COMMENT '消息内容',
                               `send_time` bigint(20) NOT NULL COMMENT '发送时间',
                               `receive_time` bigint(20) DEFAULT NULL COMMENT '接收时间',
                               `status` tinyint(4) NOT NULL DEFAULT '0' COMMENT '是否已阅，0：未阅，1：已阅',
                               PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='系统消息';

-- ----------------------------
-- Table structure for sys_notice
-- ----------------------------
DROP TABLE IF EXISTS `sys_notice`;
CREATE TABLE `sys_notice` (
                              `id` bigint(20) NOT NULL COMMENT '主键',
                              `service_id` bigint(20) DEFAULT NULL COMMENT '服务单元id',
                              `supplier_id` bigint(20) DEFAULT NULL COMMENT '供应商ID',
                              `title` varchar(200) NOT NULL COMMENT '标题',
                              `content` varchar(10000) NOT NULL COMMENT '内容',
                              `type` varchar(20) DEFAULT NULL COMMENT '类型，SYSTEM：公告，SUPPLIER：供应商，SERVICE：服务单元',
                              `status` varchar(20) DEFAULT NULL COMMENT '状态，PUBLISH：已发布，UN_PUBLISH：未发布',
                              `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                              `publish_time` bigint(20) DEFAULT NULL COMMENT '发布日期',
                              `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                              `create_time` bigint(20) DEFAULT NULL COMMENT '创建人时间',
                              `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                              `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                              PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='公告表';

-- ----------------------------
-- Table structure for sys_org
-- ----------------------------
DROP TABLE IF EXISTS `sys_org`;
CREATE TABLE `sys_org` (
                           `id` bigint(20) NOT NULL COMMENT '主键',
                           `type` varchar(50) DEFAULT NULL COMMENT '类型',
                           `code` varchar(20) DEFAULT NULL COMMENT '机构代码',
                           `name` varchar(100) NOT NULL COMMENT '机构名称',
                           `parent_id` bigint(20) DEFAULT NULL COMMENT '上级机构id',
                           `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                           `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                           `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                           `update_id` bigint(20) DEFAULT NULL COMMENT '修改人',
                           `update_time` bigint(20) DEFAULT NULL COMMENT '修改时间',
                           PRIMARY KEY (`id`) USING BTREE,
                           UNIQUE KEY `sys_org_UN_name` (`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='组织架构表';

-- ----------------------------
-- Table structure for sys_privilege
-- ----------------------------
DROP TABLE IF EXISTS `sys_privilege`;
CREATE TABLE `sys_privilege` (
                                 `id` bigint(20) NOT NULL COMMENT '主键',
                                 `name` varchar(100) NOT NULL COMMENT '菜单名称',
                                 `url` varchar(100) NOT NULL COMMENT 'URL地址',
                                 `type` varchar(50) NOT NULL COMMENT '类型，MENU：菜单，BUTTON：按钮，LINK：链接，URL：接口地址，LIST：列表，CONDITION：查询条件',
                                 `parent_id` bigint(20) DEFAULT NULL COMMENT '上级菜单',
                                 `sequence` int(11) NOT NULL COMMENT '序号',
                                 `property` varchar(30) DEFAULT NULL COMMENT '属性，NO_AUTH：无需鉴权，AUTH：鉴权，SYS：系统公用',
                                 `related` text COMMENT '关联属性，type为URL且property为AUTH才有',
                                 `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                 `default_auth` tinyint(4) DEFAULT '1' COMMENT '默认权限，0：否，1：是',
                                 `front_display` tinyint(1) DEFAULT '1' COMMENT '是否前端展示',
                                 PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='菜单权限表';

-- ----------------------------
-- Table structure for sys_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_role`;
CREATE TABLE `sys_role` (
                            `id` bigint(20) NOT NULL COMMENT '主键',
                            `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                            `name` varchar(50) NOT NULL COMMENT '角色名称',
                            `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                            `type` varchar(20) DEFAULT NULL COMMENT '角色类别，ADMIN：超级管理员，PMO：总负责人，BUSSINESS：业务线负责人，REGION_MANAGER：大区经理，REGION_COORDINATOR：区域协调人，EFFECT_ENGINEER：实施工程师，ASSISTANT_ENGINEER：助理工程师，QA：QA，CUSTOM：技术客服',
                            `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint(20) DEFAULT NULL COMMENT '修改人',
                            `update_time` bigint(20) DEFAULT NULL COMMENT '修改时间',
                            `default_role` tinyint(4) NOT NULL DEFAULT '0' COMMENT '是否系统内置角色，1：是，0：不是',
                            `interpret` varchar(1000) DEFAULT NULL COMMENT '角色说明',
                            PRIMARY KEY (`id`) USING BTREE,
                            UNIQUE KEY `sys_role_UN_type` (`name`,`type`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色表';

-- ----------------------------
-- Table structure for sys_role_privilege
-- ----------------------------
DROP TABLE IF EXISTS `sys_role_privilege`;
CREATE TABLE `sys_role_privilege` (
                                      `id` bigint(20) NOT NULL COMMENT '主键',
                                      `role_id` bigint(20) NOT NULL COMMENT '角色id',
                                      `privilege_id` bigint(20) NOT NULL COMMENT '菜单id',
                                      `enable` tinyint(4) DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                      PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色和菜单关联表';

-- ----------------------------
-- Table structure for sys_supplier
-- ----------------------------
DROP TABLE IF EXISTS `sys_supplier`;
CREATE TABLE `sys_supplier` (
                                `id` bigint(20) NOT NULL COMMENT '主键',
                                `name` varchar(100) NOT NULL COMMENT '名称',
                                `code` varchar(100) NOT NULL COMMENT '编码',
                                `type` varchar(20) NOT NULL COMMENT '类型，HUMAN：人力，DEVICE：设备',
                                `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                `remark` varchar(500) DEFAULT NULL COMMENT '备注',
                                `sync` tinyint(4) NOT NULL DEFAULT '0' COMMENT '是否crm同步，0：否，1：是',
                                `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='供应商表';

-- ----------------------------
-- Table structure for sys_user
-- ----------------------------
DROP TABLE IF EXISTS `sys_user`;
CREATE TABLE `sys_user` (
                            `id` bigint(20) NOT NULL COMMENT '主键',
                            `login_name` varchar(100) NOT NULL COMMENT '用户名',
                            `real_name` varchar(100) NOT NULL COMMENT '姓名',
                            `code` varchar(100) DEFAULT NULL COMMENT '工号',
                            `password` varchar(50) NOT NULL COMMENT '密码',
                            `mobile_number` varchar(30) DEFAULT NULL COMMENT '手机号',
                            `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                            `gender` varchar(20) DEFAULT NULL COMMENT '性别，MAN：男，WOMAN：女',
                            `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                            `pwd_count` tinyint(4) DEFAULT '0' COMMENT '密码修改次数，默认为0',
                            `pwd_update_time` bigint(20) DEFAULT NULL COMMENT '密码修改时间',
                            `remark` varchar(200) DEFAULT NULL COMMENT '备注',
                            `source` varchar(20) DEFAULT 'SYSTEM' COMMENT '来源，SYSTEM：系统自带，ARCHIVES：档案',
                            `rtzf_sync_status` varchar(20) DEFAULT NULL COMMENT '软通智服同步状态',
                            `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint(20) DEFAULT NULL COMMENT '修改人',
                            `update_time` bigint(20) DEFAULT NULL COMMENT '修改时间',
                            PRIMARY KEY (`id`) USING BTREE,
                            UNIQUE KEY `sys_user_UN_login_name` (`login_name`),
                            UNIQUE KEY `sys_user_UN_moblie_number` (`mobile_number`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户表';

-- ----------------------------
-- Table structure for sys_user_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_user_role`;
CREATE TABLE `sys_user_role` (
                                 `id` bigint(20) NOT NULL COMMENT '主键',
                                 `user_id` bigint(20) NOT NULL COMMENT '用户id',
                                 `role_id` bigint(20) NOT NULL COMMENT '角色id',
                                 `privilege_id` bigint(20) DEFAULT NULL COMMENT '权限id',
                                 `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                 PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户和角色关联表';

-- ----------------------------
-- Table structure for t_b_crm
-- ----------------------------
DROP TABLE IF EXISTS `t_b_crm`;
CREATE TABLE `t_b_crm` (
                           `id` bigint(20) NOT NULL COMMENT '主键',
                           `crm_no` varchar(100) NOT NULL COMMENT '项目单号',
                           `name` varchar(100) NOT NULL COMMENT '项目名称',
                           `begin_time` bigint(20) NOT NULL COMMENT '派单时间',
                           `service_id` bigint(20) DEFAULT NULL COMMENT '服务单元id',
                           `crm_user_id` bigint(20) NOT NULL COMMENT '派单人id',
                           `custom_id` bigint(20) NOT NULL COMMENT '客户id',
                           `exam_start_time` bigint(20) DEFAULT NULL COMMENT '考试开始时间',
                           `exam_end_time` bigint(20) DEFAULT NULL COMMENT '考试结束时间',
                           `product_id` bigint(20) NOT NULL COMMENT '产品id',
                           `lead_id` bigint(20) DEFAULT NULL COMMENT '大区经理id',
                           `region_id` bigint(20) DEFAULT NULL COMMENT '服务区域规划id',
                           `region_detail_id` bigint(20) DEFAULT NULL COMMENT '服务区域规划明细id',
                           `region_coordinator_id` bigint(20) DEFAULT NULL COMMENT '区域协调人id',
                           `sync` tinyint(4) NOT NULL DEFAULT '0' COMMENT '是否crm同步，false：否，true：是',
                           `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                           `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                           `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                           `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                           `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                           `status` varchar(20) NOT NULL COMMENT '派单状态(UN_PUBLISH("未发布"),PUBLISH("已发布"),FINISH("已完成");)',
                           `publish_user_id` bigint(20) DEFAULT NULL COMMENT '发布人id',
                           `publish_time` bigint(20) DEFAULT NULL COMMENT '发布时间',
                           PRIMARY KEY (`id`),
                           UNIQUE KEY `t_b_crm_un_crm_no` (`crm_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='派单信息表';

-- ----------------------------
-- Table structure for t_b_delay_warn
-- ----------------------------
DROP TABLE IF EXISTS `t_b_delay_warn`;
CREATE TABLE `t_b_delay_warn` (
                                  `id` bigint(20) NOT NULL COMMENT '主键',
                                  `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                  `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                                  `service_id` bigint(20) DEFAULT NULL COMMENT '服务单元id',
                                  `message_id` bigint(20) DEFAULT NULL COMMENT '消息id',
                                  `code` varchar(20) NOT NULL COMMENT '编号',
                                  `warn_time` bigint(20) DEFAULT NULL COMMENT '预警时间',
                                  `user_id` bigint(20) DEFAULT NULL COMMENT '负责人id',
                                  `type` varchar(20) NOT NULL COMMENT '类型，PLAN：关键信息及计划，TIME：处理时限',
                                  `field_obj` mediumtext COMMENT '预警字段',
                                  `status` varchar(20) NOT NULL COMMENT '状态，NOT_START：未跟进，FOLLOW：跟进，CLOSE：关闭，RESTART：重启',
                                  `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                  `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                  `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                  `close_time` bigint(20) DEFAULT NULL COMMENT '关闭时间',
                                  PRIMARY KEY (`id`),
                                  UNIQUE KEY `t_b_delay_warn_un_code` (`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='延期预警跟进表';

-- ----------------------------
-- Table structure for t_b_delay_warn_detail
-- ----------------------------
DROP TABLE IF EXISTS `t_b_delay_warn_detail`;
CREATE TABLE `t_b_delay_warn_detail` (
                                         `id` bigint(20) NOT NULL COMMENT '主键',
                                         `delay_warn_id` bigint(20) NOT NULL COMMENT '延期预警id',
                                         `remark` varchar(500) NOT NULL COMMENT '跟进说明',
                                         `attachment_ids` varchar(500) DEFAULT NULL COMMENT '附件ids',
                                         `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                         `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                         PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='延期预警明细表';

-- ----------------------------
-- Table structure for t_b_device_in_out
-- ----------------------------
DROP TABLE IF EXISTS `t_b_device_in_out`;
CREATE TABLE `t_b_device_in_out` (
                                     `id` bigint(20) NOT NULL COMMENT '主键',
                                     `serial_no` varchar(20) NOT NULL COMMENT '出入库流水号',
                                     `service_id` bigint(20) DEFAULT NULL COMMENT '服务单元id',
                                     `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                     `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                                     `in_out_time` bigint(20) NOT NULL COMMENT '出/入库时间',
                                     `user_id` bigint(20) NOT NULL COMMENT '登记人id',
                                     `custom_name` varchar(100) DEFAULT NULL COMMENT '客户名称',
                                     `type` varchar(20) DEFAULT NULL COMMENT '类型，IN：入库，OUT：出库',
                                     `device_no` varchar(50) DEFAULT NULL COMMENT '设备编号',
                                     `supplier_name` varchar(100) DEFAULT NULL COMMENT '供应商名称',
                                     `location` varchar(100) DEFAULT NULL COMMENT '当前所在地',
                                     `address` varchar(100) DEFAULT NULL COMMENT '发往地',
                                     `address_arr` varchar(100) DEFAULT NULL COMMENT '发往地（回显）',
                                     `status` varchar(20) NOT NULL COMMENT '状态，NORMAL：正常，BREAK_DOWN：故障',
                                     `usage_type` varchar(45) DEFAULT NULL COMMENT '用途类型（项目、其它）',
                                     `base_photo_path` varchar(500) DEFAULT NULL COMMENT '快递单拍照',
                                     `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                     `create_time` bigint(20) DEFAULT NULL COMMENT '创建人时间',
                                     PRIMARY KEY (`id`),
                                     UNIQUE KEY `t_b_device_in_out_un_serial_no` (`serial_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备出入库登记';

-- ----------------------------
-- Table structure for t_b_ding
-- ----------------------------
DROP TABLE IF EXISTS `t_b_ding`;
CREATE TABLE `t_b_ding` (
                            `id` bigint(20) NOT NULL COMMENT '主键',
                            `sop_no` varchar(100) NOT NULL COMMENT 'sop单号',
                            `ding_sop_no` varchar(100) NOT NULL COMMENT '实际打卡对应的sop单号',
                            `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                            `service_id` bigint(20) DEFAULT NULL COMMENT '服务单元id',
                            `user_id` bigint(20) NOT NULL COMMENT '用户id',
                            `user_archives_id` bigint(20) NOT NULL COMMENT '人员档案id',
                            `role_type` varchar(20) NOT NULL COMMENT '项目角色',
                            `flow_id` bigint(20) DEFAULT NULL COMMENT '流程id',
                            `task_id` bigint(20) DEFAULT NULL COMMENT '流程任务id',
                            `sign_year` varchar(20) NOT NULL COMMENT '签到年份',
                            `sign_date` varchar(20) NOT NULL COMMENT '签到日期',
                            `date_type` varchar(20) DEFAULT NULL COMMENT '日期类型：WEEKDAY("工作日"),WEEKEND("周末"),LEGAL_HOLIDAYS("法定节假日")',
                            `sign_in_time` bigint(20) DEFAULT NULL COMMENT '签到时间',
                            `sign_in_address` varchar(100) DEFAULT NULL COMMENT '签到地址',
                            `axis_in_x` varchar(20) DEFAULT NULL COMMENT '坐标轴X',
                            `axis_in_y` varchar(20) DEFAULT NULL COMMENT '坐标轴Y',
                            `face_in_pass` tinyint(4) DEFAULT NULL COMMENT '人脸识别是否通过，0：未通过，1：通过',
                            `sign_out_time` bigint(20) DEFAULT NULL COMMENT '签退时间',
                            `sign_out_address` varchar(100) DEFAULT NULL COMMENT '签退地址',
                            `axis_out_x` varchar(20) DEFAULT NULL COMMENT '坐标轴X',
                            `axis_out_Y` varchar(20) DEFAULT NULL COMMENT '坐标轴Y',
                            `face_out_pass` tinyint(4) DEFAULT NULL COMMENT '人脸识别是否通过，0：未通过，1：通过',
                            `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                            `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                            `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                            PRIMARY KEY (`id`),
                            UNIQUE KEY `t_b_ding_un` (`sop_no`,`user_id`,`sign_year`,`sign_date`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤打卡表';

-- ----------------------------
-- Table structure for t_b_ding_apply
-- ----------------------------
DROP TABLE IF EXISTS `t_b_ding_apply`;
CREATE TABLE `t_b_ding_apply` (
                                  `id` bigint(20) NOT NULL COMMENT '主键',
                                  `sop_no` varchar(100) NOT NULL COMMENT 'sop编号',
                                  `crm_no` varchar(100) NOT NULL COMMENT '派单编号',
                                  `ding_id` bigint(20) DEFAULT NULL COMMENT '考勤id',
                                  `type` varchar(20) NOT NULL COMMENT '类型，IN：签到，OUT：签退',
                                  `apply_time` bigint(20) NOT NULL COMMENT '补卡时间',
                                  `reason` varchar(500) NOT NULL COMMENT '补卡原因',
                                  `attachment_ids` varchar(500) DEFAULT NULL COMMENT '附件id数组',
                                  `attachment_paths` varchar(800) DEFAULT NULL COMMENT '附件地址',
                                  `ding_exception_no` varchar(100) NOT NULL COMMENT '考勤异常编号',
                                  `approve` varchar(20) DEFAULT NULL COMMENT '审批操作，PASS：通过，NO_PASS：不通过',
                                  `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                  PRIMARY KEY (`id`),
                                  UNIQUE KEY `t_b_ding_apply_UN_ding_exception_no` (`ding_exception_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤补卡申请表';

-- ----------------------------
-- Table structure for t_b_ding_history
-- ----------------------------
DROP TABLE IF EXISTS `t_b_ding_history`;
CREATE TABLE `t_b_ding_history` (
                                    `id` bigint(20) NOT NULL COMMENT '主键',
                                    `ding_id` bigint(20) DEFAULT NULL COMMENT '考勤id',
                                    `sop_no` varchar(20) DEFAULT NULL COMMENT 'sop编号',
                                    `sign_date` varchar(20) NOT NULL COMMENT '签到日期',
                                    `date_type` varchar(20) DEFAULT NULL COMMENT '日期类型：WEEKDAY("工作日"),WEEKEND("周末"),LEGAL_HOLIDAYS("法定节假日")',
                                    `sign_time` bigint(20) DEFAULT NULL COMMENT '签到时间',
                                    `axis_x` varchar(20) DEFAULT NULL COMMENT '坐标轴X',
                                    `axis_y` varchar(20) DEFAULT NULL COMMENT '坐标轴Y',
                                    `face_open` tinyint(4) DEFAULT NULL COMMENT '是否开启人脸识别',
                                    `face_pass` tinyint(4) DEFAULT NULL COMMENT '人脸识别是否通过，0：未通过，1：通过',
                                    `face_photo_path` varchar(500) DEFAULT NULL COMMENT '抓拍照片地址',
                                    `match_score` decimal(8,2) DEFAULT NULL COMMENT '对比分数',
                                    `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                    `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                    PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤打卡表';

-- ----------------------------
-- Table structure for t_b_ding_submit
-- ----------------------------
DROP TABLE IF EXISTS `t_b_ding_submit`;
CREATE TABLE `t_b_ding_submit` (
                                   `id` bigint(20) NOT NULL COMMENT '主键',
                                   `service_id` bigint(20) NOT NULL COMMENT '服务单元id',
                                   `sop_no` varchar(100) NOT NULL COMMENT 'sop单号',
                                   `crm_no` varchar(100) NOT NULL COMMENT 'crm单号',
                                   `user_archives_id` bigint(20) NOT NULL COMMENT '人员档案id',
                                   `approach_time` bigint(20) NULL COMMENT '进场时间',
                                   `departure_time` bigint(20) NULL COMMENT '出场时间',
                                   `role_type` varchar(20) NOT NULL COMMENT '项目角色类型',
                                   `actual_days` int(11) DEFAULT NULL COMMENT '实际出勤（天）',
                                   `weekdays` int(11) DEFAULT NULL COMMENT '工作日（天）',
                                   `weekends` int(11) DEFAULT NULL COMMENT '周末（天）',
                                   `legal_holidays` int(11) DEFAULT NULL COMMENT '法定节假日（天）',
                                   `work_hours` decimal(8,2) DEFAULT NULL COMMENT '累计工时（小时）',
                                   `violation_days` decimal(8,2) DEFAULT NULL COMMENT '违规工时',
                                   `submit_user_id` bigint(20) NOT NULL COMMENT '提交人id',
                                   `submit_time` bigint(20) NOT NULL COMMENT '提交时间',
                                   `submit_status` varchar(20) NOT NULL COMMENT '提交状态',
                                   `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                   `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                   `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                   `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                   PRIMARY KEY (`id`),
                                   UNIQUE KEY `t_b_ding_submit_un` (`sop_no`,`user_archives_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤提交审核表';

-- ----------------------------
-- Table structure for t_b_engineer_allocation
-- ----------------------------
DROP TABLE IF EXISTS `t_b_engineer_allocation`;
CREATE TABLE `t_b_engineer_allocation` (
                                           `id` bigint(20) NOT NULL COMMENT '主键',
                                           `service_id` bigint(20) DEFAULT NULL COMMENT '服务单元id',
                                           `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                                           `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                           `engineer_user_id` bigint(20) NOT NULL COMMENT '用户id',
                                           `role_id` bigint(20) NOT NULL COMMENT '角色id',
                                           PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='sop工程师分配表';

-- ----------------------------
-- Table structure for t_b_product
-- ----------------------------
DROP TABLE IF EXISTS `t_b_product`;
CREATE TABLE `t_b_product` (
                               `id` bigint(20) NOT NULL COMMENT '主键',
                               `name` varchar(100) NOT NULL COMMENT '名称',
                               `code` varchar(100) NOT NULL COMMENT '编码，OFFICE：教务处，CLOUD_MARK：研究生',
                               `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                               `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                               `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                               PRIMARY KEY (`id`),
                               UNIQUE KEY `t_b_product_un_code` (`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='产品表';

-- ----------------------------
-- Table structure for t_b_project_exchange
-- ----------------------------
DROP TABLE IF EXISTS `t_b_project_exchange`;
CREATE TABLE `t_b_project_exchange` (
                                        `id` bigint(20) NOT NULL COMMENT '主键',
                                        `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                                        `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                        `exchange_no` varchar(100) DEFAULT NULL COMMENT '变更编号',
                                        `service_id` bigint(20) DEFAULT NULL COMMENT '服务单元id',
                                        `type` varchar(20) NOT NULL COMMENT '变更类型，PLAN：项目信息及计划，CANCEL：项目取消',
                                        `reason` varchar(500) NOT NULL COMMENT '变更原因',
                                        `content_json` mediumtext COMMENT '变更内容json',
                                        `result` varchar(20) DEFAULT NULL COMMENT '处理结果，FINISH：已完成，NOT_UPDATE：经沟通，取消变更，PARTIAL_UPDATE：经沟通，部分变更',
                                        `remark` varchar(500) DEFAULT NULL COMMENT '变更备注',
                                        `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                        `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                        `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                        PRIMARY KEY (`id`),
                                        UNIQUE KEY `t_b_project_exchange_un_exchange_no` (`exchange_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='项目变更计划表';

-- ----------------------------
-- Table structure for t_b_quality_problem_apply
-- ----------------------------
DROP TABLE IF EXISTS `t_b_quality_problem_apply`;
CREATE TABLE `t_b_quality_problem_apply` (
                                             `id` bigint(20) NOT NULL COMMENT '主键',
                                             `sop_no` varchar(100) NOT NULL COMMENT 'sop单号',
                                             `crm_no` varchar(100) NOT NULL COMMENT 'crm单号',
                                             `problem_no` varchar(100) NOT NULL COMMENT '问题编号',
                                             `service_id` bigint(20) NOT NULL COMMENT '服务单元id',
                                             `summary` varchar(1000) NOT NULL COMMENT '问题简要',
                                             `remark` varchar(500) NOT NULL COMMENT '问题情况说明',
                                             `user_ids` varchar(500) DEFAULT NULL COMMENT '责任人id数组',
                                             `type` varchar(20) DEFAULT NULL COMMENT '问题类型，UPDATE：修正类，OPTIMIZE：优化类，NO_PROBLEM：不是问题',
                                             `reason` varchar(20) DEFAULT NULL COMMENT '问题原因类型，EXEC：执行类，MANAGER：管理协调类，FLOW：流程制度类，PRODUCT_PROBLEM：产品缺陷类，PRODUCT_MANAGER：产品运维类，OTHER：其它',
                                             `attachment_ids` varchar(500) DEFAULT NULL COMMENT '附件id数组',
                                             `Influence_degree` varchar(20) DEFAULT NULL COMMENT '影响度：A，B，C，D',
                                             `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                             `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                             `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                             `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                             PRIMARY KEY (`id`),
                                             UNIQUE KEY `t_b_quality_problem_apply_UN_probem_no` (`problem_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='质量问题反馈表';

-- ----------------------------
-- Table structure for t_b_service
-- ----------------------------
DROP TABLE IF EXISTS `t_b_service`;
CREATE TABLE `t_b_service` (
                               `id` bigint(20) NOT NULL COMMENT '主键',
                               `name` varchar(100) NOT NULL COMMENT '名称',
                               `type` varchar(20) NOT NULL COMMENT '类型，OFFICE：教务处，CLOUD_MARK：研究生',
                               `start_time` bigint(20) NOT NULL COMMENT '开始时间',
                               `end_time` bigint(20) NOT NULL COMMENT '结束时间',
                               `region_people_count` int(11) NOT NULL COMMENT '区域协调人数量',
                               `region_project_count` int(11) NOT NULL COMMENT '区域项目数量',
                               `service_lead_id` bigint(20) NOT NULL COMMENT '服务单元负责人id',
                               `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                               `status` varchar(20) NOT NULL COMMENT '状态，NEW：新建，PUBLISH：已发布，FINISH：已完结，CANCEL：已作废',
                               `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                               `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                               `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                               `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                               PRIMARY KEY (`id`),
                               UNIQUE KEY `t_b_service_un_name` (`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务单元表';

-- ----------------------------
-- Table structure for t_b_service_region
-- ----------------------------
DROP TABLE IF EXISTS `t_b_service_region`;
CREATE TABLE `t_b_service_region` (
                                      `id` bigint(20) NOT NULL COMMENT '主键',
                                      `service_id` bigint(20) NOT NULL COMMENT '服务单元id',
                                      `region_name` varchar(100) NOT NULL COMMENT '大区名称',
                                      `lead_id` bigint(20) NOT NULL COMMENT '大区经理id',
                                      `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                      `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                      `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                      `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                      `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                      PRIMARY KEY (`id`),
                                      UNIQUE KEY `t_b_service_region_un` (`service_id`,`region_name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务区域规划表';

-- ----------------------------
-- Table structure for t_b_service_region_detail
-- ----------------------------
DROP TABLE IF EXISTS `t_b_service_region_detail`;
CREATE TABLE `t_b_service_region_detail` (
                                             `id` bigint(20) NOT NULL COMMENT '主键',
                                             `service_region_id` bigint(20) NOT NULL COMMENT '服务区域规划id',
                                             `country` varchar(100) NOT NULL DEFAULT '中国' COMMENT '国家',
                                             `province` varchar(100) NOT NULL COMMENT '省份',
                                             `city` varchar(100) NOT NULL COMMENT '城市',
                                             `area` varchar(100) DEFAULT NULL COMMENT '县区',
                                             `address` varchar(150) DEFAULT NULL COMMENT '地址',
                                             PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务区域规划明细表';

-- ----------------------------
-- Table structure for t_b_service_scope
-- ----------------------------
DROP TABLE IF EXISTS `t_b_service_scope`;
CREATE TABLE `t_b_service_scope` (
                                     `id` bigint(20) NOT NULL COMMENT '主键',
                                     `service_id` bigint(20) NOT NULL COMMENT '服务单元id',
                                     `crm_no` varchar(100) NOT NULL COMMENT 'crm单号',
                                     `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                     `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                     PRIMARY KEY (`id`),
                                     UNIQUE KEY `t_b_service_scope_un` (`service_id`,`crm_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务单元范围管理表';

-- ----------------------------
-- Table structure for t_b_session
-- ----------------------------
DROP TABLE IF EXISTS `t_b_session`;
CREATE TABLE `t_b_session` (
                               `id` varchar(100) NOT NULL COMMENT '主键',
                               `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                               `identity` varchar(100) NOT NULL COMMENT '用户标识',
                               `type` varchar(500) NOT NULL COMMENT '用户类型',
                               `source` varchar(50) NOT NULL COMMENT '访问来源',
                               `platform` varchar(50) NOT NULL COMMENT '设备分类',
                               `device_id` varchar(100) NOT NULL COMMENT '设备标识',
                               `address` varchar(100) NOT NULL COMMENT '登录IP地址',
                               `access_token` varchar(50) NOT NULL COMMENT '访问令牌',
                               `last_access_time` bigint(20) DEFAULT NULL COMMENT '最近访问时间',
                               `last_access_ip` varchar(100) DEFAULT NULL COMMENT '最近访问IP地址',
                               `update_time` bigint(20) DEFAULT NULL COMMENT '令牌更新时间',
                               `expire_time` bigint(20) DEFAULT NULL COMMENT '令牌强制失效时间',
                               `app_source` varchar(50) DEFAULT NULL COMMENT '应用来源',
                               PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='会话表';

-- ----------------------------
-- Table structure for t_b_sop_info
-- ----------------------------
DROP TABLE IF EXISTS `t_b_sop_info`;
CREATE TABLE `t_b_sop_info` (
                                `id` bigint(20) NOT NULL COMMENT '主键',
                                `crm_no` varchar(100) NOT NULL COMMENT 'crm单号',
                                `sop_no` varchar(100) NOT NULL COMMENT 'sop单号',
                                `service_id` bigint(20) NOT NULL COMMENT '服务单元id',
                                `custom_id` bigint(20) NOT NULL COMMENT '客户id',
                                `product_id` bigint(20) NOT NULL COMMENT '产品id',
                                `type` varchar(20) NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP',
                                `status` varchar(20) NOT NULL COMMENT '状态，DRAFT：草稿，START：提交',
                                `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                PRIMARY KEY (`id`),
                                UNIQUE KEY `t_b_sop_info_un_sop_no` (`sop_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='sop表';

-- ----------------------------
-- Table structure for t_b_sop_info_detail
-- ----------------------------
DROP TABLE IF EXISTS `t_b_sop_info_detail`;
CREATE TABLE `t_b_sop_info_detail` (
                                       `id` bigint(20) NOT NULL COMMENT '主键',
                                       `sop_info_id` bigint(20) NOT NULL COMMENT 'sop信息id',
                                       `lead_id` bigint(20) NOT NULL COMMENT '大区经理id',
                                       `region_user_id` bigint(20) DEFAULT NULL COMMENT '区域负责人id',
                                       `engineer_user_id` bigint(20) DEFAULT NULL COMMENT '实施工程师id',
                                       `assistant_engineer_user_ids` varchar(500) DEFAULT NULL COMMENT '助理实施工程师id数组',
                                       PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='sop明细表';

-- ----------------------------
-- Table structure for t_b_task
-- ----------------------------
DROP TABLE IF EXISTS `t_b_task`;
CREATE TABLE `t_b_task` (
                            `id` bigint(20) NOT NULL COMMENT '主键',
                            `type` varchar(50) NOT NULL COMMENT '任务类型，USER_ARCHIVES_IMPORT：人员档案导入，USER_ARCHIVES_EXPORT：人员档案导出，CUSTOM_IMPORT：客户导入',
                            `entity_id` bigint(20) DEFAULT NULL COMMENT '关联业务对象id',
                            `status` varchar(30) NOT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
                            `summary` text COMMENT '实时摘要信息',
                            `progress` double DEFAULT NULL COMMENT '执行进度',
                            `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
                            `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                            `import_file_name` varchar(100) DEFAULT NULL COMMENT '导入文件名',
                            `import_file_path` varchar(500) DEFAULT NULL COMMENT '导入文件路径',
                            `result_file_path` varchar(500) DEFAULT NULL COMMENT '导出文件路径',
                            `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
                            `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                            `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                            `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                            `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                            `remark` mediumtext COMMENT '备注',
                            `obj_name` varchar(500) DEFAULT NULL COMMENT '实体名称',
                            `reset_count` int(11) DEFAULT '0' COMMENT '重试次数',
                            `version` int(11) DEFAULT '0' COMMENT '更新版本号',
                            `error_message` varchar(500) DEFAULT NULL COMMENT '人工错误原因',
                            PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='导入导出任务表';

-- ----------------------------
-- Table structure for t_b_user_archives
-- ----------------------------
DROP TABLE IF EXISTS `t_b_user_archives`;
CREATE TABLE `t_b_user_archives` (
                                     `id` bigint(20) NOT NULL COMMENT '主键',
                                     `code` varchar(50) NOT NULL COMMENT '编号',
                                     `name` varchar(100) NOT NULL COMMENT '姓名',
                                     `country` varchar(100) NOT NULL DEFAULT '中国' COMMENT '国家',
                                     `province` varchar(100) NOT NULL COMMENT '省份',
                                     `city` varchar(100) NOT NULL COMMENT '城市',
                                     `area` varchar(100) DEFAULT NULL COMMENT '县区',
                                     `address` varchar(150) DEFAULT NULL COMMENT '地址',
                                     `identity` varchar(50) NOT NULL COMMENT '身份证号',
                                     `gender` varchar(20) NOT NULL COMMENT '性别，MAN：男，WOMAN：女',
                                     `education` varchar(20) DEFAULT NULL COMMENT '学历',
                                     `mobile_number` varchar(30) NOT NULL COMMENT '手机号',
                                     `email` varchar(30) NOT NULL COMMENT '邮箱',
                                     `base_photo_path` varchar(500) DEFAULT NULL COMMENT '底照保存地址',
                                     `status` varchar(20) NOT NULL COMMENT '人力资源状态（FREE("空闲"),NON_EXECUTION_SOP("未执行SOP"),EXECUTION_SOP("执行SOP");）',
                                     `enable` tinyint(1) NOT NULL DEFAULT '1' COMMENT '是否启用（1启用，0作废）',
                                     `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                     `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                     `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                     `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                     PRIMARY KEY (`id`),
                                     UNIQUE KEY `t_b_user_archives_un_identity` (`identity`),
                                     UNIQUE KEY `t_b_user_archives_un_mobile_number` (`mobile_number`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='人员档案表';

-- ----------------------------
-- Table structure for t_b_user_archives_allocation
-- ----------------------------
DROP TABLE IF EXISTS `t_b_user_archives_allocation`;
CREATE TABLE `t_b_user_archives_allocation` (
                                                `id` bigint(20) NOT NULL COMMENT '主键',
                                                `service_id` bigint(20) DEFAULT NULL COMMENT '服务单元id',
                                                `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                                                `crm_no` varchar(100) NOT NULL COMMENT 'crm单号',
                                                `region_user_id` bigint(20) DEFAULT NULL COMMENT '区域协调人id',
                                                `user_id` bigint(20) NOT NULL COMMENT '工程师id',
                                                `role_id` bigint(20) NOT NULL COMMENT '角色id',
                                                `archives_id` bigint(20) NOT NULL COMMENT '人员档案id',
                                                PRIMARY KEY (`id`),
                                                UNIQUE KEY `t_b_user_archives_allocation_un` (`crm_no`,`archives_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='人员分配表';

-- ----------------------------
-- Table structure for t_b_user_archives_supplier
-- ----------------------------
DROP TABLE IF EXISTS `t_b_user_archives_supplier`;
CREATE TABLE `t_b_user_archives_supplier` (
                                              `id` bigint(20) NOT NULL COMMENT '主键',
                                              `user_archives_id` bigint(20) NOT NULL COMMENT '人员档案id',
                                              `supplier_id` bigint(20) DEFAULT NULL COMMENT '供应商ID',
                                              `archives_time` bigint(20) NOT NULL COMMENT '入档时间',
                                              `authentication_time` bigint(20) DEFAULT NULL COMMENT '认证时间',
                                              `authentication_score` double DEFAULT NULL COMMENT '认证成绩',
                                              `authentication_valid_time` bigint(20) DEFAULT NULL COMMENT '认证有效时间',
                                              `remark` varchar(500) DEFAULT NULL COMMENT '备注',
                                              PRIMARY KEY (`id`),
                                              UNIQUE KEY `t_b_user_archives_supplier_un` (`user_archives_id`,`supplier_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='人员档案与供应商表';

-- ----------------------------
-- Table structure for t_b_violation
-- ----------------------------
DROP TABLE IF EXISTS `t_b_violation`;
CREATE TABLE `t_b_violation` (
                                 `id` bigint(20) NOT NULL COMMENT '主键',
                                 `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                 `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                                 `code` varchar(200) NOT NULL COMMENT '编号',
                                 `service_id` bigint(20) DEFAULT NULL COMMENT '服务单元id',
                                 `user_id` bigint(20) NOT NULL COMMENT '负责人id',
                                 `type` varchar(20) NOT NULL COMMENT '类型，CONTENT_ERROR：内容错误虚假，LOGIC_ERROR：逻辑不合理，OTHER：其它',
                                 `content` varchar(500) NOT NULL COMMENT '违规情况',
                                 `status` varchar(20) NOT NULL COMMENT '状态，NOT_START：未跟进，FOLLOW：跟进，CLOSE：关闭，RESTART：重启',
                                 `attachment_ids` varchar(500) DEFAULT NULL COMMENT '附件ids',
                                 `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                 `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                 `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                 `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                 `close_time` bigint(20) DEFAULT NULL COMMENT '关闭时间',
                                 PRIMARY KEY (`id`),
                                 UNIQUE KEY `t_b_violation_un_code` (`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='sop违规登记表';

-- ----------------------------
-- Table structure for t_b_violation_detail
-- ----------------------------
DROP TABLE IF EXISTS `t_b_violation_detail`;
CREATE TABLE `t_b_violation_detail` (
                                        `id` bigint(20) NOT NULL COMMENT '主键',
                                        `violation_id` bigint(20) NOT NULL COMMENT '违规id',
                                        `remark` varchar(500) NOT NULL COMMENT '跟进说明',
                                        `attachment_ids` varchar(500) DEFAULT NULL COMMENT '附件ids',
                                        `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                        `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                        PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='违规明细表';

-- ----------------------------
-- Table structure for t_d_form_widget
-- ----------------------------
DROP TABLE IF EXISTS `t_d_form_widget`;
CREATE TABLE `t_d_form_widget` (
                                   `id` bigint(20) NOT NULL COMMENT '主键',
                                   `code` varchar(20) DEFAULT NULL COMMENT '编码，TEXT：文本，NUMBER：数字文本，TEXTAREA：文本域，SINGLE_SELECT：单选下拉框，MULTIPLE_SELECT：多选下拉框，RADIO：单选框，CHECKBOX：复选框，DATE：日期，BUTTON：按钮，IMG：图片，VIDEO：视频，AUDIO：音频，FILE：文件，LABEL：标签，TABLE：表格，SIGN：签名，CASCADE_ADDRESS：城市选择控件，OTHER：其它，FORM_GROUP_TITLE：带虚线分割线的标题，ONLE_TITLE：普通标题，独占一行，RADIO_WITH_INPUT：带1个输入框的单选框，DEVICE_IN_TABLE：设备入库表格，DEVICE_OUT_TABLE：设备出库表格',
                                   `type` varchar(20) DEFAULT NULL COMMENT '类型，FORM：表单，TABLE：表格，OTHER：其它',
                                   `form_id` varchar(100) DEFAULT NULL COMMENT '表单id',
                                   `form_name` varchar(100) DEFAULT NULL COMMENT '表单名称',
                                   `title` varchar(100) DEFAULT NULL COMMENT '标题',
                                   `input_type` varchar(20) DEFAULT NULL COMMENT '输入类型，STRING：字符串，INT：整型，LONG：长整型，DOUBLE：浮点型，ENUM：枚举，DATE：日期，BOOL：布尔，BIGDECIMAL：精确浮点，FILE：文件，ARRAY：数组',
                                   `required` tinyint(4) DEFAULT '1' COMMENT '是否必填，0：非必填，1：必填',
                                   `readable` tinyint(4) DEFAULT '0' COMMENT '是否只读，0：非只读，1：只读',
                                   `writable` tinyint(4) DEFAULT '1' COMMENT '是否可写，0：非可写，1：可写',
                                   `visable` tinyint(4) DEFAULT '1' COMMENT '是否可见，0：不可见，1：可见',
                                   `scale` int(11) DEFAULT NULL COMMENT '保留小数点几位',
                                   `length` int(11) DEFAULT NULL COMMENT '长度',
                                   `binding` varchar(30) DEFAULT NULL COMMENT '绑定字段',
                                   `data_grid` varchar(100) DEFAULT NULL COMMENT '数据来源',
                                   `tips` varchar(100) DEFAULT NULL COMMENT '提示',
                                   `format` varchar(20) DEFAULT NULL COMMENT '格式',
                                   `span` int(11) DEFAULT '6' COMMENT '占宽符',
                                   `sub_title` varchar(1000) DEFAULT NULL COMMENT '子标题',
                                   `options` varchar(500) DEFAULT NULL COMMENT '选项值，RADIO或CHECKBOX填此值',
                                   `flow_type` varchar(20) NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程',
                                   `form_setup` int(11) NOT NULL COMMENT '表单步骤',
                                   `handle` tinyint(4) NOT NULL DEFAULT '0' COMMENT '处理时限，0：不处理，1：可处理',
                                   `form_group` int(11) DEFAULT NULL COMMENT '控件组',
                                   `role_type` varchar(20) DEFAULT NULL COMMENT '角色类型',
                                   `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                   `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                   PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='动态表单控件表';

-- ----------------------------
-- Table structure for t_d_form_widget_metadata
-- ----------------------------
DROP TABLE IF EXISTS `t_d_form_widget_metadata`;
CREATE TABLE `t_d_form_widget_metadata` (
                                            `id` bigint(20) NOT NULL COMMENT '主键',
                                            `type` varchar(20) NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程',
                                            `flow_deployment_id` varchar(50) NOT NULL COMMENT '流程部署id',
                                            `version` int(11) NOT NULL DEFAULT '1' COMMENT '版本号',
                                            `code` varchar(20) NOT NULL COMMENT '编码，TEXT：文本，NUMBER：数字文本，TEXTAREA：文本域，SINGLE_SELECT：单选下拉框，MULTIPLE_SELECT：多选下拉框，RADIO：单选框，CHECKBOX：复选框，DATE：日期，BUTTON：按钮，IMG：图片，VIDEO：视频，AUDIO：音频，FILE：文件，LABEL：标签，TABLE：表格，SIGN：签名，CASCADE_ADDRESS：城市选择控件，OTHER：其它，FORM_GROUP_TITLE：带虚线分割线的标题，ONLE_TITLE：普通标题，独占一行，RADIO_WITH_INPUT：带1个输入框的单选框，DEVICE_IN_TABLE：设备入库表格，DEVICE_OUT_TABLE：设备出库表格',
                                            `field_id` varchar(100) NOT NULL COMMENT '字段id',
                                            `field_name` varchar(100) NOT NULL COMMENT '字段名称',
                                            `field_title` varchar(50) DEFAULT NULL COMMENT '字段标题',
                                            `field_check` tinyint(4) NOT NULL DEFAULT '0' COMMENT '是否勾选，0：未勾选，1：勾选',
                                            `setup` int(11) NOT NULL COMMENT '流程步骤',
                                            `change_join` tinyint(4) DEFAULT NULL COMMENT '是否是项目计划变更关联字段，0：不是，1：是',
                                            `create_time` bigint(20) NOT NULL COMMENT '创建时间',
                                            PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='动态表单元数据表';

-- ----------------------------
-- Table structure for t_d_table_prop
-- ----------------------------
DROP TABLE IF EXISTS `t_d_table_prop`;
CREATE TABLE `t_d_table_prop` (
                                  `id` bigint(20) NOT NULL COMMENT '主键',
                                  `widget_id` bigint(20) DEFAULT NULL COMMENT '控件id',
                                  `td_index` int(11) DEFAULT NULL COMMENT '序号',
                                  `td_id` varchar(50) DEFAULT NULL COMMENT '列id',
                                  `td_name` varchar(50) DEFAULT NULL COMMENT '列名称',
                                  `title` varchar(100) DEFAULT NULL COMMENT '标题',
                                  `width` double DEFAULT NULL COMMENT '宽度',
                                  `height` double DEFAULT NULL COMMENT '高度',
                                  `color` varchar(20) DEFAULT NULL COMMENT '颜色',
                                  `rowspan` int(11) DEFAULT NULL COMMENT '跨行',
                                  `colspan` int(11) DEFAULT NULL COMMENT '跨列',
                                  `sub_title` varchar(1000) DEFAULT NULL COMMENT '子标题',
                                  `tips` varchar(100) DEFAULT NULL COMMENT '提示',
                                  `td_order` tinyint(4) DEFAULT '1' COMMENT '是否排序，0：不排序，1：排序',
                                  `td_search` tinyint(4) DEFAULT '1' COMMENT '是否搜索，0：不搜索，1：搜索',
                                  `edit_widget_id` bigint(20) DEFAULT NULL COMMENT '编辑控件id',
                                  `obj` mediumtext COMMENT '实体内容',
                                  `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='表格属性表';

-- ----------------------------
-- Table structure for t_f_custom_flow
-- ----------------------------
DROP TABLE IF EXISTS `t_f_custom_flow`;
CREATE TABLE `t_f_custom_flow` (
                                   `id` bigint(20) NOT NULL COMMENT '主键',
                                   `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                   `name` varchar(100) NOT NULL COMMENT '流程名称',
                                   `model_type` varchar(50) NOT NULL COMMENT '流程模式类型，USER_FIXED：指定人员，APPROVE_SET：发起人指定',
                                   `type` varchar(100) NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程',
                                   `enable` tinyint(4) DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                   `publish` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否发布，0：否，1：是',
                                   `object_data` mediumtext COMMENT '自定义流程源数据',
                                   `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                   `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                   `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                   `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                   `version` int(11) NOT NULL DEFAULT '1' COMMENT '版本号',
                                   `flow_definition_id` varchar(50) DEFAULT NULL COMMENT '流程定义id',
                                   `flow_deployment_id` varchar(50) DEFAULT NULL COMMENT '流程部署id',
                                   `flow_process_var` mediumtext COMMENT '流程过程变量数据',
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='自定义流程表';

-- ----------------------------
-- Table structure for t_f_custom_flow_allocation
-- ----------------------------
DROP TABLE IF EXISTS `t_f_custom_flow_allocation`;
CREATE TABLE `t_f_custom_flow_allocation` (
                                              `id` bigint(20) NOT NULL COMMENT '主键',
                                              `type` varchar(100) NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程',
                                              `t_f_custom_flow_id` bigint(20) DEFAULT NULL COMMENT '自定义流程id',
                                              `t_f_custom_flow_entity_id` bigint(20) DEFAULT NULL COMMENT '流程实体id',
                                              `role_allocation` mediumtext NOT NULL COMMENT '角色分配',
                                              `version` int(11) DEFAULT NULL COMMENT '版本号',
                                              PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='流程角色分配表';

-- ----------------------------
-- Table structure for t_f_custom_flow_entity
-- ----------------------------
DROP TABLE IF EXISTS `t_f_custom_flow_entity`;
CREATE TABLE `t_f_custom_flow_entity` (
                                          `id` bigint(20) NOT NULL COMMENT '主键',
                                          `t_f_custom_flow_id` bigint(20) NOT NULL COMMENT '自定义流程id',
                                          `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                          `code` varchar(30) NOT NULL COMMENT '编号',
                                          `obj_id` bigint(20) DEFAULT NULL COMMENT '实体id',
                                          `flow_id` bigint(20) DEFAULT NULL COMMENT '流程id',
                                          `flow_process_var` mediumtext NOT NULL COMMENT '流程过程变量',
                                          `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                          `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                          `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                          `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                          PRIMARY KEY (`id`) USING BTREE,
                                          UNIQUE KEY `t_f_custom_flow_entity_UN_code` (`code`,`flow_id`),
                                          KEY `flow_entity_flowId_idx` (`flow_id`),
                                          KEY `flow_entity_custom_flowId_idx` (`t_f_custom_flow_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='自定义流程实体数据';

-- ----------------------------
-- Table structure for t_f_flow_approve
-- ----------------------------
DROP TABLE IF EXISTS `t_f_flow_approve`;
CREATE TABLE `t_f_flow_approve` (
                                    `id` bigint(20) NOT NULL COMMENT '主键',
                                    `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                    `flow_id` bigint(20) NOT NULL COMMENT '流程id',
                                    `approve_id` bigint(20) DEFAULT NULL COMMENT '审核id',
                                    `status` varchar(50) NOT NULL COMMENT '状态，START：已开始，AUDITING：审核中，REJECT：已驳回，END：已终止，FINISH：已结束',
                                    `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                    `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                    `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                    `setup` tinyint(4) NOT NULL COMMENT '计划步骤',
                                    `approve_setup` int(11) DEFAULT NULL COMMENT '审批步骤',
                                    `approve_max_setup` int(11) DEFAULT NULL COMMENT '最大审批步骤',
                                    `transfer_assignee` mediumtext COMMENT '流程转办',
                                    `all_assignee` mediumtext COMMENT '会签审批人',
                                    `reject_link` mediumtext COMMENT '驳回路线',
                                    PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='流程审核表';

-- ----------------------------
-- Table structure for t_f_flow_log
-- ----------------------------
DROP TABLE IF EXISTS `t_f_flow_log`;
CREATE TABLE `t_f_flow_log` (
                                `id` bigint(20) NOT NULL COMMENT '主键',
                                `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                `flow_id` bigint(20) NOT NULL COMMENT '流程id',
                                `object_table` varchar(100) DEFAULT NULL COMMENT '实体表',
                                `object_id` bigint(20) NOT NULL COMMENT '实体id',
                                `approve_id` bigint(20) DEFAULT NULL COMMENT '审核人id',
                                `approve_setup` tinyint(4) DEFAULT NULL COMMENT '审核步骤',
                                `approve_remark` mediumtext COMMENT '审核意见',
                                `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                `approve_operation` varchar(50) NOT NULL COMMENT '审批操作，START：提交，DRAFT：草稿，APPROVE：审批，REJECT：驳回，END：终止，CANCEL：撤销，EXCHANGE：转他人审批，FINISH：结束',
                                `pend_approve_id` mediumtext COMMENT '待审核人id',
                                `attachment_ids` mediumtext COMMENT '附件id集合',
                                `approve_multi_setup` int(11) DEFAULT NULL COMMENT '审批步骤',
                                `approve_user_approve_type` varchar(30) DEFAULT NULL COMMENT '审批用户类别，ORDER：依次审批，ALL：会签（所有人必须审批），SOME：或签（一名审批人同意或拒绝即可）',
                                `task_id` bigint(20) DEFAULT NULL COMMENT '流程节点id',
                                PRIMARY KEY (`id`) USING BTREE,
                                KEY `flow_log_objectId_idx` (`object_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='流程流水表';

-- ----------------------------
-- Table structure for t_g_error
-- ----------------------------
DROP TABLE IF EXISTS `t_g_error`;
CREATE TABLE `t_g_error` (
                             `id` bigint(20) NOT NULL COMMENT '主键',
                             `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                             `obj_id` bigint(20) DEFAULT NULL COMMENT '实体id',
                             `summary` mediumtext COMMENT '实时摘要信息',
                             `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                             PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='全局异常错误信息表';

-- ----------------------------
-- Table structure for t_s_auth
-- ----------------------------
DROP TABLE IF EXISTS `t_s_auth`;
CREATE TABLE `t_s_auth` (
                            `id` bigint(20) NOT NULL COMMENT '主键',
                            `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                            `access_key` varchar(255) DEFAULT NULL COMMENT 'accessKey',
                            `access_secret` varchar(255) DEFAULT NULL COMMENT 'accessSecret',
                            `description` text COMMENT '离线授权证书',
                            `type` varchar(16) NOT NULL COMMENT '授权类型',
                            `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                            `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                            `expire_time` bigint(20) DEFAULT NULL COMMENT '过期时间',
                            `file` mediumblob COMMENT '文件数据',
                            PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='激活授权配置表';

-- ----------------------------
-- Table structure for t_s_job_log
-- ----------------------------
DROP TABLE IF EXISTS `t_s_job_log`;
CREATE TABLE `t_s_job_log` (
                               `id` bigint(20) NOT NULL COMMENT '主键',
                               `job_remind_id` bigint(20) NOT NULL COMMENT '任务提醒id',
                               `type` varchar(20) NOT NULL COMMENT '类型，FLOW：流程，NOTICE：公告',
                               `code` varchar(100) DEFAULT NULL COMMENT '编码',
                               `obj_id` bigint(20) NOT NULL COMMENT '实体id',
                               `obj_name` varchar(100) DEFAULT NULL COMMENT '实体名称',
                               `remind_type` varchar(20) NOT NULL COMMENT '提醒类别，BEFORE：提前，AFTER：超时',
                               `create_id` bigint(20) NOT NULL COMMENT '创建人id',
                               `create_time` bigint(20) NOT NULL COMMENT '创建时间',
                               PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='任务日志表';

-- ----------------------------
-- Table structure for t_s_job_remind
-- ----------------------------
DROP TABLE IF EXISTS `t_s_job_remind`;
CREATE TABLE `t_s_job_remind` (
                                  `id` bigint(20) NOT NULL COMMENT '主键',
                                  `type` varchar(20) NOT NULL COMMENT '类型，FLOW：流程，NOTICE：公告',
                                  `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                  `code` varchar(100) DEFAULT NULL COMMENT '编码',
                                  `obj_id` bigint(20) NOT NULL COMMENT '实体id',
                                  `obj_name` varchar(100) DEFAULT NULL COMMENT '实体名称',
                                  `remind_type` varchar(20) NOT NULL COMMENT '提醒类别，BEFORE：提前，AFTER：超时',
                                  `exec_time` bigint(20) DEFAULT NULL COMMENT '执行时间',
                                  `exec_field` varchar(50) DEFAULT NULL COMMENT '执行字段',
                                  `receive_user_id` bigint(20) DEFAULT NULL COMMENT '接收人id',
                                  `approve_operation` varchar(20) DEFAULT NULL COMMENT '审批操作，START：提交，DRAFT：草稿，APPROVE：审批，REJECT：驳回，END：终止，CANCEL：撤销，EXCHANGE：转他人审批，FINISH：结束',
                                  `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                  `create_id` bigint(20) NOT NULL COMMENT '创建人id',
                                  `create_time` bigint(20) NOT NULL COMMENT '创建时间',
                                  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='任务提醒表';

-- ----------------------------
-- Table structure for t_wxapp_info
-- ----------------------------
DROP TABLE IF EXISTS `t_wxapp_info`;
CREATE TABLE `t_wxapp_info` (
                                `id` bigint(20) NOT NULL COMMENT '主键',
                                `open_id` varchar(100) NOT NULL COMMENT 'openId',
                                `phone` varchar(30) NOT NULL COMMENT '手机号',
                                PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='微信信息表';

DROP TABLE IF EXISTS `t_b_sop_plan_date_log`;
CREATE TABLE `t_b_sop_plan_date_log` (
                                         `id` bigint NOT NULL COMMENT '主键',
                                         `sop_id` bigint DEFAULT NULL COMMENT 'sopId',
                                         `flow_id` bigint NOT NULL COMMENT '流程id',
                                         `begin_time` bigint NOT NULL COMMENT '计划开始时间',
                                         `end_time` bigint NOT NULL COMMENT '计划结束时间',
                                         `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                         `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                         `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                         `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                         PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='sop进程计划时间日志表';

DROP TABLE IF EXISTS `t_b_device_delivery`;
CREATE TABLE `t_b_device_delivery` (
                                       `id` bigint NOT NULL COMMENT '主键',
                                       `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                                       `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                       `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                                       `device_no` varchar(50) NOT NULL COMMENT '设备编号',
                                       `usage_type` varchar(45) NOT NULL COMMENT '用途类型，PROJECT：项目，OTHER：其它',
                                       `supplier_id` bigint DEFAULT NULL COMMENT '供应商id',
                                       `num` int DEFAULT NULL COMMENT '数量',
                                       `mailing_address` varchar(500) DEFAULT NULL COMMENT '邮寄地址',
                                       `consignee` varchar(50) DEFAULT NULL COMMENT '收件人',
                                       `consignee_phone` varchar(30) DEFAULT NULL COMMENT '收件人联系电话',
                                       `express_no` varchar(50) DEFAULT NULL COMMENT '快递单号',
                                       `express_photo_path` varchar(500) DEFAULT NULL COMMENT '快递单拍照',
                                       `status` varchar(30) DEFAULT NULL COMMENT '状态，UN_DELIVER：待发货，DELIVER：已发货，RECEIVE：已签收，CANCEL：作废',
                                       `deliver_time` bigint DEFAULT NULL COMMENT '发货时间',
                                       `deliver_user_id` bigint DEFAULT NULL COMMENT '发货人id',
                                       `receive_time` bigint DEFAULT NULL COMMENT '签收时间',
                                       `receive_user_id` bigint DEFAULT NULL COMMENT '签收人id',
                                       `enable` tinyint DEFAULT NULL COMMENT '是否启用，0：停用，1：启用',
                                       `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                       `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                       `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                       `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                       PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备发货表';

-- ----------------------------
-- Function structure for currval
-- ----------------------------
DROP FUNCTION IF EXISTS `currval`;
delimiter ;;
CREATE FUNCTION `currval`(seq_name VARCHAR(50)) RETURNS int
    DETERMINISTIC
BEGIN
	DECLARE value LONG;
	SET value = 0;
SELECT current_value INTO value
FROM sequence
WHERE name = seq_name;
RETURN value;
END;
;;
delimiter ;

-- ----------------------------
-- Function structure for nextval
-- ----------------------------
DROP FUNCTION IF EXISTS `nextval`;
delimiter ;;
CREATE FUNCTION `nextval`(seq_name VARCHAR(50)) RETURNS int
    DETERMINISTIC
BEGIN
UPDATE sequence
SET current_value = current_value + increment
WHERE name = seq_name;
RETURN currval(seq_name);
END;
;;
delimiter ;

INSERT INTO `t_b_product` VALUES (1, '教务处', 'OFFICE', 1, 1, NULL);
INSERT INTO `t_b_product` VALUES (2, '云阅卷', 'CLOUD_MARK', 1, 1, NULL);

INSERT INTO `sequence` VALUES ('cloudMarkSopFlow', 0, 1);
INSERT INTO `sequence` VALUES ('delayWarn', 0, 1);
INSERT INTO `sequence` VALUES ('deviceInOutCode', 0, 1);
INSERT INTO `sequence` VALUES ('dingExceptionFlow', 0, 1);
INSERT INTO `sequence` VALUES ('officeSopFlow', 0, 1);
INSERT INTO `sequence` VALUES ('projectExchangeFlow', 0, 1);
INSERT INTO `sequence` VALUES ('qualityProblemFlow', 0, 1);
INSERT INTO `sequence` VALUES ('userArchivesCode', 0, 1);
INSERT INTO `sequence` VALUES ('violationWarn', 0, 1);

INSERT INTO `sys_privilege` VALUES (1, '系统管理', 'systemManage', 'MENU', NULL, 8, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (2, '配置管理', 'configureManage', 'MENU', 1, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (3, '客户配置', 'custom', 'MENU', 2, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (4, '供应商配置', 'supplier', 'MENU', 2, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (5, '设备配置', 'device', 'MENU', 2, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (6, '服务档位配置', 'level', 'MENU', 2, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (7, '考勤配置', 'dingConfigure', 'MENU', 2, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (8, '系统日志', 'logManage', 'MENU', 1, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (9, '通知公告管理', 'noticeManage', 'MENU', 8, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (10, '日志查询', 'log', 'MENU', 8, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (11, '用户管理', 'userManage', 'MENU', NULL, 7, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (12, '组织架构管理', 'orgManage', 'MENU', 11, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (13, '组织架构管理', 'org', 'MENU', 12, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (14, '用户权限管理', 'userPrivilegeManage', 'MENU', 11, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (15, '用户管理', 'user', 'MENU', 14, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (16, '角色管理', 'role', 'MENU', 14, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (17, '初始化密码', 'InitPwd', 'LINK', 15, 3, NULL, '18', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (18, '用户初始化密码', '/api/admin/user/init_password', 'URL', 15, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (19, '项目质量管理', 'projectQualityManage', 'MENU', NULL, 6, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (20, '项目质量管理', 'projectChildQualityManage', 'MENU', 19, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (21, '质量问题反馈', 'qualityProblem', 'MENU', 20, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (23, '质量监控分析', 'qualityProblemControl', 'MENU', 20, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (24, '工时管理', 'hoursManage', 'MENU', NULL, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (25, '工时管理', 'hoursChildManage', 'MENU', 24, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (26, '考勤打卡', 'ding', 'MENU', 25, 1, NULL, '3017,2044,2045,2046', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (27, '异常审核', 'exception', 'MENU', 25, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (28, '考勤提交', 'dingSubmit', 'MENU', 25, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (29, '工时统计', 'hourSubmit', 'MENU', 25, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (30, '考勤监控预警', 'dingWarn', 'MENU', 25, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (31, '资源保障', 'resourceManage', 'MENU', NULL, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (32, '人资保障', 'humanManage', 'MENU', 31, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (33, '人员档案管理', 'userArchives', 'MENU', 32, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (34, '人员调配', 'userAllocate', 'MENU', 32, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (35, '人资监控', 'userArchivesControl', 'MENU', 32, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (36, '设备保障', 'deviceManage', 'MENU', 31, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (37, '出入库登记查询', 'deviceInOut', 'MENU', 36, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (38, '设备资源监控', 'deviceControl', 'MENU', 36, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (39, 'SOP管理', 'sopManage', 'MENU', NULL, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (40, 'SOP管理', 'sopChildManage', 'MENU', 39, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (41, '教务处SOP管理', 'office', 'MENU', 40, 1, NULL, '3034,3035', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (42, '研究生SOP管理', 'cloudMark', 'MENU', 40, 2, NULL, '3034,3035', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (43, '设备出入库登记', 'deviceInOutSop', 'MENU', 40, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (44, '项目计划变更报备', 'projectExchange', 'MENU', 40, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (45, 'SOP监控', 'sopControlManage', 'MENU', 39, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (46, 'SOP监控台', 'sopControl', 'MENU', 45, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (47, '延期预警', 'delayWarn', 'MENU', 45, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (48, '违规登记', 'violation', 'MENU', 45, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (49, '服务单元管理', 'serviceManage', 'MENU', NULL, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (50, '派单管理', 'crmManage', 'MENU', 49, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (51, '派单管理', 'crm', 'MENU', 50, 1, 'AUTH', '1032,1035,1036,1037,1038', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (52, '统计查询', 'crmCount', 'MENU', 50, 2, 'AUTH', '1034', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (53, '服务单元管理', 'serviceChildManage', 'MENU', 49, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (54, '服务单元管理', 'service', 'MENU', 53, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (55, '服务范围管理', 'serviceScope', 'MENU', 53, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (56, '服务区域规划', 'serviceRegion', 'MENU', 53, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (57, '统计查询', 'serviceCount', 'MENU', 53, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (58, '我的工作台', 'workManage', 'MENU', NULL, 1, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (59, '工作台', 'workChildManage', 'MENU', 58, 1, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (60, '消息提醒', 'message', 'MENU', 59, 1, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (61, '待办任务', 'work', 'MENU', 59, 2, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (62, '通知公告', 'notice', 'MENU', 59, 3, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (63, '公共接口', 'common', 'MENU', NULL, 9, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (64, '权限管理', 'menuManage', 'MENU', 63, 1, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (65, '用户登录', '/api/admin/common/login', 'URL', 64, 1, 'NO_AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (66, '用户登出', '/api/admin/common/logout', 'URL', 64, 2, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (67, '文件上传', '/api/admin/common/file/upload', 'URL', 64, 3, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (68, '文件下载', '/api/admin/common/file/download', 'URL', 64, 4, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (69, '文件预览', '/api/admin/common/file/preview', 'URL', 64, 5, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (72, '任务管理查询', '/api/admin/task/query', 'URL', 240, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (73, '下载导入模板', '/api/admin/common/download_import_template', 'URL', 64, 6, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (74, '列表', 'List', 'LIST', 3, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (75, '查询', 'Select', 'BUTTON', 3, 3, 'AUTH', '262', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (76, '查询条件', 'Condition', 'CONDITION', 3, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (77, '修改', 'Update', 'LINK', 3, 1, 'AUTH', '260,263', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (78, '删除', 'Delete', 'LINK', 3, 2, 'AUTH', '259,260', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (79, '新增', 'Add', 'BUTTON', 3, 1, 'AUTH', '258', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (80, '批量导入', 'BatchImport', 'BUTTON', 3, 2, 'AUTH', '261', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (81, '新增', 'Add', 'BUTTON', 4, 1, 'AUTH', '264,266', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (82, '列表', 'List', 'LIST', 4, 1, 'AUTH', '268', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (83, '修改', 'Update', 'LINK', 4, 1, 'AUTH', '269,266', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (84, '启用/禁用', 'Enable', 'LINK', 4, 2, 'AUTH', '265', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (85, '列表', 'List', 'LIST', 5, 1, 'AUTH', '275', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (86, '查询条件', 'Condition', 'CONDITION', 5, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (87, '新增', 'Add', 'BUTTON', 5, 1, 'AUTH', '270', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (88, '作废', 'Cancel', 'BUTTON', 5, 2, 'AUTH', '271', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (89, '列表', 'List', 'LIST', 6, 1, 'AUTH', '256', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (90, '新增', 'Add', 'BUTTON', 6, 1, 'AUTH', '254,300', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (91, '修改', 'Update', 'LINK', 6, 1, 'AUTH', '254,257', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (92, '删除', 'Delete', 'LINK', 6, 2, 'AUTH', '253', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (93, '列表', 'List', 'LIST', 7, 1, 'AUTH', '281', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (94, '新增', 'Add', 'BUTTON', 7, 1, 'AUTH', '277', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (95, '修改', 'Update', 'LINK', 7, 1, 'AUTH', '282', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (96, '删除', 'Delete', 'LINK', 7, 2, 'AUTH', '278', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (97, '查询条件', 'Condition', 'CONDITION', 7, 1, 'AUTH', '281', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (98, '列表', 'List', 'LIST', 9, 1, 'AUTH', '1004', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (99, '新增', 'Add', 'BUTTON', 9, 1, 'AUTH', '1005', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (100, '撤销', 'Cancel', 'LINK', 9, 2, 'AUTH', '1003', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (101, '回执查询', 'Select', 'LINK', 9, 2, 'AUTH', '1006,1007,1008', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (102, '查询条件', 'Condition', 'CONDITION', 9, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (103, '修改', 'Update', 'LINK', 9, 3, 'AUTH', '1002,1005', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (104, '发布', 'Publish', 'LINK', 9, 1, 'AUTH', '1003', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (105, '查询', 'Select', 'BUTTON', 5, 3, 'AUTH', '275', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (106, '查询', 'Select', 'BUTTON', 7, 2, 'AUTH', '281', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (107, '查询', 'Select', 'BUTTON', 9, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (108, '列表', 'List', 'LIST', 10, 1, 'AUTH', '2014', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (109, '查询条件', 'Condition', 'CONDITION', 10, 1, 'AUTH', '2014', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (110, '查询', 'Select', 'BUTTON', 10, 2, 'AUTH', '2014', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (111, '列表', 'List', 'LIST', 13, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (112, '新增', 'Add', 'BUTTON', 13, 1, 'AUTH', '249', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (113, '查询条件', 'Condition', 'CONDITION', 13, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (114, '修改', 'Update', 'LINK', 13, 1, 'AUTH', '249', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (115, '启用/禁用', 'Enable', 'LINK', 13, 2, 'AUTH', '250', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (116, '查询', 'Select', 'BUTTON', 13, 2, 'AUTH', '248', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (117, '列表', 'List', 'LIST', 15, 1, 'AUTH', '252', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (118, '新增', 'Add', 'BUTTON', 15, 1, 'AUTH', '283', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (119, '查询条件', 'Condition', 'CONDITION', 15, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (120, '修改', 'Update', 'LINK', 15, 1, 'AUTH', '283', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (121, '启用/禁用', 'Enable', 'LINK', 15, 2, 'AUTH', '284', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (122, '查询', 'Select', 'BUTTON', 15, 2, 'AUTH', '252', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (123, '列表', 'List', 'LIST', 16, 1, 'AUTH', '288', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (124, '新增', 'Add', 'BUTTON', 16, 1, 'AUTH', '289,3016', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (125, '修改', 'Update', 'LINK', 16, 1, 'AUTH', '289,3016', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (126, '删除', 'Delete', 'LINK', 16, 2, 'AUTH', '290', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (127, '列表', 'List', 'LIST', 21, 1, 'AUTH', '1031,3044,1030', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (128, '作废', 'Cancel', 'BUTTON', 21, 1, 'AUTH', '1029,3044', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (129, '查询条件', 'Condition', 'CONDITION', 21, 1, 'AUTH', '3044', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (130, '查询', 'Select', 'BUTTON', 21, 2, 'AUTH', '1030,3044', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (131, '待审核-列表', 'List', 'LIST', 27, 1, 'AUTH', '3017,3019', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (132, '待审核-批量通过', 'BatchPass', 'BUTTON', 27, 1, 'AUTH', '3017,3022', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (133, '待审核-查询条件', 'Condition', 'CONDITION', 27, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (134, '待审核-查询', 'Select', 'BUTTON', 27, 3, 'AUTH', '3017,3019', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (135, '待审核-批量不通过', 'BatchNoPass', 'BUTTON', 27, 2, 'AUTH', '3017,3022', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (136, '待审核-通过', 'Pass', 'LINK', 27, 1, 'AUTH', '3017,3021', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (137, '待审核-不通过', 'NoPass', 'LINK', 27, 2, 'AUTH', '3017,3021', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (138, '已审核-列表', 'List', 'LIST', 27, 1, 'AUTH', '3017,3020', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (139, '已审核-查询条件', 'Condition', 'CONDITION', 27, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (140, '已审核-查询', 'Select', 'BUTTON', 27, 3, 'AUTH', '3017,3020', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (141, '列表', 'List', 'LIST', 28, 1, 'AUTH', '2037,2038,1098', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (142, '批量提交', 'BatchSubmit', 'BUTTON', 28, 1, 'AUTH', '2039', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (143, '查询条件', 'Condition', 'CONDITION', 28, 1, 'AUTH', '2037,2038', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (144, '查询', 'Select', 'BUTTON', 28, 3, 'AUTH', '2037,2038', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (145, '批量导出', 'BatchExport', 'BUTTON', 28, 2, 'AUTH', '2042', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (146, '提交', 'Submit', 'LINK', 28, 1, 'AUTH', '2039', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (147, '撤回/取消撤回', 'Reject', 'LINK', 28, 2, 'AUTH', '2040,2041', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (148, '列表', 'List', 'LIST', 29, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (149, '导出', 'Export', 'BUTTON', 29, 1, 'AUTH', '1027', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (150, '查询条件', 'Condition', 'CONDITION', 29, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (151, '查询', 'Select', 'BUTTON', 29, 2, 'AUTH', '1026, 1028', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (152, '撤回/取消撤回', 'Reject', 'LINK', 29, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (153, '列表', 'List', 'LIST', 33, 1, 'AUTH', '2016,2017', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (154, '新增', 'Add', 'BUTTON', 33, 1, 'AUTH', '2018,2027', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (155, '批量导入', 'BatchImport', 'BUTTON', 33, 2, 'AUTH', '2019', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (156, '查询条件', 'Condition', 'CONDITION', 33, 1, 'AUTH', '2016,2017', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (157, '查询', 'Select', 'BUTTON', 33, 5, 'AUTH', '2016,2017', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (158, '批量导出', 'BatchExport', 'BUTTON', 33, 3, 'AUTH', '2020', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (159, '作废', 'Cancel', 'BUTTON', 33, 4, 'AUTH', '2051', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (160, '修改', 'Update', 'LINK', 33, 1, 'AUTH', '2018', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (161, '列表', 'List', 'LIST', 34, 1, 'AUTH', '2021,2022,2023', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (162, '查询条件', 'Condition', 'CONDITION', 34, 1, 'AUTH', '2021,2022', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (163, '查询', 'Select', 'BUTTON', 34, 1, 'AUTH', '2021,2022', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (164, '调配', 'Allocate', 'LINK', 34, 1, 'AUTH', '2024,2030,2031', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (165, '发布SOP', 'Publish', 'LINK', 34, 2, 'AUTH', '2028', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (166, '撤销发布', 'UnPublish', 'LINK', 34, 3, 'AUTH', '2029', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (167, '列表', 'List', 'LIST', 37, 1, 'AUTH', '2036', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (168, '查询条件', 'Condition', 'CONDITION', 37, 1, 'AUTH', '2036', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (169, '查询', 'Select', 'BUTTON', 37, 1, 'AUTH', '2036', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (170, '列表', 'List', 'LIST', 38, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (171, '列表', 'List', 'LIST', 41, 1, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (172, '查询条件', 'Condition', 'CONDITION', 41, 1, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (173, '作废', 'Cancel', 'BUTTON', 41, 1, 'AUTH', '3046', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (174, '查询', 'Select', 'BUTTON', 41, 2, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (175, '填报', 'Fill', 'LINK', 41, 1, 'AUTH', '3042', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (176, '计划变更报备', 'ProjectExchange', 'LINK', 41, 2, 'AUTH', '3025', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (177, '新增SOP', 'AddSop', 'LINK', 41, 3, 'AUTH', '3041', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (178, '新增违规 ', 'AddViolation', 'LINK', 41, 4, 'AUTH', '1025', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (179, '上报质量问题', 'ProblemSubmit', 'LINK', 41, 5, 'AUTH', '1031', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (180, '列表', 'List', 'LIST', 42, 1, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (181, '查询条件', 'Condition', 'CONDITION', 42, 1, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (182, '作废', 'Cancel', 'BUTTON', 42, 1, 'AUTH', '3046', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (183, '查询', 'Select', 'BUTTON', 42, 2, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (184, '填报', 'Fill', 'LINK', 42, 1, 'AUTH', '3042', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (185, '计划变更报备', 'ProjectExchange', 'LINK', 42, 2, 'AUTH', '3025', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (186, '新增违规 ', 'AddViolation', 'LINK', 42, 3, 'AUTH', '1025', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (187, '上报质量问题', 'ProblemSubmit', 'LINK', 42, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (188, '列表', 'List', 'LIST', 43, 1, 'AUTH', '2032', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (189, '查询条件', 'Condition', 'CONDITION', 43, 1, 'AUTH', '2032', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (190, '新增', 'Add', 'BUTTON', 43, 1, 'AUTH', '2033,2034,2035', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (191, '作废', 'Cancel', 'BUTTON', 43, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (192, '查询', 'Select', 'BUTTON', 43, 3, 'AUTH', '2032', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (193, '列表', 'List', 'LIST', 44, 1, 'AUTH', '3026', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (194, '查询条件', 'Condition', 'CONDITION', 44, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (196, '查询', 'Select', 'BUTTON', 44, 2, 'AUTH', '3026', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (197, '列表', 'List', 'LIST', 47, 1, 'AUTH', '1015', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (198, '查询条件', 'Condition', 'CONDITION', 47, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (199, '查询', 'Select', 'BUTTON', 47, 1, 'AUTH', '1015', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (200, '跟进', 'Follow', 'LINK', 47, 1, 'AUTH', '1014,1017,1018', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (201, '关闭', 'Close', 'LINK', 47, 2, 'AUTH', '1013', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (202, '重启', 'Restart', 'LINK', 47, 3, 'AUTH', '1016', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (203, '列表', 'List', 'LIST', 48, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (204, '查询条件', 'Condition', 'CONDITION', 48, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (205, '查询', 'Select', 'BUTTON', 48, 1, 'AUTH', '1023', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (206, '跟进', 'Follow', 'LINK', 48, 1, 'AUTH', '1019,1020,1022,1096', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (207, '关闭', 'Close', 'LINK', 48, 2, 'AUTH', '1021', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (208, '重启', 'Restart', 'LINK', 48, 3, 'AUTH', '1024', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (209, '列表', 'List', 'LIST', 51, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (210, '查询条件', 'Condition', 'CONDITION', 51, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (211, '新增', 'Add', 'BUTTON', 51, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (212, '作废', 'Cancel', 'BUTTON', 51, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (213, '批量划定', 'BatchDelimit', 'BUTTON', 51, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (214, '查询', 'Select', 'BUTTON', 51, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (215, '划定', 'Delimit', 'LINK', 51, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (216, '修改', 'Update', 'LINK', 51, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (217, '重新划定', 'ReDelimit', 'LINK', 51, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (218, '列表', 'List', 'LIST', 54, 1, 'AUTH', '2001', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (219, '查询条件', 'Condition', 'CONDITION', 54, 1, 'AUTH', '2001', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (220, '新增', 'Add', 'BUTTON', 54, 1, 'AUTH', '2000', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (221, '查询', 'Select', 'BUTTON', 54, 2, 'AUTH', '2001', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (222, '修改', 'Update', 'LINK', 54, 1, 'AUTH', '2000', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (223, '作废', 'Cancel', 'LINK', 54, 2, 'AUTH', '2005', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (224, '设置考勤组', 'Set', 'LINK', 54, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (225, '发布', 'Publish', 'LINK', 54, 4, 'AUTH', '2002', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (226, '重启', 'Restart', 'LINK', 54, 5, 'AUTH', '2004', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (227, '关闭', 'Close', 'LINK', 54, 6, 'AUTH', '2003', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (228, '列表', 'List', 'LIST', 55, 1, 'AUTH', '2006,2007', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (229, '查询条件', 'Condition', 'CONDITION', 55, 1, 'AUTH', '2006', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (230, '新增服务范围', 'Add', 'BUTTON', 55, 1, 'AUTH', '2008,2009', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (231, '查询', 'Select', 'BUTTON', 55, 2, 'AUTH', '2006,2007', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (232, '移除', 'Delete', 'LINK', 55, 1, 'AUTH', '2010', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (233, '列表', 'List', 'LIST', 56, 1, 'AUTH', '2011', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (234, '查询条件', 'Condition', 'CONDITION', 56, 1, 'AUTH', '2011', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (235, '新增', 'Add', 'BUTTON', 56, 1, 'AUTH', '2012,2047', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (236, '查询', 'Select', 'BUTTON', 56, 2, 'AUTH', '2011', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (237, '修改', 'Update', 'LINK', 56, 1, 'AUTH', '2012,2047', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (238, '删除', 'Delete', 'LINK', 56, 2, 'AUTH', '2013', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (239, '任务管理', 'taskManage', 'MENU', 1, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (240, '任务管理', 'task', 'MENU', 239, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (241, '列表', 'List', 'LIST', 240, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (242, '查询条件', 'Condition', 'CONDITION', 240, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (243, '查询', 'Select', 'BUTTON', 240, 2, 'AUTH', '72', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (244, '导出日志', 'Export', 'LINK', 240, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (245, '下载文件', 'Download', 'LINK', 240, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (246, '获取菜单权限', '/api/admin/sys/privilege/list', 'URL', 64, 7, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (247, '获取用户权限', '/api/admin/common/get_menu', 'URL', 64, 8, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (248, '组织架构查询', '/api/admin/org/list', 'URL', 13, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (249, '组织架构保存/修改', '/api/admin/org/save', 'URL', 13, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (250, '组织架构启用/禁用', '/api/admin/org/enable', 'URL', 13, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (251, '获取机构下所有用户', '/api/admin/common/get_org_user_list', 'URL', 64, 9, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (252, '用户查询', '/api/admin/user/list', 'URL', 15, 1, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (253, '服务档位删除', '/api/sys/level/delete', 'URL', 6, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (254, '服务档位', '/api/sys/level/get', 'URL', 6, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (255, '服务档位列表', '/api/sys/level/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (256, '服务档位列表', '/api/sys/level/query', 'URL', 6, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (257, '服务档位修改', '/api/sys/level/update', 'URL', 6, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (258, '客户新增', '/api/sys/custom/add', 'URL', 3, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (259, '客户删除', '/api/sys/custom/delete', 'URL', 3, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (260, '客户查询', '/api/sys/custom/get', 'URL', 3, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (261, '客户批量导入', '/api/sys/custom/import', 'URL', 3, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (262, '客户查询条件', '/api/sys/custom/query', 'URL', 3, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (263, '客户修改', '/api/sys/custom/update', 'URL', 3, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (264, '供应商新增', '/api/sys/supplier/add', 'URL', 4, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (265, '供应商启用/禁用', '/api/sys/supplier/enable', 'URL', 4, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (266, '供应商列表', '/api/sys/supplier/get', 'URL', 4, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (267, '供应商列表', '/api/sys/supplier/list', 'URL', 64, 27, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (268, '供应商列表', '/api/sys/supplier/query', 'URL', 4, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (269, '供应商修改', '/api/sys/supplier/update', 'URL', 4, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (270, '设备新增修改', '/api/sys/device/save', 'URL', 5, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (271, '设备作废', '/api/sys/device/delete', 'URL', 5, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (272, '设备禁用', '/api/sys/device/disable', 'URL', 5, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (273, '设备查询条件', '/api/sys/device/get', 'URL', 5, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (274, '设备列表', '/api/sys/device/list', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (275, '设备查询条件', '/api/sys/device/query', 'URL', 5, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (276, '设备新增', '/api/sys/device/update', 'URL', 5, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (277, '考勤新增', '/api/sys/ding/group/add', 'URL', 7, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (278, '考勤删除', '/api/sys/ding/group/delete', 'URL', 7, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (279, '考勤查询', '/api/sys/ding/group/get', 'URL', 7, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (280, '考勤查询', '/api/sys/ding/group/list', 'URL', 7, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (281, '考勤查询条件', '/api/sys/ding/group/query', 'URL', 7, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (282, '考勤修改', '/api/sys/ding/group/update', 'URL', 7, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (283, '用户新增/修改', '/api/admin/user/save', 'URL', 15, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (284, '用户启用/禁用', '/api/admin/user/enable', 'URL', 15, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (285, '获取所有角色', '/api/admin/common/get_role_list', 'URL', 64, 10, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (286, '获取所有用户', '/api/admin/common/get_user_list', 'URL', 64, 11, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (287, '获取所有机构', '/api/admin/common/get_org_list', 'URL', 64, 12, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (288, '角色查询', '/api/admin/role/list', 'URL', 16, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (289, '角色新增/修改', '/api/admin/role/save', 'URL', 16, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (290, '角色删除', '/api/admin/role/delete', 'URL', 16, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (300, '服务档位新增', '/api/sys/level/add', 'URL', 6, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1000, '通知公告', '/api/sys/notice/delete', 'URL', 9, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1001, '通知公告', '/api/sys/notice/disable', 'URL', 9, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1002, '通知公告', '/api/sys/notice/get', 'URL', 9, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1003, '通知公告发布撤销', '/api/sys/notice/publishOrUnpublish', 'URL', 9, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1004, '通知公告列表', '/api/sys/notice/query', 'URL', 9, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1005, '通知公告新增修改', '/api/sys/notice/save', 'URL', 9, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1006, '回执查询统计', '/api/sys/message/count', 'URL', 9, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1007, '回执查询到出', '/api/sys/message/export', 'URL', 9, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1008, '回执查询', '/api/sys/message/query', 'URL', 9, 9, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1009, '按多个类型统计当前用户未读消息', '/api/sys/message/countByTypes', 'URL', 59, 1, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1010, '查看消息详细', '/api/sys/message/get', 'URL', 59, 2, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1011, '按多个类型查询当前用户未读消息', '/api/sys/message/pageByTypes', 'URL', 59, 3, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1012, '标记消息已阅读', '/api/sys/message/read', 'URL', 59, 4, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1013, '延期预警关闭', '/api/admin/tb/delay/warn/close', 'URL', 47, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1014, '延期预警跟进', '/api/admin/tb/delay/warn/get', 'URL', 47, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1015, '延期预警查询', '/api/admin/tb/delay/warn/query', 'URL', 47, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1016, '延期预警重启', '/api/admin/tb/delay/warn/restart', 'URL', 47, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1017, '延期预警跟进', '/api/admin/tb/delay/warn/detail/list', 'URL', 47, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1018, '延期预警跟进', '/api/admin/tb/delay/warn/detail/save', 'URL', 47, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1019, '违规登记跟进', '/api/admin/tb/violation/detail/query', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1020, '违规登记跟进', '/api/admin/tb/violation/detail/save', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1021, '违规登记关闭', '/api/admin/tb/violation/close', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1022, '违规登记跟进', '/api/admin/tb/violation/get', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1023, '违规登记查询', '/api/admin/tb/violation/query', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1024, '违规登记重启', '/api/admin/tb/violation/restart', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1025, '违规登记保存', '/api/admin/tb/violation/save', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1026, '工时统计', '/api/admin/tb/ding/count', 'URL', 29, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1027, '工时统计', '/api/admin/tb/ding/export', 'URL', 29, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1028, '工时统计', '/api/admin/tb/ding/query', 'URL', 29, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1029, '质量问题反馈', '/api/admin/tb/quality/problem/apply/delete', 'URL', 21, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1030, '质量问题反馈', '/api/admin/tb/quality/problem/apply/query', 'URL', 21, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1031, '质量问题反馈', '/api/admin/tb/quality/problem/apply/save', 'URL', 21, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1032, '派单管理', '/api/admin/tb/crm/batchDisable', 'URL', 50, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1034, '派单管理', '/api/admin/tb/crm/countWaitZone', 'URL', 50, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1035, '派单管理', '/api/admin/tb/crm/delete', 'URL', 50, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1036, '派单管理', '/api/admin/tb/crm/get', 'URL', 50, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1037, '派单管理', '/api/admin/tb/crm/query', 'URL', 50, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1038, '派单管理', '/api/admin/tb/crm/save', 'URL', 50, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1039, '客户表查询', '/api/sys/custom/list', 'URL', 64, 25, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1040, '获取产品信息列表接口', '/api/admin/tb/product/list', 'URL', 64, 26, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1041, '考勤明细查询', '/api/admin/tb/ding/attendance/count', 'URL', 1044, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1042, '考勤明细查询', '/api/admin/tb/ding/attendance/export', 'URL', 1044, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1043, '考勤明细查询', '/api/admin/tb/ding/attendance/query', 'URL', 1044, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1044, '考勤明细查询', 'dingDetail', 'MENU', 25, 1, NULL, '1041,1042,1043', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (1045, '设备品牌表列表接口', '/api/sys/device/brand/list', 'URL', 64, 25, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1046, '设备启用', '/api/sys/device/enable', 'URL', 5, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1047, '设备导入', '/api/sys/device/import', 'URL', 5, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1048, '批量启用', 'BatchEnable', 'BUTTON', 5, 4, 'AUTH', '1045', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (1049, '批量禁用', 'BatchUnEnable', 'BUTTON', 5, 5, 'AUTH', '272', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (1050, '批量导入', 'BatchImport', 'BUTTON', 5, 6, 'AUTH', '1047', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (1051, '获取SOP用户', '/api/admin/sop/user/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1052, '修改', 'Update', 'LINK', 5, 1, 'AUTH', '276', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (1053, '禁用启用', 'Enable', 'LINK', 5, 2, 'AUTH', '272,1046', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (1054, '获取用户', '/api/admin/tb/violation/creator/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1055, '获取用户', '/api/admin/tb/violation/user/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1056, '获取用户', '/api/admin/tb/quality/problem/apply/user/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1057, '二期接口', '/api/sop/analyse/attendance', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1058, '二期接口', '/api/sop/analyse/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1059, '二期接口', '/api/sop/analyse/overview', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1060, '二期接口', '/api/sop/analyse/processing', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1061, '二期接口', '/api/sop/analyse/trend', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1062, '二期接口', '/api/service/analyse/device/use', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1063, '二期接口', '/api/service/analyse/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1064, '二期接口', '/api/service/analyse/overview', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1065, '二期接口', '/api/service/analyse/personnel/quota', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1066, '二期接口', '/api/service/analyse/project/allocation', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1067, '二期接口', '/api/service/analyse/region/personnel', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1068, '二期接口', '/api/service/analyse/supplier', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1069, '二期接口', '/api/service/analyse/supplier/detail', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1070, '二期接口', '/api/service/analyse/supplier/region', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1071, '二期接口', '/api/service/analyse/supplier/region/detail', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1072, '二期接口', '/api/service/analyse/supplier/role', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1073, '二期接口', '/api/admin/tb/crm/analyse/custom/type', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1074, '二期接口', '/api/admin/tb/crm/analyse/custom/type/detail', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1075, '二期接口', '/api/admin/tb/crm/analyse/monthly', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1076, '二期接口', '/api/admin/tb/crm/analyse/monthly/detail', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1077, '二期接口', '/api/admin/tb/crm/analyse/project', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1078, '二期接口', '/api/admin/tb/crm/analyse/project/detail', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1079, '二期接口', '/api/admin/tb/crm/analyse/region', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1080, '二期接口', '/api/admin/tb/crm/analyse/region/detail', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1081, '二期接口', '/api/admin/tb/crm/analyse/supplier', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1082, '二期接口', '/api/admin/tb/crm/analyse/supplier/detail', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1083, '二期接口', '/api/quality/analyse/influence', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1084, '二期接口', '/api/quality/analyse/influence/detail', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1085, '二期接口', '/api/quality/analyse/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1086, '二期接口', '/api/quality/analyse/pie', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1087, '二期接口', '/api/quality/analyse/progress', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1088, '二期接口', '/api/quality/analyse/radar', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1089, '二期接口', '/api/quality/analyse/supplier/detail', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1090, '二期接口', '/api/sys/device/monitor/count', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1091, '二期接口', '/api/sys/device/monitor/countByModel', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1092, '二期接口', '/api/sys/device/monitor/countByServiceUnit', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1093, '二期接口', '/api/sys/device/monitor/countByServiceUnitDetail', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1094, '二期接口', '/api/sop/schedule/detail', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1095, '二期接口', '/api/sop/schedule/progress', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1096, '违规登记跟进', '/api/admin/tb/violation/getByCode', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1097, '二期接口', '/api/sop/schedule/projectProgress', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1098, '考勤提交-明细', '/api/admin/tb/ding/detail', 'URL', 28, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (1099, '二期接口', '/api/admin/tb/crm/analyse/project/count', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2000, '服务单元管理-编辑', '/api/service/service/unit/edit', 'URL', 54, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2001, '服务单元管理-分页查询', '/api/service/service/unit/page', 'URL', 54, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2002, '服务单元管理-发布', '/api/service/service/unit/publish', 'URL', 54, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2003, '服务单元管理-关闭', '/api/service/service/unit/cloze', 'URL', 54, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2004, '服务单元管理-重启', '/api/service/service/unit/restart', 'URL', 54, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2005, '服务单元管理-作废', '/api/service/service/unit/cancel', 'URL', 54, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2006, '服务范围管理-分页查询', '/api/service/service/scope/page', 'URL', 55, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2007, '服务范围管理-派单小计', '/api/service/service/scope/subTotal', 'URL', 55, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2008, '服务范围管理-新增服务范围-未绑定派单分页查询', '/api/service/service/scope/unbind/page', 'URL', 55, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2009, '服务范围管理-批量划定派单和服务单元关系', '/api/service/service/scope/bind_batch', 'URL', 55, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2010, '服务范围管理-移出', '/api/service/service/scope/unbind', 'URL', 55, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2011, '服务区域规划-分页查询', '/api/service/service/region/page', 'URL', 56, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2012, '服务区域规划-编辑', '/api/service/service/region/edit', 'URL', 56, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2013, '服务区域规划-删除', '/api/service/service/region/remove', 'URL', 56, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2014, '日志管理-查询', '/api/sys/log/page', 'URL', 10, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2015, '公共接口-查询服务单元列表', '/api/admin/common/query_service_unit', 'URL', 64, 25, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2016, '人员档案管理-分页查询', '/api/admin/user/archives/page', 'URL', 33, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2017, '人员档案管理-小计', '/api/admin/user/archives/sub_total', 'URL', 33, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2018, '人员档案管理-编辑', '/api/admin/user/archives/edit', 'URL', 33, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2019, '人员档案管理-导入', '/api/admin/user/archives/import', 'URL', 33, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2020, '人员档案管理-导出', '/api/admin/user/archives/export', 'URL', 33, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2021, '人员调配管理-分页查询', '/api/admin/user/archives/allocation/page', 'URL', 34, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2022, '人员调配管理-小计查询', '/api/admin/user/archives/allocation/sub_total', 'URL', 34, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2023, '人员调配管理-根据派单号查询派单分配信息', '/api/admin/user/archives/allocation/detail_by_crm_no', 'URL', 34, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2024, '人员调配管理-调配', '/api/admin/user/archives/allocation/edit', 'URL', 34, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2025, '人员调配管理—自动调配', '/api/admin/user/archives/allocation/auto_edit', 'URL', 34, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2026, '批量调配', 'AllocateBatch', 'BUTTON', 34, 4, 'AUTH', '2025', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (2027, '人员档案管理-生成编号', '/api/admin/user/archives/create_code', 'URL', 33, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2028, '人员调配管理-发布', '/api/admin/user/archives/allocation/publish', 'URL', 34, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2029, '人员调配管理-撤销发布', '/api/admin/user/archives/allocation/un_publish', 'URL', 34, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2030, '人员调配管理-查询空闲工程师', '/api/admin/user/archives/allocation/free_engineer', 'URL', 34, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2031, '人员调配管理-查询空闲区域协调人', '/api/admin/user/archives/allocation/free_coordinator', 'URL', 34, 9, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2032, 'sop管理 - 设备出入库登记查询', '/api/admin/device/in/out/sop_page', 'URL', 43, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2033, '查询可出库的设备信息', '/api/admin/device/in/out/can_out_info', 'URL', 43, 2, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2034, '查询可入库的设备信息', '/api/admin/device/in/out/can_in_info', 'URL', 43, 3, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2035, '设备出入库提交', '/api/admin/device/in/out/submit', 'URL', 43, 4, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2036, '资源保障-设备保障-出入库登记查询', '/api/admin/device/in/out/source_page', 'URL', 37, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2037, '考勤提交-分页查询', '/api/admin/tb/ding/submit/page', 'URL', 28, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2038, '考勤提交-小计', '/api/admin/tb/ding/submit/sub_total', 'URL', 28, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2039, '考勤提交-批量提交', '/api/admin/tb/ding/submit/submit_batch', 'URL', 28, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2040, '考勤提交-考勤撤回', '/api/admin/tb/ding/submit/apply_withdraw', 'URL', 28, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2041, '考勤提交-取消撤回', '/api/admin/tb/ding/submit/cancel_withdraw', 'URL', 28, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2042, '考勤提交-批量导出', '/api/admin/tb/ding/submit/export', 'URL', 28, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2043, '查询项目派单信息', '/api/admin/tb/crm/crm_project_info', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2044, '考勤打卡信息', '/api/admin/tb/ding/ding_info', 'URL', 26, 1, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2045, '考勤结果统计', '/api/admin/tb/ding/ding_statistic', 'URL', 26, 2, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2046, '考勤打卡保存', '/api/admin/tb/ding/ding_save', 'URL', 26, 3, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2047, '服务区域规划-查询可选大区经理', '/api/service/service/region/find_leader', 'URL', 56, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2048, '考勤打卡-查询进行中的sop', '/api/admin/tb/ding/ding_find_running_sop', 'URL', 26, 4, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2049, '考勤打卡-查询所有sop', '/api/admin/tb/ding/ding_find_all_sop', 'URL', 26, 5, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2051, '人员档案管理-作废', '/api/admin/user/archives/cancel', 'URL', 33, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2052, '出入库登记查询-查询登记人列表', '/api/admin/device/in/out/registrant_list', 'URL', 64, 33, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (2053, '智能客服', 'aiCustomerManage', 'MENU', NULL, 11, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (2054, '智能客服', 'aiCustomerChildManage', 'MENU', 2053, 1, NULL, '2055', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (2055, '软通智能客服单点登录', '/api/sso/rtzf/login', 'URL', 2054, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3000, '修改密码', '/api/admin/common/update_password', 'URL', 64, 13, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3001, '流程审批接口', '/api/admin/flow/approve', 'URL', 64, 14, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3002, '创建流程接口', '/api/admin/flow/createDeployment', 'URL', 64, 15, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3003, '结束流程接口', '/api/admin/flow/delete', 'URL', 64, 16, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3004, '获取流程部署信息接口', '/api/admin/flow/deployment/data', 'URL', 64, 17, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3005, '获取流程表单属性接口', '/api/admin/flow/form_properties/get', 'URL', 64, 18, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3006, '获取流程待办', '/api/admin/flow/task/list', 'URL', 64, 19, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3007, '流程转办接口', '/api/admin/flow/task/approver/exchange', 'URL', 64, 20, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3008, '上传流程接口', '/api/admin/flow/upload', 'URL', 64, 21, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3009, '获取流程详细信息接口', '/api/admin/flow/view', 'URL', 64, 22, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3010, '清除缓存接口', '/api/admin/common/cache_clear', 'URL', 3012, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3011, '超管菜单', 'adminManage', 'MENU', NULL, 10, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3012, '清除缓存', 'cacheManage', 'MENU', 3011, 1, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3013, '消息提醒规则', 'messageRuleManage', 'MENU', 3011, 2, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3014, '清除缓存', 'Clear', 'BUTTON', 3012, 1, 'AUTH', '3010', 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3015, '查询枚举接口', '/api/admin/common/query_enum', 'URL', 64, 23, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3016, '角色权限查询接口', '/api/admin/role/privilege/list', 'URL', 16, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3017, '考勤异常申请接口', '/api/admin/ding/exception/apply/save', 'URL', 26, 1, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3018, '根据角色类型获取所有用户', '/api/admin/common/get_user_list_by_role_type', 'URL', 64, 24, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3019, '考勤异常待审核接口', '/api/admin/ding/exception/apply/flow/task/un_done/list', 'URL', 27, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3020, '考勤异常已审核接口', '/api/admin/ding/exception/apply/flow/task/done/list', 'URL', 27, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3021, '考勤异常审核接口', '/api/admin/ding/exception/apply/flow/approve', 'URL', 27, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3022, '考勤异常批量审核接口', '/api/admin/ding/exception/apply/flow/batch/approve', 'URL', 27, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3023, '获取违规延期预警待办接口', '/api/admin/my/work_bench/warn/list', 'URL', 64, 25, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3024, '处理申请', 'Finish', 'LINK', 44, 1, 'AUTH', '3030,3031', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (3025, '项目变更计划申请接口', '/api/admin/project/exchange/save', 'URL', 40, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3026, '项目变更计划列表接口', '/api/admin/project/exchange/list', 'URL', 40, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3027, '流程控件查询接口', '/api/admin/widget/flow/select', 'URL', 64, 26, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3028, '控件查询列表接口', '/api/admin/widget/list', 'URL', 64, 27, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3029, '批量结束流程接口', '/api/admin/flow/batch/delete', 'URL', 64, 28, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3030, '项目变更计划审核接口', '/api/admin/project/exchange/flow/approve', 'URL', 40, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3031, '项目变更计划详情接口', '/api/admin/project/exchange/view', 'URL', 40, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3032, '获取流程待办计数接口', '/api/admin/flow/task/list/count', 'URL', 64, 29, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3033, '获取违规/延期预警待办计数接口', '/api/admin/my/work_bench/warn/list/count', 'URL', 64, 30, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3034, 'sop元数据查询接口', '/api/admin/sop/metadata/list', 'URL', 40, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3035, 'sop元数据修改接口', '/api/admin/sop/metadata/save', 'URL', 40, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3036, '获取流程版本信息接口', '/api/admin/flow/version/list', 'URL', 64, 31, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3037, '编辑', 'Edit', 'LINK', 41, 6, 'AUTH', '3047,3048', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (3038, '作废', 'Cancel', 'LINK', 41, 7, 'AUTH', '3045', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (3039, '编辑', 'Edit', 'LINK', 42, 6, 'AUTH', '3047,3048', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (3040, '作废', 'Cancel', 'LINK', 42, 7, 'AUTH', '3045', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (3041, 'sop申请接口', '/api/admin/sop/apply', 'URL', 40, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3042, 'sop填报接口', '/api/admin/sop/approve', 'URL', 40, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3043, 'sop列表接口', '/api/admin/sop/list', 'URL', 40, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3044, '质量问题反馈审核', '/api/admin/tb/quality/problem/apply/approve', 'URL', 21, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3045, 'sop作废', '/api/admin/sop/cancel', 'URL', 40, 9, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3046, 'sop批量作废', '/api/admin/sop/batch/cancel', 'URL', 40, 10, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3047, 'sop编辑', '/api/admin/sop/edit', 'URL', 40, 11, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3048, 'sop保存 ', '/api/admin/sop/save', 'URL', 40, 12, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3049, '审核', 'Approve', 'LINK', 21, 1, 'AUTH', '1030,3044', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (3050, '获取所有用户（动态表单用）接口', '/api/admin/common/get_user_list/form_widget', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3051, '小程序签名预览', '/api/admin/common/file/preview/app', 'URL', 64, 6, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (3052, '考勤异常详情接口', '/api/admin/ding/exception/apply/view', 'URL', 27, 6, 'SYS', NULL, 1, 1, 0);

INSERT INTO `sys_role` VALUES (1, NULL, '系统管理员', 1, 'ADMIN', NULL, NULL, NULL, NULL, 1, '系统角色');
INSERT INTO `sys_role` VALUES (2, NULL, '总负责人', 1, 'PMO', NULL, NULL, 443437441441857536, 1697425600601, 1, '系统角色');
INSERT INTO `sys_role` VALUES (3, NULL, '业务线负责人', 1, 'BUSSINESS', NULL, NULL, 443437441441857536, 1697425607705, 1, '系统角色');
INSERT INTO `sys_role` VALUES (4, NULL, '大区经理', 1, 'REGION_MANAGER', NULL, NULL, 443437441441857536, 1697425614676, 1, '系统角色');
INSERT INTO `sys_role` VALUES (5, NULL, '区域协调人', 1, 'REGION_COORDINATOR', NULL, NULL, 443437441441857536, 1697425621388, 1, '外包角色');
INSERT INTO `sys_role` VALUES (6, NULL, '实施工程师', 1, 'EFFECT_ENGINEER', NULL, NULL, 443437441441857536, 1697425629355, 1, '外包角色');
INSERT INTO `sys_role` VALUES (7, NULL, '助理工程师', 1, 'ASSISTANT_ENGINEER', NULL, NULL, 443437441441857536, 1697425637315, 1, '外包角色');
INSERT INTO `sys_role` VALUES (8, NULL, 'QA', 1, 'QA', NULL, NULL, 443437441441857536, 1697785940307, 1, '系统角色');
INSERT INTO `sys_role` VALUES (9, NULL, '技术客服', 1, 'CUSTOM', NULL, NULL, 443437441441857536, 1697425672305, 1, '系统角色');
INSERT INTO `sys_role` VALUES (10, NULL, '客户经理', 1, 'ACCOUNT_MANAGER', NULL, NULL, 443437441441857536, 1697425679196, 1, '系统角色');
INSERT INTO `sys_role` VALUES (11, NULL, '乙方总负责人', 1, 'THIRD_PMO', NULL, NULL, 443437441441857536, 1697621767961, 1, '系统角色');
INSERT INTO `sys_role` VALUES (12, NULL, '设备发货员', 1, 'DEVICE_DELIVERY', NULL, NULL, 443437441441857536, 1697621767961, 1, '系统角色');

INSERT INTO `sys_role_privilege` VALUES (446629625196445696, 2, 1024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445697, 2, 1, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445698, 2, 1025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445699, 2, 2, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445700, 2, 1026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445701, 2, 1027, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445702, 2, 2051, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445703, 2, 1028, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445704, 2, 1029, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445705, 2, 1030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445706, 2, 7, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445707, 2, 1031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445708, 2, 1032, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445709, 2, 1033, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445710, 2, 1034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445711, 2, 1035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445712, 2, 1036, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445713, 2, 1037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445714, 2, 1038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445715, 2, 1041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445716, 2, 1042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445717, 2, 19, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445718, 2, 1043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445719, 2, 1044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445720, 2, 20, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445721, 2, 21, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445722, 2, 23, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445723, 2, 24, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445724, 2, 25, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445725, 2, 27, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445726, 2, 28, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445727, 2, 29, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445728, 2, 30, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445729, 2, 31, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445730, 2, 32, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445731, 2, 33, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445732, 2, 34, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445733, 2, 35, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445734, 2, 36, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445735, 2, 37, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445736, 2, 38, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445737, 2, 39, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445738, 2, 40, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445739, 2, 41, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445740, 2, 42, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445741, 2, 43, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445742, 2, 44, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445743, 2, 45, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445744, 2, 46, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445745, 2, 47, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445746, 2, 48, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445747, 2, 49, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445748, 2, 50, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445749, 2, 51, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445750, 2, 52, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445751, 2, 53, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445752, 2, 54, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445753, 2, 55, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445754, 2, 56, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445755, 2, 57, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445756, 2, 93, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445757, 2, 94, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445758, 2, 95, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445759, 2, 96, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445760, 2, 97, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445761, 2, 106, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445762, 2, 127, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445763, 2, 128, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445764, 2, 129, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445765, 2, 130, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445766, 2, 131, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445767, 2, 132, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445768, 2, 133, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445769, 2, 134, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445770, 2, 135, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445771, 2, 136, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445772, 2, 137, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445773, 2, 138, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445774, 2, 139, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445775, 2, 140, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445776, 2, 141, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445777, 2, 142, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445778, 2, 143, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445779, 2, 144, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445780, 2, 145, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445781, 2, 146, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445782, 2, 147, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445783, 2, 148, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445784, 2, 149, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445785, 2, 150, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445786, 2, 151, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445787, 2, 152, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445788, 2, 153, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445789, 2, 154, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445790, 2, 155, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445791, 2, 156, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445792, 2, 157, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445793, 2, 158, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445794, 2, 159, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445795, 2, 160, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445796, 2, 161, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445797, 2, 162, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445798, 2, 163, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445799, 2, 164, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445800, 2, 165, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445801, 2, 166, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445802, 2, 167, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445803, 2, 168, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445804, 2, 169, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445805, 2, 170, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445806, 2, 171, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445807, 2, 172, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445808, 2, 173, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445809, 2, 174, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445810, 2, 175, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445811, 2, 176, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445812, 2, 177, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445813, 2, 178, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445814, 2, 179, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445815, 2, 180, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445816, 2, 181, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445817, 2, 182, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445818, 2, 183, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445819, 2, 184, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445820, 2, 185, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445821, 2, 186, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445822, 2, 187, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445823, 2, 188, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445824, 2, 189, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445825, 2, 190, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445826, 2, 191, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445827, 2, 192, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445828, 2, 193, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445829, 2, 194, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445830, 2, 196, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445831, 2, 197, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445832, 2, 198, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445833, 2, 199, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445834, 2, 200, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445835, 2, 201, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445836, 2, 202, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445837, 2, 203, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445838, 2, 204, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445839, 2, 205, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445840, 2, 206, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445841, 2, 207, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445842, 2, 208, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445843, 2, 209, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445844, 2, 210, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445845, 2, 211, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445846, 2, 212, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445847, 2, 213, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445848, 2, 214, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445849, 2, 215, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445850, 2, 216, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445851, 2, 217, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445852, 2, 218, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445853, 2, 219, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445854, 2, 220, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445855, 2, 221, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445856, 2, 222, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445857, 2, 223, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445858, 2, 224, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445859, 2, 225, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445860, 2, 226, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445861, 2, 227, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445862, 2, 228, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445863, 2, 229, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445864, 2, 230, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445865, 2, 231, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445866, 2, 232, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445867, 2, 233, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445868, 2, 234, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445869, 2, 235, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445870, 2, 236, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445871, 2, 237, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445872, 2, 238, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445873, 2, 277, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445874, 2, 278, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445875, 2, 281, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445876, 2, 282, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445877, 2, 3017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445878, 2, 3019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445879, 2, 3020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445880, 2, 3021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445881, 2, 3022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445882, 2, 3024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445883, 2, 2000, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445884, 2, 2001, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445885, 2, 3025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445886, 2, 2002, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445887, 2, 3026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445888, 2, 2003, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445889, 2, 2004, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445890, 2, 2005, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445891, 2, 2006, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445892, 2, 3030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445893, 2, 2007, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445894, 2, 3031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445895, 2, 2008, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445896, 2, 2009, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445897, 2, 3034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445898, 2, 2010, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445899, 2, 3035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445900, 2, 2011, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445901, 2, 2012, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445902, 2, 3037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445903, 2, 2013, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445904, 2, 3038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445905, 2, 3039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445906, 2, 3040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445907, 2, 2016, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445908, 2, 3041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445909, 2, 2017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445910, 2, 3042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445911, 2, 2018, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445912, 2, 3043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445913, 2, 2019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445914, 2, 3044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445915, 2, 2020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445916, 2, 3045, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445917, 2, 2021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445918, 2, 3046, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445919, 2, 2022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445920, 2, 2023, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445921, 2, 3047, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445922, 2, 3048, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445923, 2, 2024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445924, 2, 3049, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445925, 2, 2025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445926, 2, 2026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445927, 2, 2027, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445928, 2, 2028, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445929, 2, 2029, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445930, 2, 2030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445931, 2, 2031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445932, 2, 2032, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445933, 2, 2033, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445934, 2, 2034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445935, 2, 2035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445936, 2, 2036, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445937, 2, 1013, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445938, 2, 2037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445939, 2, 2038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445940, 2, 1014, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445941, 2, 1015, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445942, 2, 2039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445943, 2, 1016, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445944, 2, 2040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445945, 2, 2041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445946, 2, 1017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445947, 2, 2042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445948, 2, 1018, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445949, 2, 1019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445950, 2, 1020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445951, 2, 1021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445952, 2, 1022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445953, 2, 2047, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445954, 2, 1023, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169920, 3, 1024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169921, 3, 1, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169922, 3, 1025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169923, 3, 1026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169924, 3, 1027, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169925, 3, 2051, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169926, 3, 1028, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169927, 3, 1029, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169928, 3, 1030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169929, 3, 1031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169930, 3, 8, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169931, 3, 9, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169932, 3, 1034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169933, 3, 1041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169934, 3, 1042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169935, 3, 19, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169936, 3, 1043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169937, 3, 1044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169938, 3, 20, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169939, 3, 21, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169940, 3, 23, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169941, 3, 24, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169942, 3, 25, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169943, 3, 27, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169944, 3, 28, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169945, 3, 29, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169946, 3, 30, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169947, 3, 31, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169948, 3, 32, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169949, 3, 33, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169950, 3, 34, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169951, 3, 35, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169952, 3, 36, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169953, 3, 37, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169954, 3, 38, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169955, 3, 39, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169956, 3, 40, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169957, 3, 41, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169958, 3, 42, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169959, 3, 43, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169960, 3, 44, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169961, 3, 45, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169962, 3, 46, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169963, 3, 47, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169964, 3, 48, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169965, 3, 49, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169966, 3, 50, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169967, 3, 52, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169968, 3, 53, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169969, 3, 54, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169970, 3, 55, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169971, 3, 56, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169972, 3, 57, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169973, 3, 98, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169974, 3, 99, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169975, 3, 100, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169976, 3, 101, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169977, 3, 102, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169978, 3, 103, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169979, 3, 104, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169980, 3, 107, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169981, 3, 127, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169982, 3, 128, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169983, 3, 129, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169984, 3, 130, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169985, 3, 131, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169986, 3, 132, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169987, 3, 133, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169988, 3, 134, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169989, 3, 135, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169990, 3, 136, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169991, 3, 137, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169992, 3, 138, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169993, 3, 139, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169994, 3, 140, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169995, 3, 141, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169996, 3, 142, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169997, 3, 143, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169998, 3, 144, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169999, 3, 145, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170000, 3, 146, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170001, 3, 147, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170002, 3, 148, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170003, 3, 149, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170004, 3, 150, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170005, 3, 151, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170006, 3, 152, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170007, 3, 153, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170008, 3, 154, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170009, 3, 155, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170010, 3, 156, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170011, 3, 157, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170012, 3, 158, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170013, 3, 159, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170014, 3, 160, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170015, 3, 161, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170016, 3, 162, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170017, 3, 163, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170018, 3, 164, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170019, 3, 165, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170020, 3, 166, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170021, 3, 167, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170022, 3, 168, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170023, 3, 169, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170024, 3, 170, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170025, 3, 171, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170026, 3, 172, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170027, 3, 173, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170028, 3, 174, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170029, 3, 175, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170030, 3, 176, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170031, 3, 177, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170032, 3, 178, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170033, 3, 179, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170034, 3, 180, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170035, 3, 181, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170036, 3, 182, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170037, 3, 183, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170038, 3, 184, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170039, 3, 185, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170040, 3, 186, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170041, 3, 187, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170042, 3, 188, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170043, 3, 189, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170044, 3, 190, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170045, 3, 191, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170046, 3, 192, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170047, 3, 193, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170048, 3, 194, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170049, 3, 196, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170050, 3, 197, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170051, 3, 198, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170052, 3, 199, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170053, 3, 200, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170054, 3, 201, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170055, 3, 202, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170056, 3, 203, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170057, 3, 204, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170058, 3, 205, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170059, 3, 206, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170060, 3, 207, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170061, 3, 208, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170062, 3, 218, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170063, 3, 219, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170064, 3, 220, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170065, 3, 221, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170066, 3, 222, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170067, 3, 223, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170068, 3, 224, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170069, 3, 225, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170070, 3, 226, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170071, 3, 227, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170072, 3, 228, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170073, 3, 229, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170074, 3, 230, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170075, 3, 231, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170076, 3, 232, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170077, 3, 233, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170078, 3, 234, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170079, 3, 235, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170080, 3, 236, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170081, 3, 237, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170082, 3, 238, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170083, 3, 3017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170084, 3, 3019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170085, 3, 3020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170086, 3, 3021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170087, 3, 3022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170088, 3, 3024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170089, 3, 2000, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170090, 3, 2001, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170091, 3, 3025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170092, 3, 2002, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170093, 3, 3026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170094, 3, 2003, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170095, 3, 2004, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170096, 3, 2005, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170097, 3, 2006, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170098, 3, 3030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170099, 3, 2007, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170100, 3, 3031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170101, 3, 2008, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170102, 3, 2009, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170103, 3, 3034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170104, 3, 2010, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170105, 3, 3035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170106, 3, 2011, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170107, 3, 2012, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170108, 3, 3037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170109, 3, 2013, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170110, 3, 3038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170111, 3, 3039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170112, 3, 3040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170113, 3, 2016, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170114, 3, 3041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170115, 3, 2017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170116, 3, 3042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170117, 3, 2018, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170118, 3, 3043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170119, 3, 2019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170120, 3, 3044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170121, 3, 2020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170122, 3, 3045, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170123, 3, 2021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170124, 3, 3046, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170125, 3, 2022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170126, 3, 2023, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170127, 3, 3047, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170128, 3, 3048, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170129, 3, 2024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170130, 3, 3049, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170131, 3, 2025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170132, 3, 2026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170133, 3, 1002, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170134, 3, 2027, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170135, 3, 1003, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170136, 3, 2028, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170137, 3, 1004, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170138, 3, 2029, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170139, 3, 1005, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170140, 3, 2030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170141, 3, 1006, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170142, 3, 2031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170143, 3, 1007, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170144, 3, 2032, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170145, 3, 1008, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170146, 3, 2033, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170147, 3, 2034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170148, 3, 2035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170149, 3, 2036, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170150, 3, 1013, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170151, 3, 2037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170152, 3, 2038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170153, 3, 1014, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170154, 3, 1015, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170155, 3, 2039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170156, 3, 1016, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170157, 3, 2040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170158, 3, 2041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170159, 3, 1017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170160, 3, 2042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170161, 3, 1018, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170162, 3, 1019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170163, 3, 1020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170164, 3, 1021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170165, 3, 1022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170166, 3, 2047, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170167, 3, 1023, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497280, 4, 1024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497281, 4, 1, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497282, 4, 1025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497283, 4, 1026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497284, 4, 1027, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497285, 4, 1028, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497286, 4, 1029, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497287, 4, 1030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497288, 4, 1031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497289, 4, 8, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497290, 4, 9, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497291, 4, 1041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497292, 4, 1042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497293, 4, 19, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497294, 4, 1043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497295, 4, 1044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497296, 4, 20, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497297, 4, 21, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497298, 4, 24, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497299, 4, 25, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497300, 4, 26, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497301, 4, 27, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497302, 4, 28, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497303, 4, 29, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497304, 4, 30, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497305, 4, 31, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497306, 4, 32, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497307, 4, 34, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497308, 4, 36, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497309, 4, 39, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497310, 4, 40, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497311, 4, 41, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497312, 4, 42, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497313, 4, 43, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497314, 4, 44, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497315, 4, 45, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497316, 4, 47, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497317, 4, 48, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497318, 4, 98, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497319, 4, 99, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497320, 4, 100, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497321, 4, 101, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497322, 4, 102, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497323, 4, 103, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497324, 4, 104, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497325, 4, 107, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497326, 4, 127, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497327, 4, 128, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497328, 4, 129, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497329, 4, 130, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497330, 4, 131, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497331, 4, 132, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497332, 4, 133, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497333, 4, 134, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497334, 4, 135, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497335, 4, 136, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497336, 4, 137, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497337, 4, 138, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497338, 4, 139, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497339, 4, 140, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497340, 4, 141, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497341, 4, 142, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497342, 4, 143, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497343, 4, 144, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497344, 4, 145, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497345, 4, 146, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497346, 4, 147, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497347, 4, 148, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497348, 4, 149, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497349, 4, 150, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497350, 4, 151, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497351, 4, 152, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497352, 4, 161, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497353, 4, 162, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497354, 4, 163, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497355, 4, 164, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497356, 4, 165, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497357, 4, 166, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497358, 4, 171, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497359, 4, 172, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497360, 4, 173, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497361, 4, 174, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497362, 4, 175, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497363, 4, 176, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497364, 4, 177, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497365, 4, 178, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497366, 4, 179, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497367, 4, 180, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497368, 4, 181, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497369, 4, 182, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497370, 4, 183, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497371, 4, 184, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497372, 4, 185, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497373, 4, 186, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497374, 4, 187, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497375, 4, 188, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497376, 4, 189, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497377, 4, 190, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497378, 4, 191, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497379, 4, 192, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497380, 4, 193, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497381, 4, 194, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497382, 4, 196, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497383, 4, 197, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497384, 4, 198, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497385, 4, 199, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497386, 4, 200, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497387, 4, 201, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497388, 4, 3017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497389, 4, 202, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497390, 4, 203, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497391, 4, 3019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497392, 4, 204, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497393, 4, 3020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497394, 4, 205, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497395, 4, 3021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497396, 4, 206, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497397, 4, 3022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497398, 4, 207, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497399, 4, 3024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497400, 4, 208, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497401, 4, 3025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497402, 4, 3026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497403, 4, 3030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497404, 4, 3031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497405, 4, 3034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497406, 4, 3035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497407, 4, 3037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497408, 4, 3038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497409, 4, 3039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497410, 4, 3040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497411, 4, 3041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497412, 4, 3042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497413, 4, 3043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497414, 4, 3044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497415, 4, 3045, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497416, 4, 2021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497417, 4, 3046, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497418, 4, 2022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497419, 4, 2023, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497420, 4, 3047, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497421, 4, 3048, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497422, 4, 2024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497423, 4, 3049, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497424, 4, 2025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497425, 4, 2026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497426, 4, 1002, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497427, 4, 1003, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497428, 4, 2028, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497429, 4, 1004, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497430, 4, 2029, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497431, 4, 1005, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497432, 4, 2030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497433, 4, 1006, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497434, 4, 2031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497435, 4, 1007, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497436, 4, 2032, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497437, 4, 1008, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497438, 4, 2033, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497439, 4, 2034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497440, 4, 2035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497441, 4, 1013, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497442, 4, 2037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497443, 4, 2038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497444, 4, 1014, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497445, 4, 1015, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497446, 4, 2039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497447, 4, 1016, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497448, 4, 2040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497449, 4, 2041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497450, 4, 1017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497451, 4, 2042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497452, 4, 1018, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497453, 4, 1019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497454, 4, 2044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497455, 4, 1020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497456, 4, 1021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497457, 4, 2045, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497458, 4, 2046, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497459, 4, 1022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629684214497460, 4, 1023, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025856, 5, 1024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025857, 5, 1025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025858, 5, 1026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025859, 5, 1027, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025860, 5, 1028, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025861, 5, 1029, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025862, 5, 1030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025863, 5, 1031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025864, 5, 1041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025865, 5, 1042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025866, 5, 19, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025867, 5, 1043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025868, 5, 1044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025869, 5, 20, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025870, 5, 21, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025871, 5, 24, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025872, 5, 25, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025873, 5, 26, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025874, 5, 27, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025875, 5, 28, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025876, 5, 29, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025877, 5, 30, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025878, 5, 39, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025879, 5, 40, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025880, 5, 41, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025881, 5, 42, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025882, 5, 43, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025883, 5, 44, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025884, 5, 45, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025885, 5, 46, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025886, 5, 47, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025887, 5, 48, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025888, 5, 127, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025889, 5, 128, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025890, 5, 129, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025891, 5, 130, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025892, 5, 131, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025893, 5, 132, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025894, 5, 133, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025895, 5, 134, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025896, 5, 135, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025897, 5, 136, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025898, 5, 137, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025899, 5, 138, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025900, 5, 139, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025901, 5, 140, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025902, 5, 141, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025903, 5, 142, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025904, 5, 143, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025905, 5, 144, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025906, 5, 145, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025907, 5, 146, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025908, 5, 147, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025909, 5, 148, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025910, 5, 149, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025911, 5, 150, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025912, 5, 151, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025913, 5, 152, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025914, 5, 171, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025915, 5, 172, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025916, 5, 173, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025917, 5, 174, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025918, 5, 175, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025919, 5, 176, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025920, 5, 177, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025921, 5, 178, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025922, 5, 179, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025923, 5, 180, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025924, 5, 181, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025925, 5, 182, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025926, 5, 183, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025927, 5, 184, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025928, 5, 185, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025929, 5, 186, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025930, 5, 187, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025931, 5, 188, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025932, 5, 189, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025933, 5, 190, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025934, 5, 191, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025935, 5, 192, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025936, 5, 193, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025937, 5, 194, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025938, 5, 196, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025939, 5, 197, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025940, 5, 198, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025941, 5, 199, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025942, 5, 200, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025943, 5, 201, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025944, 5, 3017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025945, 5, 202, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025946, 5, 203, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025947, 5, 3019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025948, 5, 204, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025949, 5, 3020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025950, 5, 205, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025951, 5, 3021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025952, 5, 206, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025953, 5, 3022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025954, 5, 207, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025955, 5, 3024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025956, 5, 208, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025957, 5, 3025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025958, 5, 3026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025959, 5, 3030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025960, 5, 3031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025961, 5, 3034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025962, 5, 3035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025963, 5, 3037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025964, 5, 3038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025965, 5, 3039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025966, 5, 3040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025967, 5, 3041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025968, 5, 3042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025969, 5, 3043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025970, 5, 3044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025971, 5, 3045, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025972, 5, 3046, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025973, 5, 3047, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025974, 5, 3048, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025975, 5, 3049, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025976, 5, 2032, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025977, 5, 2033, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025978, 5, 2034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025979, 5, 2035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025980, 5, 1013, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025981, 5, 2037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025982, 5, 1014, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025983, 5, 2038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025984, 5, 1015, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025985, 5, 2039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025986, 5, 1016, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025987, 5, 2040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025988, 5, 2041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025989, 5, 1017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025990, 5, 2042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025991, 5, 1018, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025992, 5, 1019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025993, 5, 2044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025994, 5, 1020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025995, 5, 1021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025996, 5, 2045, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025997, 5, 2046, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025998, 5, 1022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025999, 5, 1023, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196096, 6, 129, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196097, 6, 130, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196098, 6, 1030, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196099, 6, 1031, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196100, 6, 3025, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196101, 6, 19, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196102, 6, 20, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196103, 6, 21, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196104, 6, 3034, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196105, 6, 3035, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196106, 6, 3042, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196107, 6, 3043, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196108, 6, 3044, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196109, 6, 39, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196110, 6, 40, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196111, 6, 41, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196112, 6, 42, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196113, 6, 171, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196114, 6, 172, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196115, 6, 174, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196116, 6, 175, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196117, 6, 176, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196118, 6, 180, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196119, 6, 181, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196120, 6, 183, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196121, 6, 184, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196122, 6, 185, 1);
INSERT INTO `sys_role_privilege` VALUES (455410212296196123, 6, 127, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073792, 7, 3042, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073793, 7, 3043, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073794, 7, 39, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073795, 7, 40, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073796, 7, 41, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073797, 7, 42, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073798, 7, 171, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073799, 7, 172, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073800, 7, 174, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073801, 7, 175, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073802, 7, 176, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073803, 7, 3025, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073804, 7, 180, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073805, 7, 181, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073806, 7, 183, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073807, 7, 184, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073808, 7, 185, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073809, 7, 3034, 1);
INSERT INTO `sys_role_privilege` VALUES (455409081927073810, 7, 3035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803968, 8, 1024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803969, 8, 1, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803970, 8, 1025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803971, 8, 1029, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803972, 8, 1030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803973, 8, 1031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803974, 8, 8, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803975, 8, 9, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803976, 8, 10, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803977, 8, 1041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803978, 8, 1042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803979, 8, 19, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803980, 8, 1043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803981, 8, 1044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803982, 8, 20, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803983, 8, 21, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803984, 8, 23, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803985, 8, 24, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803986, 8, 25, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803987, 8, 39, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803988, 8, 40, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803989, 8, 41, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803990, 8, 42, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803991, 8, 44, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803992, 8, 45, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803993, 8, 46, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803994, 8, 47, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803995, 8, 48, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803996, 8, 72, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803997, 8, 98, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803998, 8, 99, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803999, 8, 100, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804000, 8, 101, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804001, 8, 102, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804002, 8, 103, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804003, 8, 104, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804004, 8, 107, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804005, 8, 108, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804006, 8, 109, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804007, 8, 110, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804008, 8, 127, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804009, 8, 128, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804010, 8, 129, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804011, 8, 130, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804012, 8, 171, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804013, 8, 172, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804014, 8, 173, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804015, 8, 174, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804016, 8, 175, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804017, 8, 176, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804018, 8, 177, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804019, 8, 178, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804020, 8, 179, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804021, 8, 180, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804022, 8, 181, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804023, 8, 182, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804024, 8, 183, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804025, 8, 184, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804026, 8, 185, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804027, 8, 186, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804028, 8, 187, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804029, 8, 193, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804030, 8, 194, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804031, 8, 196, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804032, 8, 197, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804033, 8, 198, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804034, 8, 199, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804035, 8, 200, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804036, 8, 201, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804037, 8, 202, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804038, 8, 203, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804039, 8, 204, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804040, 8, 205, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804041, 8, 206, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804042, 8, 207, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804043, 8, 3024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804044, 8, 208, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804045, 8, 3025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804046, 8, 3026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804047, 8, 3030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804048, 8, 3031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804049, 8, 3034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804050, 8, 3035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804051, 8, 3037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804052, 8, 3038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804053, 8, 2014, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804054, 8, 3039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804055, 8, 3040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804056, 8, 3041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804057, 8, 3042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804058, 8, 3043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804059, 8, 3044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804060, 8, 3045, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804061, 8, 3046, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804062, 8, 3047, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804063, 8, 3048, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804064, 8, 3049, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804065, 8, 1002, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804066, 8, 1003, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804067, 8, 1004, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804068, 8, 1005, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804069, 8, 1006, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804070, 8, 239, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804071, 8, 1007, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804072, 8, 240, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804073, 8, 1008, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804074, 8, 241, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804075, 8, 242, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804076, 8, 243, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804077, 8, 244, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804078, 8, 245, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804079, 8, 1013, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804080, 8, 1014, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804081, 8, 1015, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804082, 8, 1016, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804083, 8, 1017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804084, 8, 1018, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804085, 8, 1019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804086, 8, 1020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804087, 8, 1021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804088, 8, 1022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804089, 8, 1023, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459584, 9, 240, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459585, 9, 1, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459586, 9, 241, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459587, 9, 242, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459588, 9, 243, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459589, 9, 244, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459590, 9, 245, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459591, 9, 72, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459592, 9, 239, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214144, 10, 240, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214145, 10, 1, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214146, 10, 241, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214147, 10, 242, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214148, 10, 243, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214149, 10, 244, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214150, 10, 245, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214151, 10, 72, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214152, 10, 239, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717568, 11, 1024, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717569, 11, 1025, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717570, 11, 1026, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717571, 11, 1027, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717572, 11, 1028, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717573, 11, 1029, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717574, 11, 1030, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717575, 11, 1031, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717576, 11, 1041, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717577, 11, 1042, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717578, 11, 19, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717579, 11, 1043, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717580, 11, 1044, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717581, 11, 20, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717582, 11, 21, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717583, 11, 23, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717584, 11, 24, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717585, 11, 25, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717586, 11, 26, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717587, 11, 27, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717588, 11, 28, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717589, 11, 29, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717590, 11, 30, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717591, 11, 39, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717592, 11, 40, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717593, 11, 41, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717594, 11, 42, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717595, 11, 43, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717596, 11, 44, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717597, 11, 45, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717598, 11, 46, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717599, 11, 47, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717600, 11, 48, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717601, 11, 127, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717602, 11, 128, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717603, 11, 129, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717604, 11, 130, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717605, 11, 131, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717606, 11, 132, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717607, 11, 133, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717608, 11, 134, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717609, 11, 135, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717610, 11, 136, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717611, 11, 137, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717612, 11, 138, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717613, 11, 139, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717614, 11, 140, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717615, 11, 141, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717616, 11, 142, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717617, 11, 143, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717618, 11, 144, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717619, 11, 145, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717620, 11, 146, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717621, 11, 147, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717622, 11, 148, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717623, 11, 149, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717624, 11, 150, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717625, 11, 151, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717626, 11, 152, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717627, 11, 171, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717628, 11, 172, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717629, 11, 173, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717630, 11, 174, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717631, 11, 175, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717632, 11, 176, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717633, 11, 177, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717634, 11, 178, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717635, 11, 179, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717636, 11, 180, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717637, 11, 181, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717638, 11, 182, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717639, 11, 183, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717640, 11, 184, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717641, 11, 185, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717642, 11, 186, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717643, 11, 187, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717644, 11, 188, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717645, 11, 189, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717646, 11, 190, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717647, 11, 191, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717648, 11, 192, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717649, 11, 193, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717650, 11, 194, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717651, 11, 196, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717652, 11, 197, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717653, 11, 198, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717654, 11, 199, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717655, 11, 200, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717656, 11, 201, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717657, 11, 3017, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717658, 11, 202, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717659, 11, 203, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717660, 11, 3019, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717661, 11, 204, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717662, 11, 3020, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717663, 11, 205, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717664, 11, 3021, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717665, 11, 206, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717666, 11, 3022, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717667, 11, 207, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717668, 11, 3024, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717669, 11, 208, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717670, 11, 3025, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717671, 11, 3026, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717672, 11, 3030, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717673, 11, 3031, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717674, 11, 3034, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717675, 11, 3035, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717676, 11, 3037, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717677, 11, 3038, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717678, 11, 3039, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717679, 11, 3040, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717680, 11, 3041, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717681, 11, 3042, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717682, 11, 3043, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717683, 11, 3044, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717684, 11, 3045, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717685, 11, 3046, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717686, 11, 3047, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717687, 11, 3048, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717688, 11, 3049, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717689, 11, 2032, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717690, 11, 2033, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717691, 11, 2034, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717692, 11, 2035, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717693, 11, 1013, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717694, 11, 2037, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717695, 11, 1014, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717696, 11, 2038, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717697, 11, 1015, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717698, 11, 2039, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717699, 11, 1016, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717700, 11, 2040, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717701, 11, 2041, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717702, 11, 1017, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717703, 11, 2042, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717704, 11, 1018, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717705, 11, 1019, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717706, 11, 2044, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717707, 11, 1020, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717708, 11, 1021, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717709, 11, 2045, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717710, 11, 2046, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717711, 11, 1022, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717712, 11, 1023, 1);

INSERT INTO `sys_user` VALUES (1, 'sysadmin', 'sysadmin', NULL, 'MTIzNDU2', '0123456789*', NULL, NULL, 1, 1, 1629353956868, NULL, 'SYSTEM', NULL, NULL, NULL, 1, 1692077912643);

INSERT INTO `sys_user_role` VALUES (1, 1, 1, NULL, 1);

INSERT INTO `t_d_form_widget` VALUES (1, 'TEXT', 'FORM', 'problem_point', 'problem_point', '问题简要', 'STRING', 1, 0, 1, 1, NULL, 30, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (2, 'TEXTAREA', 'FORM', 'problem_remark', 'problem_remark', '问题情况说明', 'STRING', 1, 0, 1, 1, NULL, 300, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (3, 'FILE', 'FORM', 'attachments_info', 'attachments_info', '附件说明', 'FILE', 0, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (4, 'TEXT', 'FORM', 'verify_point', 'verify_point', '核实情况简要', 'STRING', 1, 0, 1, 1, NULL, 50, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (5, 'TEXTAREA', 'FORM', 'verify_remark', 'verify_remark', '核实情况备注', 'STRING', 1, 0, 1, 1, NULL, 500, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (6, 'RADIO', 'FORM', 'problem_type', 'problem_type', '质量问题类型', 'ENUM', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"UPDATE\",\"label\":\"修正类\"},{\"value\":\"OPTIMIZE\",\"label\":\"优化类\"},{\"value\":\"NO_PROBLEM\",\"label\":\"不是问题\"}]', 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (7, 'RADIO', 'FORM', 'problem_reason_radio', 'problem_reason_radio', '问题归因', 'ENUM', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"EXEC\",\"label\":\"执行类\"},{\"value\":\"MANAGER\",\"label\":\"管理协调类\"},{\"value\":\"PRODUCT_PROBLEM\",\"label\":\"产品缺陷类\"},{\"value\":\"PRODUCT_MANAGER\",\"label\":\"产品运维类\"},{\"value\":\"FLOW\",\"label\":\"流程制度类\"},{\"value\":\"OTHER\",\"label\":\"其它\"}]', 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (8, 'RADIO', 'FORM', 'problem_degree_radio', 'problem_degree_radio', '影响度', 'ENUM', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"A\",\"label\":\"A级\"},{\"value\":\"B\",\"label\":\"B级\"},{\"value\":\"C\",\"label\":\"C级\"},{\"value\":\"D\",\"label\":\"D级\"}]', 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (9, 'MULTIPLE_SELECT', 'FORM', 'problem_users', 'problem_users', '责任人', 'ARRAY', 0, 0, 1, 1, NULL, NULL, NULL, '/api/admin/common/get_user_list/form_widget?userSource=ALL', NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (10, 'MULTIPLE_SELECT', 'FORM', 'approve_check_users', 'approve_check_users', '甲方复审人员', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, '/api/admin/common/get_user_list/form_widget?userSource=SYSTEM', NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (11, 'MULTIPLE_SELECT', 'FORM', 'approve_check_third_users', 'approve_check_third_users', '乙方复审人员', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, '/api/admin/common/get_user_list/form_widget?userSource=SYSTEM', NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (12, 'RADIO', 'FORM', 'approve_radio', 'approve_radio', '审批意见', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"同意\"},{\"value\":\"0\",\"label\":\"不同意\"}]', 'QUALITY_PROBLEM_FLOW', 3, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (13, 'SIGN', 'FORM', 'sign', 'sign', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 3, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (14, 'TEXTAREA', 'FORM', 'approve_remark', 'approve_remark', '备注', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 3, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (15, 'RADIO', 'FORM', 'approve_radio', 'approve_radio', '审批意见', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"同意\"},{\"value\":\"0\",\"label\":\"不同意\"}]', 'QUALITY_PROBLEM_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (16, 'SIGN', 'FORM', 'sign', 'sign', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (17, 'TEXTAREA', 'FORM', 'approve_remark', 'approve_remark', '备注', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (18, 'FORM_GROUP_TITLE', 'FORM', 'project_basic_info_title', 'project_basic_info_title', '项目基本信息', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (19, 'NUMBER', 'FORM', 'course_count', 'course_count', '科目数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 2);
INSERT INTO `t_d_form_widget` VALUES (20, 'NUMBER', 'FORM', 'exam_student_course_count', 'exam_student_course_count', '考生科次', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` VALUES (21, 'NUMBER', 'FORM', 'scan_count', 'scan_count', '扫描仪数量(台)', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 4);
INSERT INTO `t_d_form_widget` VALUES (22, 'NUMBER', 'FORM', 'week_people_day', 'week_people_day', '现场标准服务周期（人天）', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 5);
INSERT INTO `t_d_form_widget` VALUES (23, 'DATE', 'FORM', 'exam_student_Internal_date', 'exam_student_Internal_date', '考生数据内部交接时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 6);
INSERT INTO `t_d_form_widget` VALUES (24, 'DATE', 'FORM', 'exam_plan_date', 'exam_plan_date', '考试时间安排表获取时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 7);
INSERT INTO `t_d_form_widget` VALUES (25, 'DATE', 'FORM', 'project_key_info_date', 'project_key_info_date', '项目关键信息提交截止时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 1, NULL, NULL, 1, 8);
INSERT INTO `t_d_form_widget` VALUES (26, 'DATE', 'FORM', 'service_finish_plan_date', 'service_finish_plan_date', '现场服务完成撤场计划时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 1, NULL, NULL, 1, 10);
INSERT INTO `t_d_form_widget` VALUES (27, 'FORM_GROUP_TITLE', 'FORM', 'project_people_plan_title', 'project_people_plan_title', '项目人员安排及风险预估', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 11);
INSERT INTO `t_d_form_widget` VALUES (28, 'SINGLE_SELECT', 'FORM', 'region_user_id', 'region_user_id', '区域协调人', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 4, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 12);
INSERT INTO `t_d_form_widget` VALUES (29, 'SINGLE_SELECT', 'FORM', 'engineer_user_id', 'engineer_user_id', '实施工程师', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 4, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 13);
INSERT INTO `t_d_form_widget` VALUES (30, 'MULTIPLE_SELECT', 'FORM', 'assistant_engineer_user_id', 'assistant_engineer_user_id', '助理实施工程师', 'ARRAY', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 4, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 14);
INSERT INTO `t_d_form_widget` VALUES (31, 'ONLY_TITLE', 'FORM', 'project_rish_title', 'project_rish_title', '项目风险预估（仅供参考）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 15);
INSERT INTO `t_d_form_widget` VALUES (32, 'RADIO', 'FORM', 'delay_rish_cb', 'delay_rish_cb', '延期风险', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"LOW\",\"label\":\"低\"},{\"value\":\"MIDDLE\",\"label\":\"中\"},{\"value\":\"HIGH\",\"label\":\"高\"}]', 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 16);
INSERT INTO `t_d_form_widget` VALUES (33, 'RADIO', 'FORM', 'engineer_rish_cb', 'engineer_rish_cb', '实施难度', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"LOW\",\"label\":\"低\"},{\"value\":\"MIDDLE\",\"label\":\"中\"},{\"value\":\"HIGH\",\"label\":\"高\"}]', 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 17);
INSERT INTO `t_d_form_widget` VALUES (34, 'TEXTAREA', 'FORM', 'other_remark', 'other_remark', '其它备注（建议关注的其它方面）', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 18);
INSERT INTO `t_d_form_widget` VALUES (35, 'FORM_GROUP_TITLE', 'FORM', 'project_contacts_title', 'project_contacts_title', '项目联系人', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 19);
INSERT INTO `t_d_form_widget` VALUES (36, 'TABLE', 'FORM', 'project_contacts_table', 'project_contacts_table', '项目联系人表格', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 20);
INSERT INTO `t_d_form_widget` VALUES (37, 'TEXT', 'TABLE', 'school_college', 'school_college', '项目联系人表格_机构输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 21);
INSERT INTO `t_d_form_widget` VALUES (38, 'TEXT', 'TABLE', 'school_name', 'school_name', '项目联系人表格_姓名输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 22);
INSERT INTO `t_d_form_widget` VALUES (39, 'TEXT', 'TABLE', 'school_job', 'school_job', '项目联系人表格_职务输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 23);
INSERT INTO `t_d_form_widget` VALUES (40, 'TEXT', 'TABLE', 'school_tel', 'school_tel', '项目联系人表格_电话输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 24);
INSERT INTO `t_d_form_widget` VALUES (41, 'FORM_GROUP_TITLE', 'FORM', 'cloud_mark_title', 'cloud_mark_title', '云阅卷（填写前请先认真查阅移交的项目初审及项目基本信息）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (42, 'RADIO', 'FORM', 'deploy_way_radio', 'deploy_way_radio', '部署方式', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"ONLINE\",\"label\":\"线上部署\"},{\"value\":\"LOCAL\",\"label\":\"本地部署\"}]', 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (43, 'RADIO', 'FORM', 'mark_way_radio', 'mark_way_radio', '阅卷方式', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"TOGETHER\",\"label\":\"集中阅卷\"},{\"value\":\"ANYTIME\",\"label\":\"分散阅卷\"}]', 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (44, 'RADIO', 'FORM', 'scan_net_radio', 'scan_net_radio', '扫描场地网络情况', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"INTERNET\",\"label\":\"提供外网\"},{\"value\":\"LOCALHOST\",\"label\":\"不提供外网\"}]', 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (45, 'DATE', 'FORM', 'scan_start_time', 'scan_start_time', '扫描开始时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (46, 'DATE', 'FORM', 'scan_end_time', 'scan_end_time', '扫描结束时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (47, 'DATE', 'FORM', 'mark_start_time', 'mark_start_time', '评卷开始时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (48, 'DATE', 'FORM', 'mark_end_time', 'mark_end_time', '评卷结束时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (49, 'TEXTAREA', 'FORM', 'remark', 'remark', '其它特殊要求', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (50, 'FILE', 'FORM', 'exam_plan_photos', 'exam_plan_photos', '考试时间安排表拍照上传', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (51, 'FORM_GROUP_TITLE', 'FORM', 'inside_title_region', 'inside_title_region', '大区经理项目内审（请于24小时内完成内审）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (52, 'RADIO', 'FORM', 'approve_radio_region', 'approve_radio_region', '我对以上项目关键信息已审核，确认内容无误：', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"同意\"},{\"value\":\"0\",\"label\":\"不同意\"}]', 'OFFICE_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (53, 'SIGN', 'FORM', 'sign_region', 'sign_region', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (54, 'TEXTAREA', 'FORM', 'approve_remark_region', 'approve_remark_region', '审批意见', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (55, 'ONLY_TITLE', 'FORM', 'device_in_out_label', 'device_in_out_label', '设备出库', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (56, 'DATE', 'FORM', 'device_out_time', 'device_out_time', '设备出库时间', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (57, 'DEVICE_OUT_TABLE', 'FORM', 'device_out_table', 'device_out_table', '设备出库表格', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (58, 'FORM_GROUP_TITLE', 'FORM', 'import_data_title', 'import_data_title', '导入数据', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (59, 'CHECKBOX', 'FORM', 'data_same', 'data_same', '导入数据', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"true\",\"label\":\"导入数据与原始数据是否一致\"}]', 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (60, 'TEXT', 'FORM', 'import_exam_student_count', 'import_exam_student_count', '导入考生数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (61, 'FORM_GROUP_TITLE', 'FORM', 'scan_set_title', 'scan_set_title', '扫描仪设置', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (62, 'CHECKBOX', 'FORM', 'scan_set_cb', 'scan_set_cb', '扫描仪设置', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、扫描仪分辨率设置是否150、扫描模式是否双面、图像类型是否灰度\"},{\"value\":\"2\",\"label\":\"2、选择“扫描模式”-“设置”-“装订边缘”-“上”\"}]', 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (63, 'FORM_GROUP_TITLE', 'FORM', 'card_format_title', 'card_format_title', '卡格式制作', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (64, 'CHECKBOX', 'FORM', 'card_format_cb', 'card_format_cb', '卡格式制作', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、定位点、校验点是否制作\"},{\"value\":\"2\",\"label\":\"2、缺考区域是否框选\"},{\"value\":\"3\",\"label\":\"3、客观题区域是否正确框选、行列以及单选多选是否正确\"},{\"value\":\"4\",\"label\":\"4、条码识别区是否制作\"},{\"value\":\"5\",\"label\":\"5、屏蔽区是否制作\"},{\"value\":\"6\",\"label\":\"6、卷型：若有，则框选\"},{\"value\":\"7\",\"label\":\"7、页码：若答题卡超过1张，则框选\"}]', 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (65, 'FORM_GROUP_TITLE', 'FORM', 'scan_test_title', 'scan_test_title', '试扫检查', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (66, 'CHECKBOX', 'FORM', 'scan_test_cb', 'scan_test_cb', '试扫检查', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、查看条码是否识别正确\"},{\"value\":\"2\",\"label\":\"2、缺考、客观题是否识别正确\"},{\"value\":\"3\",\"label\":\"3、扫描原图正反面是否正确显示\"},{\"value\":\"4\",\"label\":\"4、裁切图屏蔽区是否正确\"},{\"value\":\"5\",\"label\":\"5、卷型：若有，是否识别正确\"},{\"value\":\"6\",\"label\":\"6、页码：若答题卡超过1张，是否页码识别正确\"}]', 'OFFICE_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (67, 'FORM_GROUP_TITLE', 'FORM', 'un_upload_exam_student_title', 'un_upload_exam_student_title', '未上传考生核对', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (68, 'CHECKBOX', 'FORM', 'un_upload_exam_student_cb', 'un_upload_exam_student_cb', '未上传考生核对', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 9, NULL, '[{\"value\":\"1\",\"label\":\"1、未上传考生是否与签到表进行核对，是否为缺考、违纪、免考等情况\"},{\"value\":\"2\",\"label\":\"2、是否将未上传考生发给学校确认\"}]', 'OFFICE_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (69, 'TEXT', 'FORM', 'finish_count', 'finish_count', '完成数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'OFFICE_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (70, 'FILE', 'FORM', 'upload_file_un_upload_exam_student_photo', 'upload_file_un_upload_exam_student_photo', '上传学校未上传考生沟通确认截图', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (71, 'FORM_GROUP_TITLE', 'FORM', 'Identify_title', 'Identify_title', '识别对照', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (72, 'CHECKBOX', 'FORM', 'Identify_cb', 'Identify_cb', '识别对照', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"true\",\"label\":\"是否将客观题识别异常的全部人工修改处理\"}]', 'OFFICE_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (73, 'FORM_GROUP_TITLE', 'FORM', 'absent_exam_student_title', 'absent_exam_student_title', '缺考名单比对', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (74, 'CHECKBOX', 'FORM', 'absent_exam_student_cb', 'absent_exam_student_cb', '缺考名单比对', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"true\",\"label\":\"核对未上传条数是否与学校提供的缺考名单吻合，人工指定为缺考（考生管理>导入名单>导入缺考名单）\"}]', 'OFFICE_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (75, 'TEXT', 'FORM', 'absent_count', 'absent_count', '缺考科次数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'OFFICE_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (76, 'FORM_GROUP_TITLE', 'FORM', 'manual_code_title', 'manual_code_title', '手工输入条码确认', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (77, 'CHECKBOX', 'FORM', 'manual_code_cb', 'manual_code_cb', '手工输入条码确认', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"true\",\"label\":\"“数据检查“\' -“人工确认“，“确认完成“\"}]', 'OFFICE_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (78, 'FORM_GROUP_TITLE', 'FORM', 'mark_model_title', 'mark_model_title', '评卷模式确认', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (79, 'RADIO', 'FORM', 'mark_model_radio', 'mark_model_radio', '评卷模式', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"TRAIL\",\"label\":\"轨迹模式\"},{\"value\":\"NORMAL\",\"label\":\"普通模式\"}]', 'OFFICE_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (80, 'FORM_GROUP_TITLE', 'FORM', 'mark_params_title', 'mark_params_title', '评卷参数核对', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (81, 'CHECKBOX', 'FORM', 'mark_params_cb', 'mark_params_cb', '评卷参数核对', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、核查每个科目的满分是否正确（大部分科目都是100分）\"},{\"value\":\"2\",\"label\":\"2、核对客观题标答以及分数是否录入正确、多选题判分规则（漏选是否得分、任选得分）\"},{\"value\":\"3\",\"label\":\"检查所有科目结构和分组是否全部导入。（评卷管理>评卷进度查看分组状态）\"}]', 'OFFICE_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (82, 'FORM_GROUP_TITLE', 'FORM', 'mark_submit_title', 'mark_submit_title', '成绩提交核查', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (83, 'CHECKBOX', 'FORM', 'mark_submit_cb', 'mark_submit_cb', '成绩提交核查', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、是否已核对不缺考、客观题为0分的情况\"},{\"value\":\"2\",\"label\":\"2、是否已核对主观题为0、客观题有分的情况\"},{\"value\":\"3\",\"label\":\"3、是否检查客观题小题得分率低于20%的情况\"},{\"value\":\"4\",\"label\":\"4、核对导出成绩表数据条数是否与考生表数量一致，不能带有“未导出全量数据”字样；请填写导出成绩数量以及完成时间\"},{\"value\":\"5\",\"label\":\"5、核对导出图片工具是否正常运行\"}]', 'OFFICE_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (84, 'FILE', 'FORM', 'upload_success_photos', 'upload_success_photos', '上传验收报告(纸质)拍照', 'ARRAY', 0, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (85, 'ONLY_TITLE', 'FORM', 'device_in_out_label', 'device_in_out_label', '设备入库', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'OFFICE_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (86, 'DATE', 'FORM', 'device_in_time', 'device_in_time', '设备入库时间', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (87, 'DEVICE_IN_TABLE', 'FORM', 'device_in_table', 'device_in_table', '设备入库表格', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (88, 'FORM_GROUP_TITLE', 'FORM', 'inside_title_engineer', 'inside_title_engineer', '工程师项目内审（请于24小时内完成内审）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (89, 'RADIO', 'FORM', 'approve_radio_engineer', 'approve_radio_engineer', '我对以上项目关键信息已审核，确认内容无误：', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"同意\"},{\"value\":\"0\",\"label\":\"不同意\"}]', 'OFFICE_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (90, 'SIGN', 'FORM', 'sign_engineer', 'sign_engineer', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (91, 'TEXTAREA', 'FORM', 'approve_remark_engineer', 'approve_remark_engineer', '审批意见', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (92, 'FORM_GROUP_TITLE', 'FORM', 'project_basic_info_title', 'project_basic_info_title', '项目基本信息', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (93, 'NUMBER', 'FORM', 'course_count', 'course_count', '科目数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 2);
INSERT INTO `t_d_form_widget` VALUES (94, 'NUMBER', 'FORM', 'exam_student_course_count', 'exam_student_course_count', '考生科次', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` VALUES (95, 'RADIO', 'FORM', 'bar_code_paper_count', 'bar_code_paper_count', '条码标准张数', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"3\",\"label\":\"3张\"},{\"value\":\"4\",\"label\":\"4张\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 4);
INSERT INTO `t_d_form_widget` VALUES (96, 'RADIO_WITH_INPUT', 'FORM', 'bar_code_paste_way', 'bar_code_paste_way', '条码粘贴方式', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"EXAM_STUDENT\",\"label\":\"考生自贴/数码印刷\"},{\"value\":\"PAPER\",\"label\":\"回卷后粘贴\"},{\"value\":\"OTHER\",\"label\":\"其它\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 5);
INSERT INTO `t_d_form_widget` VALUES (97, 'RADIO', 'FORM', 'has_objective_card', 'has_objective_card', '有无客观题卡', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"HAVA\",\"label\":\"有\"},{\"value\":\"NOT_HAVA\",\"label\":\"无\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 6);
INSERT INTO `t_d_form_widget` VALUES (98, 'NUMBER', 'FORM', 'scan_count', 'scan_count', '扫描仪数量(台)', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 7);
INSERT INTO `t_d_form_widget` VALUES (99, 'DATE', 'FORM', 'exam_student_Internal_date', 'exam_student_Internal_date', '考生数据内部交接时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 8);
INSERT INTO `t_d_form_widget` VALUES (100, 'NUMBER', 'FORM', 'week_people_day', 'week_people_day', '现场标准服务周期（人天）', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 9);
INSERT INTO `t_d_form_widget` VALUES (101, 'DATE', 'FORM', 'project_key_info_date', 'project_key_info_date', '项目关键信息提交截止时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 1, NULL, NULL, 1, 10);
INSERT INTO `t_d_form_widget` VALUES (102, 'DATE', 'FORM', 'service_finish_plan_date', 'service_finish_plan_date', '现场服务完成撤场计划时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 1, NULL, NULL, 1, 12);
INSERT INTO `t_d_form_widget` VALUES (103, 'FORM_GROUP_TITLE', 'FORM', 'project_people_plan_title', 'project_people_plan_title', '项目人员安排及风险预估', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 13);
INSERT INTO `t_d_form_widget` VALUES (104, 'SINGLE_SELECT', 'FORM', 'region_user_id', 'region_user_id', '区域协调人', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 4, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 14);
INSERT INTO `t_d_form_widget` VALUES (105, 'SINGLE_SELECT', 'FORM', 'engineer_user_id', 'engineer_user_id', '实施工程师', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 4, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 15);
INSERT INTO `t_d_form_widget` VALUES (106, 'MULTIPLE_SELECT', 'FORM', 'assistant_engineer_user_id', 'assistant_engineer_user_id', '助理实施工程师', 'ARRAY', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 4, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 16);
INSERT INTO `t_d_form_widget` VALUES (107, 'ONLY_TITLE', 'FORM', 'project_rish_title', 'project_rish_title', '项目风险预估（仅供参考）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 17);
INSERT INTO `t_d_form_widget` VALUES (108, 'RADIO', 'FORM', 'delay_rish_cb', 'delay_rish_cb', '延期风险', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"LOW\",\"label\":\"低\"},{\"value\":\"MIDDLE\",\"label\":\"中\"},{\"value\":\"HIGH\",\"label\":\"高\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 18);
INSERT INTO `t_d_form_widget` VALUES (109, 'RADIO', 'FORM', 'engineer_rish_cb', 'engineer_rish_cb', '实施难度', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"LOW\",\"label\":\"低\"},{\"value\":\"MIDDLE\",\"label\":\"中\"},{\"value\":\"HIGH\",\"label\":\"高\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 19);
INSERT INTO `t_d_form_widget` VALUES (110, 'TEXTAREA', 'FORM', 'other_remark', 'other_remark', '其它备注（建议关注的其它方面）', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 20);
INSERT INTO `t_d_form_widget` VALUES (111, 'FORM_GROUP_TITLE', 'FORM', 'project_contacts_title', 'project_contacts_title', '项目联系人', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 21);
INSERT INTO `t_d_form_widget` VALUES (112, 'TABLE', 'FORM', 'project_contacts_table', 'project_contacts_table', '项目联系人表格', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 22);
INSERT INTO `t_d_form_widget` VALUES (113, 'TEXT', 'TABLE', 'school_college', 'school_college', '项目联系人表格_机构输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 23);
INSERT INTO `t_d_form_widget` VALUES (114, 'TEXT', 'TABLE', 'school_name', 'school_name', '项目联系人表格_姓名输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 24);
INSERT INTO `t_d_form_widget` VALUES (115, 'TEXT', 'TABLE', 'school_job', 'school_job', '项目联系人表格_职务输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 25);
INSERT INTO `t_d_form_widget` VALUES (116, 'TEXT', 'TABLE', 'school_tel', 'school_tel', '项目联系人表格_电话输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 26);
INSERT INTO `t_d_form_widget` VALUES (117, 'FORM_GROUP_TITLE', 'FORM', 'cloud_mark_title', 'cloud_mark_title', '云阅卷（填写前请先认真查阅移交的项目初审及项目基本信息）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (118, 'RADIO', 'FORM', 'run_env_accord', 'run_env_accord', '运行环境调研是否符合要求', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"ACCORD\",\"label\":\"符合\"},{\"value\":\"UN_ACCORD\",\"label\":\"不符合\"}]', 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (119, 'DATE', 'FORM', 'env_deployment_time', 'env_deployment_time', '环境部署时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (120, 'LABEL', 'FORM', 'env_list_label', 'env_list_label', '[{\"label\":\"参照《软硬件准备检查清单》对学校进行调研\",\"color\":\"blue\"}]', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (121, 'DATE', 'FORM', 'scan_start_time', 'scan_start_time', '扫描开始时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (122, 'DATE', 'FORM', 'scan_end_time', 'scan_end_time', '扫描结束时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (123, 'LABEL', 'FORM', 'unite_label', 'unite_label', '[{\"label\":\"统一元旦后安排进场\",\"color\":\"blue\"}]', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (124, 'DATE', 'FORM', 'mark_room_check_time', 'mark_room_check_time', '评卷机房检查时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (125, 'TEXT', 'FORM', 'mark_room_count', 'mark_room_count', '评卷机房数量（个）', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (126, 'TEXT', 'FORM', 'max_mark_teacher_count', 'max_mark_teacher_count', '最大评卷教师数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (127, 'DATE', 'FORM', 'mark_param_time', 'mark_param_time', '评卷参数提供时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (128, 'DATE', 'FORM', 'mark_start_time', 'mark_start_time', '评卷开始时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (129, 'DATE', 'FORM', 'mark_end_time', 'mark_end_time', '评卷结束时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (130, 'TEXT', 'FORM', 'choose_course_count', 'choose_course_count', '有选做题的科目数量（门）', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (131, 'TEXTAREA', 'FORM', 'ramark', 'ramark', '其它特殊要求', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (132, 'FILE', 'FORM', 'project_info_photos', 'project_info_photos', '上传项目关键信息表(纸质)拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (133, 'FORM_GROUP_TITLE', 'FORM', 'inside_title_region', 'inside_title_region', '大区经理项目内审（请于24小时内完成内审）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (134, 'RADIO', 'FORM', 'approve_radio_region', 'approve_radio_region', '我对以上项目关键信息已审核，确认内容无误：', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"同意\"},{\"value\":\"0\",\"label\":\"不同意\"}]', 'CLOUD_MARK_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (135, 'SIGN', 'FORM', 'sign_region', 'sign_region', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (136, 'TEXTAREA', 'FORM', 'approve_ramark_region', 'approve_ramark_region', '审批意见', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (137, 'FORM_GROUP_TITLE', 'FORM', 'inside_title_engineer', 'inside_title_engineer', '工程师项目内审（请于24小时内完成内审）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (138, 'RADIO', 'FORM', 'approve_radio_engineer', 'approve_radio_engineer', '我对以上项目关键信息已审核，确认内容无误：', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"同意\"},{\"value\":\"0\",\"label\":\"不同意\"}]', 'CLOUD_MARK_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (139, 'SIGN', 'FORM', 'sign_engineer', 'sign_engineer', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (140, 'TEXTAREA', 'FORM', 'approve_ramark_engineer', 'approve_ramark_engineer', '审批意见', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` VALUES (141, 'FORM_GROUP_TITLE', 'FORM', 'upload_list_title', 'upload_list_title', '上传清单和报告', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (142, 'FILE', 'FORM', 'upload_env_photos', 'upload_env_photos', '上传《软硬件准备检查清单》拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (143, 'FILE', 'FORM', 'upload_env_test_photos', 'upload_env_test_photos', '上传《现场环境测试报告》拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (144, 'ONLY_TITLE', 'FORM', 'device_in_out_label', 'device_in_out_label', '设备出库', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (145, 'DATE', 'FORM', 'device_out_time', 'device_out_time', '设备出库时间', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (146, 'DEVICE_OUT_TABLE', 'FORM', 'device_out_table', 'device_out_table', '设备出库表格', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (147, 'FORM_GROUP_TITLE', 'FORM', 'import_data_title', 'import_data_title', '导入数据', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (148, 'CHECKBOX', 'FORM', 'data_same', 'data_same', '导入数据', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"true\",\"label\":\"导入数据与原始数据是否一致\"}]', 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (149, 'TEXT', 'FORM', 'import_exam_student_count', 'import_exam_student_count', '导入考生数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (150, 'FORM_GROUP_TITLE', 'FORM', 'scan_set_title', 'scan_set_title', '扫描仪设置', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (151, 'CHECKBOX', 'FORM', 'scan_set_cb', 'scan_set_cb', '扫描仪设置', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、扫描仪分辨率设置是150\"},{\"value\":\"2\",\"label\":\"2、是否勾选“自动剪切”，“纠正歪斜”\"},{\"value\":\"3\",\"label\":\"3、是否勾选“边界去除”\"},{\"value\":\"4\",\"label\":\"4、勾选“双张进纸检测”-“有效”-动作“停止”\"},{\"value\":\"5\",\"label\":\"5、爱普生型号，选择“扫描模式”-“设置”-“装订边缘”-“上”\"},{\"value\":\"6\",\"label\":\"6、华高扫描仪型号，检查背面图像；如背面图像倒转，则勾选“背面旋转180°”\"}]', 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (152, 'FORM_GROUP_TITLE', 'FORM', 'card_format_title', 'card_format_title', '卡格式制作', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (153, 'CHECKBOX', 'FORM', 'card_format_cb', 'card_format_cb', '卡格式制作', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、条码遮挡区、考生信息遮挡区需沿装订线框选\"},{\"value\":\"2\",\"label\":\"2、扫描原图正反面检查是否正确\"},{\"value\":\"3\",\"label\":\"3、扫描裁切图屏蔽区检查是否正确\"}]', 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (154, 'FORM_GROUP_TITLE', 'FORM', 'paper_list_title', 'paper_list_title', '纸质检查表上传', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (155, 'FILE', 'FORM', 'paper_check_photos', 'paper_check_photos', '上传关键节点检查(纸质)表单拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (156, 'FORM_GROUP_TITLE', 'FORM', 'manual_bar_code_title', 'manual_bar_code_title', '手工输入条码确认', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (157, 'CHECKBOX', 'FORM', 'manual_bar_code_cb', 'manual_bar_code_cb', '手工输入条码确认', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"1\",\"label\":\"“数据检查”-“人工确认”，”确认已完成”\"}]', 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (158, 'NUMBER', 'FORM', 'complete_count', 'complete_count', '完成数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (159, 'FORM_GROUP_TITLE', 'FORM', 'bar_code_full_title', 'bar_code_full_title', '条码数量完整性检查', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (160, 'CHECKBOX', 'FORM', 'bar_code_full_cb', 'bar_code_full_cb', '条码数量完整性检查', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"“缺页异常”查询需无数据“\"},{\"value\":\"2\",\"label\":\"“无条码考生”查询需无数据“\"},{\"value\":\"3\",\"label\":\"“追加题卡”需和学校记录一致“\"}]', 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (161, 'FORM_GROUP_TITLE', 'FORM', 'missing_list_title', 'missing_list_title', '缺考名单比对', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (162, 'CHECKBOX', 'FORM', 'missing_list_cb', 'missing_list_cb', '缺考名单比对', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"核对未上传条数是否与学校提供的缺考名单吻合，人工指定为缺考（考生管理>导入名单>导入缺考名单）\"}]', 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (163, 'NUMBER', 'FORM', 'missing_course_count', 'missing_course_count', '缺考科次数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (164, 'FORM_GROUP_TITLE', 'FORM', 'paper_form_upload_title', 'paper_form_upload_title', '纸质表单上传', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (165, 'RADIO', 'FORM', 'school_sign_radio', 'school_sign_radio', '学校是否确认签字', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"YES\",\"label\":\"是\"},{\"value\":\"NO\",\"label\":\"否\"}]', 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (166, 'FILE', 'FORM', 'paper_form_photos', 'paper_form_photos', '上传关键节点检查表(纸质)拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (167, 'FORM_GROUP_TITLE', 'FORM', 'mark_param_check_title', 'mark_param_check_title', '评卷参数核对', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (168, 'CHECKBOX', 'FORM', 'mark_param_check_cb', 'mark_param_check_cb', '评卷参数核对', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、核查每个科目的满分是否正确(100分、150分、300分)\"},{\"value\":\"2\",\"label\":\"2、检查所有科目结构和分组是否全部导入。（评卷管理>评卷进度查看分组状态）\"}]', 'CLOUD_MARK_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (169, 'FORM_GROUP_TITLE', 'FORM', 'paper_form_upload_title', 'paper_form_upload_title', '纸质表单上传', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (170, 'FILE', 'FORM', 'paper_form_photos', 'paper_form_photos', '上传关键节点检查表(纸质)拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (171, 'FORM_GROUP_TITLE', 'FORM', 'score_check_title', 'score_check_title', '分数复核', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (172, 'CHECKBOX', 'FORM', 'score_check_cb', 'score_check_cb', '分数复核', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"是否建议学校复核小题0分，大题0分，全卷0分的情况\"}]', 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (173, 'RADIO', 'FORM', 'school_score_check_radio', 'school_score_check_radio', '学校是否选择复核', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"YES\",\"label\":\"复核(推荐)\"},{\"value\":\"NO\",\"label\":\"不复核\",\"remark\":[{\"content\":\"若选择该项，则需要学校签字确认，签字随关键节点检查表上传\",\"color\":\"red\"}]}]', 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (175, 'FORM_GROUP_TITLE', 'FORM', 'paper_form_upload_title', 'paper_form_upload_title', '纸质表单上传', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (176, 'FILE', 'FORM', 'paper_form_photos', 'paper_form_photos', '上传关键节点检查表(纸质)拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (177, 'FORM_GROUP_TITLE', 'FORM', 'mark_progress_title', 'mark_progress_title', '评卷进度核查', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (178, 'CHECKBOX', 'FORM', 'mark_progress_cb', 'mark_progress_cb', '评卷进度核查', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、核对评卷进度是否为100%\"},{\"value\":\"2\",\"label\":\"2、核对未评数量是否为0\"},{\"value\":\"3\",\"label\":\"3、问题卷是否处理完成\"},{\"value\":\"4\",\"label\":\"4、所有科目状态点击“结束”\"}]', 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (179, 'FORM_GROUP_TITLE', 'FORM', 'cross_check_title', 'cross_check_title', '交叉比对', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (180, 'CHECKBOX', 'FORM', 'cross_check_cb', 'cross_check_cb', '交叉比对', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"是否建议学校进行同一考生多科目分数交叉比对\"}]', 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (181, 'FORM_GROUP_TITLE', 'FORM', 'mark_submit_title', 'mark_submit_title', '成绩提交核查', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (182, 'CHECKBOX', 'FORM', 'mark_submit_cb', 'mark_submit_cb', '成绩提交核查', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、核对导出成绩表数据条数是否与考生表数量一致，不能带有“未导出全量数据”字样\"},{\"value\":\"2\",\"label\":\"2、核对导出图片工具是否正常运行\"}]', 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (183, 'NUMBER', 'FORM', 'export_score_count', 'export_score_count', '导出成绩数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (184, 'FILE', 'FORM', 'upload_check_report_photos', 'upload_check_report_photos', '上传验收报告(纸质)拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (185, 'ONLY_TITLE', 'FORM', 'device_in_out_label', 'device_in_out_label', '设备入库', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 11, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (186, 'DATE', 'FORM', 'device_in_time', 'device_in_time', '设备入库时间', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 11, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (187, 'DEVICE_IN_TABLE', 'FORM', 'device_in_table', 'device_in_table', '设备入库表格', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 11, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (188, 'FILE', 'FORM', 'upload_device_version_photos', 'upload_device_version_photos', '上传扫描仪固件版本号拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` VALUES (189, 'DATE', 'FORM', 'service_finish_plan_begin_date', 'service_finish_plan_begin_date', '现场服务完成进场计划时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 1, NULL, NULL, 1, 9);
INSERT INTO `t_d_form_widget` VALUES (190, 'DATE', 'FORM', 'service_finish_plan_begin_date', 'service_finish_plan_begin_date', '现场服务完成进场计划时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 1, NULL, NULL, 1, 11);

INSERT INTO `t_d_table_prop` VALUES (1, 36, 1, 'college', 'college', '学院/分（子）机构', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 37, NULL, 1, 1);
INSERT INTO `t_d_table_prop` VALUES (2, 36, 2, 'name', 'name', '姓名', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 38, NULL, 1, 1);
INSERT INTO `t_d_table_prop` VALUES (3, 36, 3, 'job', 'job', '职务', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 39, NULL, 1, 1);
INSERT INTO `t_d_table_prop` VALUES (4, 36, 4, 'mobileNumber', 'mobileNumber', '电话', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 40, NULL, 1, 1);
INSERT INTO `t_d_table_prop` VALUES (5, 112, 1, 'college', 'college', '学院/分（子）机构', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 113, NULL, 1, 1);
INSERT INTO `t_d_table_prop` VALUES (6, 112, 2, 'name', 'name', '姓名', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 114, NULL, 1, 1);
INSERT INTO `t_d_table_prop` VALUES (7, 112, 3, 'job', 'job', '职务', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 115, NULL, 1, 1);
INSERT INTO `t_d_table_prop` VALUES (8, 112, 4, 'mobileNumber', 'mobileNumber', '电话', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 116, NULL, 1, 1);

INSERT INTO `sys_device_brand` VALUES (1, '佳能', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_brand` VALUES (2, '东芝', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_brand` VALUES (3, '爱普生', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_brand` VALUES (4, '华高', NULL, NULL, NULL, NULL);

INSERT INTO `sys_device_model` VALUES (1, 1, '3300', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_model` VALUES (2, 2, '3400', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_model` VALUES (3, 3, 'DS-31100', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_model` VALUES (4, 3, 'DS-31200', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_model` VALUES (5, 4, '1080', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_model` VALUES (6, 4, '5100', NULL, NULL, NULL, NULL);

INSERT INTO `sys_ding_date` VALUES (1, 2023, 9, 29, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (2, 2023, 9, 30, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (3, 2023, 10, 1, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (4, 2023, 10, 2, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (5, 2023, 10, 3, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (6, 2023, 10, 4, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (7, 2023, 10, 5, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (8, 2023, 10, 6, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (9, 2023, 10, 7, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (10, 2023, 10, 8, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (11, 2023, 12, 30, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (12, 2023, 12, 30, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (13, 2024, 1, 1, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (14, 2024, 2, 4, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (15, 2024, 2, 10, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (16, 2024, 2, 11, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (17, 2024, 2, 12, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (18, 2024, 2, 13, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (19, 2024, 2, 14, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (20, 2024, 2, 15, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (21, 2024, 2, 16, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (22, 2024, 2, 17, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (23, 2024, 2, 18, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (24, 2024, 4, 4, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (25, 2024, 4, 5, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (26, 2024, 4, 6, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (27, 2024, 4, 7, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (28, 2024, 5, 1, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (29, 2024, 5, 2, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (30, 2024, 5, 3, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (31, 2024, 5, 4, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (32, 2024, 5, 5, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (33, 2024, 5, 11, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (34, 2024, 6, 8, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (35, 2024, 6, 9, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (36, 2024, 6, 10, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (37, 2024, 9, 14, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (38, 2024, 9, 29, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (39, 2024, 9, 15, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (40, 2024, 9, 16, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (41, 2024, 9, 17, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (42, 2024, 10, 12, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (43, 2024, 10, 1, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (44, 2024, 10, 2, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (45, 2024, 10, 3, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (46, 2024, 10, 4, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (47, 2024, 10, 5, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (48, 2024, 10, 7, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (49, 2024, 4, 28, 'WEEKDAY');

INSERT INTO `t_f_custom_flow_allocation` VALUES (1, 'PROJECT_EXCHANGE_FLOW', NULL, NULL, '{\"approveConfig\":[{\"setup\":2,\"roleTypes\":[{\"role\":\"QA\"}]}]}', NULL);
INSERT INTO `t_f_custom_flow_allocation` VALUES (2, 'QUALITY_PROBLEM_FLOW', NULL, NULL, '{\"approveConfig\":[{\"setup\":2,\"roleTypes\":[{\"role\":\"QA\"}]}]}', NULL);

INSERT INTO `sys_config` VALUES (1, NULL, 'sys.user.initPassword', '初始密码', '123456', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (2, NULL, 'sys.warning.days', '预警天数', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (3, NULL, 'sys.code.enable', '是否启用短信验证码', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (4, NULL, 'sys.message.enable', '是否启用短信消息提示', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (5, NULL, 'sys.message.resendCount', '失败短信重试次数', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (6, NULL, 'attachment.type', '附件类型', '[.xlsx,.xls,.doc,.docx,.pdf,.jpg,.jpeg,.png,.html,.zip,.mp3,.wav,.dll,.exe,.ftl,.bpmn,.xml]', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (7, NULL, 'attachment.length', '附件名称长度', '100', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (8, NULL, 'attachment.size', '附件大小 ', '200', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (9, NULL, 'thread.pool.core.size', '自定义线程池大小', '200', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (10, NULL, 'custom.thread.pool.core.size', '是否自定义线程池大小', 'true', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (11, NULL, 'session.active', 'session会话时长', '6h', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (12, NULL, 'sms.normal.code', '万能短信验证码', 'qmth', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (13, NULL, 'code.expired.time', '短信验证码有效时间', '2', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (14, NULL, 'code.send.interval', '短信验证码发送间隔', '60', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (15, NULL, 'sys.txt.charset', 'txt文件编码', 'UTF-8', NULL, 1, 1, 1, NULL, NULL, 1684994975313);
INSERT INTO `sys_config` VALUES (16, NULL, 'admin.logo.url', '超级管理员logo', 'http://qmth-test.oss-cn-shenzhen.aliyuncs.com/frontend/wx_logo.png', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (17, NULL, 'sms.sign.name', '短信签名', '启明泰和', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (18, NULL, 'sms.tpl.code', '登录短信验证码', 'SMS_147416565', NULL, 1, 2, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (19, NULL, 'flow.code.length', '流程编码长度', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (20, NULL, 'flow.job.before.time', '流程提前提醒任务时间（单位：分）', '720', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (21, NULL, 'flow.job.after.time', '流程超时提醒任务时间（单位：分）', '1440', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (22, NULL, 'notice.job.after.time', '公告超时提醒任务时间（单位：分）', '30', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (23, NULL, 'remind.job.poll.time', '任务提醒轮询时间', '0 0/5 * * * ?', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (24, NULL, 'remind.job.db.limit', '任务提醒查询条数', 'limit 0,100000', NULL, 1, 2, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (25, NULL, 'sms.notice.code', '系统通知公告', 'SMS_462685325', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (26, NULL, 'sms.sop.before.remind.code', '教务处/研究生SOP提前提醒通知', 'SMS_462805277', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (27, NULL, 'sms.sop.done.code', '教务处/研究生SOP节点负责人通知', 'SMS_462730333', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (28, NULL, 'sms.flow.project.exchange.finish.code', '系统计划变更审核通知', 'SMS_462680680', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (29, NULL, 'sms.flow.ding.exception.finish.code', '异常审核通知', 'SMS_462700329', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (30, NULL, 'sms.sop.after.remind.code', '延期提醒通知', 'SMS_462700328', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (31, NULL, 'sms.violation.remind.code', '违规提醒通知', 'SMS_462805303', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (32, NULL, 'baidu.face.compare.url', '百度人脸识别人脸比对接口', '/face-api/v3/face/match', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (33, NULL, 'baidu.face.compare.score', '百度人脸识别人脸比对阈值', '70', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (34, NULL, 'fxxk.app.auth.url', '纷享销客应用级授权接口', 'https://open.fxiaoke.com/cgi/corpAccessToken/get/V2', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (35, NULL, 'fxxk.user.by.mobile.url', '纷享销客根据手机号查询员工接口', 'https://open.fxiaoke.com/cgi/user/getByMobile', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (36, NULL, 'fxxk.custom.query.url', '纷享销客查询自定义对象接口', 'https://open.fxiaoke.com/cgi/crm/custom/v2/data/query', NULL, 1, 1, 1, NULL, NULL, NULL);
